/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/** Ce main permet, suivant le mode, de :
 * \li tester le module GEST hors-ligne, 
 * \li d'évaluer le module en comparant ses résultats avec une vérité de terrain construite préalablement, 
 * \li d'apprendre un modèle de couleur, 
 * \li d'étiqueter une séquence d'images (permettant ensuite d'évaluer le module).
 * 
 * \author BURGER Brice
 */

#include <cv.h>
#include <highgui.h>
#include <iostream>
#include <unistd.h>

#define GLOBALE//les variables globales sont déclarée dans params.h
#include "gestParams.h"
#include "videoimgseq.h"
#include "traqueurDeDeuxMains.h"
#include "mesureCouleur.h"
//...#include "mesureDisparite.h"


videoimgseq* vid;
videoimgseq* vidd;
Traqueur2* traqueur;
int init = 0;
int numimg;
int compteur;
int codeCouleur_CV;//code couleur d'opencv
IplImage *imgTg, *imgTd;//images de travail
IplImage *imgg, *imgd;//images pour l'affichage (ou de travail s'il n'y a pas de changement d'espace de couleur)
int* pfState;//particule renvoyee par le traqueur
clock_t start, end;
char kk;
bool stopPause = false;
ETAT_GEST ETAT;

//...MesureDisparite* disparite;
//...int** pts3D;

static bool evenement;
static int X,Y;
static char cheminTravail[256];
static int scoreT, scoreMG, scoreMD;
static int scoreFLT, scoreFLMG, scoreFLMD;//faux label
static int FRp,FRl;//false rate position et label : si 1 cible est fausse, l'ensemble est faux
static int margeT, margeM;
static FILE* fichier;
static bool balanceDesBlancs;

/* autres fonctions */
void apprentissage_couleur(char** argv);				//mode 0
void etiquetage_sequence(char** argv);					//mode 1
void comp_etiquetage(int* vectEtat, ETAT_GEST etat, int numImage);	//mode 5

void on_mouse( int event, int x, int y, int flags, void* unused );

void faitBalanceDesBlancs(IplImage *imgg, IplImage *imgd, int couleurDeRefBalanceDesBlancs);

int*** MesureCouleur::bins_cub = NULL;//on initialise ici ce tableau (static) car il est très gros (~60Mo), on ne veux donc le créer qu'une seule fois


/**
  ./traqueur 4 <dirname> <fileprefix> <extensiong> <start number> <end number> <number of digits> <extensiond>
  ( ./traqueur 5 <dirname> <fileprefix> <extensiong> <start number> <end number> <number of digits> <extensiond> pour comparer avec l'étiquetage )
  ( ./traqueur 0 <dirname> <fileprefix> <extensiong> <start number> <end number> <number of digits> <extensiond> <fileprefixsortie> pour l'apprentissage d'un modèle de couleur )
  ( ./traqueur 1 <dirname> <fileprefix> <extensiong> <start number> <end number> <number of digits> <extensiond> pour l'étiquetage d'une séquence d'images )
  
  ex: src/LeMain/traqueur 4 $DONNEES/GEST/données_base/TESTS/1/ image g.png 0041 0189 4 d.png
  ex: src/LeMain/traqueur 4 $DONNEES/GEST/données_base/imagesSalleMocap/1/ image. g.jpg 0000 1270 4 d.jpg
*/
int main(int argc, char** argv) {
	
	balanceDesBlancs = false;
	espaceCoul = baseLab;
	
	/* les marges sont à adapter à la distance (au cas par cas par séquence) */
	margeT = 15;//pour l'évaluation
	margeM = 12;//pour l'évaluation
	/* 0) gestion des différents modes */
	if (argc < 9) { printf("ERREUR : il manque certainement des arguments, voir src/LeMain/main.cpp\n"); exit(0); }
	if (atoi(argv[1])==0) { apprentissage_couleur(argv); return 1; }
	if (atoi(argv[1])==1) { etiquetage_sequence(argv); return 1; }
	
	
	/* 1) initialisation */
	/* ouverture de la séquence d'images */
	numimg = atoi(argv[5]);
	vid = new videoimgseq(argv[2],argv[3],argv[4],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image gauche
	vidd = new videoimgseq(argv[2],argv[3],argv[8],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image droite
	
	if(vid->init() && vidd->init()) cout << "Video ouverte\n";
	else cout << "Erreur lors de l'initialisation de la video\n";
	
	/* réglage des paramètres */
	BASE = GLOBAL_PATH_TO_GEST + string("/code/");//chemin de base du code de suivi
	w=256;
	h=192;
	distMaxEval = 10;
	seuilbas = 170;
	nbpart = 100;
	surfaceminblob = 20;
	CalculeImgDist = false;
	
	if (espaceCoul==baseRGB) nbfermeture = 2;
	else nbfermeture = 0;
	
	/* paramètres intrinsèques :*/
	
	/* 1) les vieux, mais valables pour toute la base donnée ancienne :
	gauche : u0=124.448 , v0=99.435 , au=247.521 , av=248.607
	droite : u0=139.260 , v0=99.680 , au=247.299 , av=248.567*/
	
	Mdg[3]=270;
	Mgd[3]=-270;
	setintrinsicsLeft(124.448 , 99.435 , 247.521 , 248.607, 240, 320);
	setintrinsicsRight(139.260 , 99.680 , 247.299 , 248.567, 240, 320);
	
	
	/* 2) caméras COMMROB :
	gauche : u0=324.275 , v0=369.599 , au=942.034 , av=856.654 (pour 1024x768)
	droite : u0=324.275 , v0=369.599 , au=942.034 , av=856.654 (pour 1024x768)
	+9,4203435328477701e+02	+0,0000000000000000e+00	+3,2427544085346040e+02
	+0,0000000000000000e+00	+8,5665494837143626e+02	+3,6959958817578666e+02
	+0,0000000000000000e+00	+0,0000000000000000e+00	+1,0000000000000000e+00
	GEST : taille de l'image importée de camera : 320 x 240
	*/
	/*
	Mdg[3]=305;
	Mgd[3]=-305;
	setintrinsicsLeft(324.275, 369.599, 942.034, 856.654, 768, 1024);
	setintrinsicsRight(324.275, 369.599, 942.034, 856.654, 768, 1024);
	*/
	
	/* 3) caméras Inbot (LAAS) :*/
	/*
	Mdg[3]=400;
	Mgd[3]=-400;
	setintrinsicsLeft(452.691, 356.421, 865.008, 933.840, 768, 1024);
	setintrinsicsRight(452.691, 356.421, 865.008, 933.840, 768, 1024);
	*/
	
	//...disparite = new MesureDisparite();
	//...pts3D = new int*[BLOBS_PEAU_MAX];
	//...for (int i=0;i<BLOBS_PEAU_MAX;i++) pts3D[i] = new int[3];
	
	imgg=cvCreateImage(cvSize(w,h),8,3);
	imgd=cvCreateImage(cvSize(w,h),8,3);
	
	switch (espaceCoul) {
		case baseRGB:
		case baseBGR:
		case basergbN:
			codeCouleur_CV = 0;//=CV_BGR2BGRA, qui ne servira jamais ici
			break;
		case baseYCrCb:
			codeCouleur_CV = CV_BGR2YCrCb;
			break;
		case baseHSV:
			codeCouleur_CV = CV_BGR2HSV;
			break;
		case baseLab:
			codeCouleur_CV = CV_BGR2Lab;
			break;
		case baseIrg:
		case baseI1I2I3:
		default:
			printf("espace de couleur (%d) non traité!!\n",espaceCoul);
			exit(0);
	}
	
	if (codeCouleur_CV) {
		imgTg=cvCreateImage(cvSize(w,h),8,3);
		imgTd=cvCreateImage(cvSize(w,h),8,3);
	} else {
		imgTg=NULL;
		imgTd=NULL;
	}
/*
//...pour afficher les 3 plans de l'image de travail de gauche
IplImage* imgP1 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage* imgP2 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage* imgP3 = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage** images = new IplImage*[3];
images[0] = imgP1;
images[1] = imgP2;
images[2] = imgP3;
IplImage** imageNG0 = new IplImage*[2];
imageNG0[0] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
imageNG0[1] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage** imageNG1 = new IplImage*[2];
imageNG1[0] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
imageNG1[1] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage** imageNG2 = new IplImage*[2];
imageNG2[0] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
imageNG2[1] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,1);
IplImage** imagett = new IplImage*[3];
imagett[0] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
imagett[1] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
imagett[2] = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,3);
*/
	/* création du traqueur */
	traqueur = new Traqueur2(imgg,imgd);
	
	traqueur->avec_interface(true);
	traqueur->affiche_image_sup(true,true);
	
	traqueur->affiche_proba_peau(true);
//	traqueur->affiche_mvt(true);
//	traqueur->affiche_dist(true);
	
//	traqueur->affiche_particules(true);

//	traqueur->sauvegarde(true,false,"/home/bburger/truc/");
	
	
	/* initialisation du mode comparaison avec étiquetage */
	int couleurDeRefBalanceDesBlancs = 2;
	if (atoi(argv[1])==5) {
		sprintf(cheminTravail,"%s",argv[2]);
		comp_etiquetage(NULL, ETAT, -1);
	}
	
	start = clock();
	
	int tetedetecte = 0;
	/* 2) boucle principale */
	while(numimg<=atoi(argv[6])) {//tant qu'il reste des images
		//cvWaitKey(0);
		//init = 0;//init en boucle
		if (!init) {
			printf ("INIT\n");
			/******************/	
			/* initialisation */
			/******************/
			
			cvResize(vid->next(), imgg, CV_INTER_LINEAR);//On redimensionne
			cvResize(vidd->next(), imgd, CV_INTER_LINEAR);//On redimensionne
			
			if (balanceDesBlancs) {
				faitBalanceDesBlancs(imgg, imgd, couleurDeRefBalanceDesBlancs);
				faitBalanceDesBlancs(imgTg, imgTd, couleurDeRefBalanceDesBlancs);
			}
			
			if (codeCouleur_CV) {
				cvCvtColor( imgg, imgTg, codeCouleur_CV );
				cvCvtColor( imgd, imgTd, codeCouleur_CV );
			}
			
/*
//...pour afficher les 3 plans de l'image de travail de gauche
cvSplit(imgTg,imgP1,imgP2,imgP3,NULL);
cvCopyImage(imgP1, imageNG0[0]);
cvCopyImage(imgP1, imageNG0[1]);
cvMerge(imgP1,imageNG0[0],imageNG0[1],NULL,imagett[0]);
cvCopyImage(imgP2, imageNG1[0]);
cvCopyImage(imgP2, imageNG1[1]);
cvMerge(imgP2,imageNG1[0],imageNG1[1],NULL,imagett[1]);
cvCopyImage(imgP3, imageNG2[0]);
cvCopyImage(imgP3, imageNG2[1]);
cvMerge(imgP3,imageNG2[0],imageNG2[1],NULL,imagett[2]);
traqueur->affiche (imagett[0], imagett[1], imagett[2]);
getchar();//pour faire une pause
*/
			
			if (codeCouleur_CV) ETAT = traqueur->INIT(imgg,imgd , imgTg,imgTd);
			else ETAT = traqueur->INIT(imgg,imgd , imgg,imgd);
			
			if (ETAT==0) {//la traque n'a pas démarré
				printf("GEST : INITIALISATION IMPOSSIBLE, on recommence\n");
				init = 0;
			} else {
				compteur = 0;
				init = 1;
			}
			
			cout << "ETAT = " << ETAT << endl;
			pfState = traqueur->RECUP();
			for (int i=0;i<traqueur->GETNBVAR();i++) cout << "pfstate[" << i << "] = " << pfState[i] << " " << endl;
			
			/* comparaison avec l'étiquetage */
			if (atoi(argv[1])==5) comp_etiquetage(pfState, ETAT, numimg);
			
			if (stopPause) getchar();//pour faire une pause
			
			numimg++;
			
			continue;
		}

		/**********************/
		/* on avance d'un pas */
		/**********************/
		printf("GEST : pas numero %d\n",compteur);
		
		cvResize(vid->next(), imgg, CV_INTER_LINEAR);//On redimensionne
		cvResize(vidd->next(), imgd, CV_INTER_LINEAR);//On redimensionne
		
		if (balanceDesBlancs) {
			faitBalanceDesBlancs(imgg, imgd, couleurDeRefBalanceDesBlancs);
			faitBalanceDesBlancs(imgTg, imgTd, couleurDeRefBalanceDesBlancs);
		}
		
		if (codeCouleur_CV) {
			cvCvtColor( imgg, imgTg, codeCouleur_CV );
			cvCvtColor( imgd, imgTd, codeCouleur_CV );
		}
/*
//...pour afficher les 3 plans de l'image de travail de gauche
cvSplit(imgTg,imgP1,imgP2,imgP3,NULL);
cvCopyImage(imgP1, imageNG0[0]);
cvCopyImage(imgP1, imageNG0[1]);
cvMerge(imgP1,imageNG0[0],imageNG0[1],NULL,imagett[0]);
cvCopyImage(imgP2, imageNG1[0]);
cvCopyImage(imgP2, imageNG1[1]);
cvMerge(imgP2,imageNG1[0],imageNG1[1],NULL,imagett[1]);
cvCopyImage(imgP3, imageNG2[0]);
cvCopyImage(imgP3, imageNG2[1]);
cvMerge(imgP3,imageNG2[0],imageNG2[1],NULL,imagett[2]);
traqueur->affiche (imagett[0], imagett[1], imagett[2]);
getchar();//pour faire une pause
*/
		
		if (codeCouleur_CV) ETAT = traqueur->PAS(imgg,imgd , imgTg,imgTd);
		else ETAT = traqueur->PAS(imgg,imgd , imgg,imgd);
		
		if (ETAT==RIEN) {//probleme -> il faut reinitialiser
			printf("GEST : plus rien n'est traque, on recommence\n");
			init = 0;
		} else compteur++;
		
		cout << "ETAT = " << ETAT << endl;
		pfState = traqueur->RECUP();
		for (int i=0;i<traqueur->GETNBVAR();i++) cout << "pfstate[" << i << "] = " << pfState[i] << " " << endl;
		if (traqueur->TETEDETECTEE()) { cout << "tête détectée" << endl; tetedetecte++; }
		
		//...if (ETAT==TOUT) disparite->CalcDispariteDemiDroite(pfState, &(pfState[3]), imgTg, imgTd, pts3D);
		//...traqueur->affiche(imgTg, imgTd, imgg);
		
		/* comparaison avec l'étiquetage */
		if (atoi(argv[1])==5) comp_etiquetage(pfState, ETAT, numimg);
		
		if (stopPause) getchar();//pour faire une pause
		
		numimg++;
	}
	
	/* 3) finalisation */
	/* on relache de la mémoire */
	cvReleaseImage(&imgg);
	cvReleaseImage(&imgd);
	
	if (codeCouleur_CV) {
		cvReleaseImage(&imgTg);
		cvReleaseImage(&imgTd);
	}

	/* affichage du temps d'exécution */
	end = clock();
	cout << endl << "temps d'execution du code (sans l'affichage) : " << (double)(end-start)/(double)1000 << " ms\n";
	cout << "soit par image : " << ((double)(end-start)/(double)1000)/(double)(atoi(argv[6])-atoi(argv[5])) << " ms\n";
	printf("nb detect : %d\n",tetedetecte);
	
	/* comparaison avec l'étiquetage */
	if (atoi(argv[1])==5) {
		comp_etiquetage(NULL, ETAT, -2);
		//scoreCible contient le score total (c'est à dire avec les faux labels)
		printf("\nFRp (taux d'erreur en position) par cible : T %1.2f | MG %1.2f | MD %1.2f\n",1-float(scoreT+scoreFLT)/float(atoi(argv[6])-atoi(argv[5])+1),1-float(scoreMG+scoreFLMG)/float(atoi(argv[6])-atoi(argv[5])+1),1-float(scoreMD+scoreFLMD)/float(atoi(argv[6])-atoi(argv[5])+1));
		printf("FRl (taux de faux labels) par cible : T %1.2f | MG %1.2f | MD %1.2f\n",float(scoreFLT)/float(atoi(argv[6])-atoi(argv[5])+1),float(scoreFLMG)/float(atoi(argv[6])-atoi(argv[5])+1),float(scoreFLMD)/float(atoi(argv[6])-atoi(argv[5])+1));
		printf("total : FRp = %1.2f | FRl = %1.2f\n",float(FRp)/float(atoi(argv[6])-atoi(argv[5])+1),float(FRl)/float(atoi(argv[6])-atoi(argv[5])+1));
	}
	
	return 1;
}


/** pour apprendre un modèle de couleur :
 *
 * prend en entrée une série d'images ne contenants que des pixels d'une couleur à apprendre (pseudo coté gauche) et non couleur à apprendre (pseudo coté droit)
 * s'il n'y a rien du coté droit, on apprend le modèle de couleur sans le modèle de non couleur
 */
void apprentissage_couleur(char** argv) {
	
	int taille_entree_w=1000;
	int taille_entree_h=1000;
	
	bool apnoncoul = true;//si on apprend aussi avec la non couleur
	
	char fichier[256];
	sprintf(fichier,"%s/%s",argv[2],argv[9]);
	
	imgg=cvCreateImage(cvSize(taille_entree_w,taille_entree_h),8,3);
	imgd=cvCreateImage(cvSize(taille_entree_w,taille_entree_h),8,3);
	imgTg=cvCreateImage(cvSize(taille_entree_w,taille_entree_h),8,3);
	imgTd=cvCreateImage(cvSize(taille_entree_w,taille_entree_h),8,3);
	
	float max;
	int dimhist;//dimension de l'histogramme utilisé pour la rétro-projection
	int limiteMin,limiteMax;//limites de luminosité au-delà desquelles on ne regarde pas la couleur
	int nbpixC,nbpixNC;//nb de pixels de couleur et de non couleur
	int codeCouleur_CV;
	int* tailleshist;
	float** range;
	int tailleshist3[] = {32,32,32};
	int tailleshist2[] = {32,32};
	float rangeP1[] = {0,255};
	float rangeP2[] = {0,255};
	float rangeP3[] = {0,255};
	float *range3[] = {rangeP1,rangeP2,rangeP3};
	float *range2[] = {rangeP1,rangeP2};
	nbpixC = 0;
	nbpixNC = 0;
	
	numimg = atoi(argv[5]);
	vid = new videoimgseq(argv[2],argv[3],argv[4],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image couleur à apprendre
	vidd = new videoimgseq(argv[2],argv[3],argv[8],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image couleur non couleur à apprendre
	
	if(vid->init()) cout << "Video couleur à apprendre (" << argv[2] << "/" << argv[3] << "*." << argv[4] << ") ouverte\n";
	else { cout << "Erreur lors de l'initialisation de la video couleur à apprendre\n"; return; }
	if (vidd->init()) cout << "Video couleur non couleur à apprendre (" << argv[2] << "/" << argv[3] << "*." << argv[8] << ") ouverte\n";
	else { cout << "pas d'images de couleur non couleur à apprendre\n"; apnoncoul = false; }
	
	IplImage* imgP1 = cvCreateImage(cvSize(taille_entree_w,taille_entree_h),IPL_DEPTH_8U,1);
	IplImage* imgP2 = cvCreateImage(cvSize(taille_entree_w,taille_entree_h),IPL_DEPTH_8U,1);
	IplImage* imgP3 = cvCreateImage(cvSize(taille_entree_w,taille_entree_h),IPL_DEPTH_8U,1);
	IplImage** images = new IplImage*[3];
	images[0] = imgP1;
	images[1] = imgP2;
	images[2] = imgP3;
	
	switch (espaceCoul) {
		case baseRGB:
		case baseBGR:
		case basergbN:
			dimhist = 3;
			tailleshist = tailleshist3;
			range = range3;
			codeCouleur_CV = 0;//=CV_BGR2BGRA, qui ne servira jamais ici
			break;
		case baseYCrCb:
			dimhist = 2;
			tailleshist = tailleshist2;
			range = range2;
			codeCouleur_CV = CV_BGR2YCrCb;
			limiteMin = 10;
			limiteMax = 245;
			break;
		case baseHSV:
			dimhist = 3;
			tailleshist = tailleshist3;
			range = range3;
			codeCouleur_CV = CV_BGR2HSV;
			break;
		case baseLab:
			dimhist = 2;
			tailleshist = tailleshist2;
			range = range2;
			codeCouleur_CV = CV_BGR2Lab;
			limiteMin = 5;
			limiteMax = 250;
			break;
		case baseIrg:
		case baseI1I2I3:
		default:
			printf("espace de couleur (%d) non traité!!\n",espaceCoul);
			exit(0);
	}
	CvHistogram* hist = cvCreateHist(dimhist, tailleshist, CV_HIST_ARRAY, range, 1);
	CvHistogram* histtemp = cvCreateHist(dimhist, tailleshist, CV_HIST_ARRAY, range, 1);
	CvHistogram* histcoul = cvCreateHist(dimhist, tailleshist, CV_HIST_ARRAY, range, 1);
	CvHistogram* histnoncoul = cvCreateHist(dimhist, tailleshist, CV_HIST_ARRAY, range, 1);
	cvClearHist (hist);
	cvClearHist (histtemp);
	cvClearHist (histcoul);
	cvClearHist (histnoncoul);
	
	/* calcul des histogrammes de couleur et non couleur sur l'ensemble des images en entrée */
	while(numimg<=atoi(argv[6])) {//tant qu'il reste des images
		
		cvResize(vid->next(), imgg, CV_INTER_LINEAR);//On redimensionne
		if (apnoncoul) cvResize(vidd->next(), imgd, CV_INTER_LINEAR);//On redimensionne
		
		if (codeCouleur_CV) {//on change la base de couleur de l'image
			cvCvtColor( imgg, imgTg, codeCouleur_CV );
			if (apnoncoul) cvCvtColor( imgd, imgTd, codeCouleur_CV );
			cvCopyImage(imgTg, imgg);
			if (apnoncoul) cvCopyImage(imgTd, imgd);
		}
		
		/* couleur à apprendre */
		cvSplit(imgg,imgP3,imgP2,imgP1,NULL);//imgP3,imgP2,imgP1 pour avoir RGB à partir de l'image BGR et ba pour l'image Lab, etc...
		
		if (dimhist==2) {//il faut supprimmer les cases ayant une luminosité trop forte (blanc) ou trop faible (noir) (si dimhist==2, c'est que imgP3 est la luminosité)
			cvThreshold(imgP3,imgP3,limiteMin,255,CV_THRESH_TOZERO);
			cvThreshold(imgP3,imgP3,limiteMax,255,CV_THRESH_TOZERO_INV);
			cvCalcHist(images, histcoul, 1, imgP3);//calcul de l'histogramme (on rajoute à chaque image)
			nbpixC += cvCountNonZero(imgP3);
		}
		else {
			cvCalcHist(images, histcoul, 1);//calcul de l'histogramme (on rajoute à chaque image)
			nbpixC += taille_entree_w*taille_entree_h;
		}
		
		if (apnoncoul) {
			/* non couleur à apprendre */
			cvSplit(imgd,imgP3,imgP2,imgP1,NULL);
			cvCalcHist(images, histnoncoul, 1);//calcul de l'histogramme
			nbpixNC += taille_entree_w*taille_entree_h;
		}

		numimg++;
	}
	
	printf("nb de pixels de couleur apris : %d\n",nbpixC);
	
	if (apnoncoul) {
		/* ratio = couleur / (non couleur + couleur) *///de plus non couleur doit être remis à l'échelle : * nbpixC/nbpixNC
		//marche pas?! : cvScaleAdd (histnoncoul->bins, cvScalar(nbpixC/(double)nbpixNC), histcoul->bins, hist->bins);//i.e. hist = histnoncoul * nbpixC/nbpixNC + histcoul
		cvConvertScale(histnoncoul->bins, histnoncoul->bins, nbpixC/(double)nbpixNC);
		cvAdd(histnoncoul->bins,histcoul->bins,hist->bins);
		cvDiv(histcoul->bins, hist->bins, histtemp->bins);//i.e. histtemp = histcoul / hist
	}
	else cvCopyHist (histcoul, &histtemp);
	
	cvGetMinMaxHistValue(histtemp, 0, &max, 0, 0 );
	
	cvConvertScale(histtemp->bins, hist->bins, 255*max);//on remet tout entre 0 et 255
	
	
	/* affichage de l'histogramme s'il est 2D */
	if (dimhist==2) {
		int h,s,scale=10;
		IplImage* hist_img = cvCreateImage( cvSize(tailleshist[0]*scale,tailleshist[1]*scale), 8, 3 );
		
		cvZero( hist_img );
		
		for( h = 0; h < tailleshist[0]; h++ ) {
			for( s = 0; s < tailleshist[1]; s++ ) {
				int intensity = cvRound(cvQueryHistValue_2D( hist, h, s ));
				if(intensity) cvRectangle( hist_img, cvPoint(h*scale,s*scale),cvPoint( (h+1)*scale-1, (s+1)*scale-1),CV_RGB(intensity,intensity,intensity),CV_FILLED );
			}
		}
		
		cvNamedWindow("azerty",1);
		cvMoveWindow("azerty",0,0);
		cvShowImage("azerty",hist_img);
		cvWaitKey(0);
		cvReleaseImage(&hist_img);
	}

	/* sauvegarde dans un xml à la icu */// (reprise de SkinColorModel::save(string model_filename))
	ofstream ficout;
	string basename;
	
	ficout.open(fichier,ios::out|ios::trunc);
	if(!ficout.good()) return;
	
	/* Ecriture de la base a charger */
	if (espaceCoul==baseRGB) basename = "RGB";
	else if (espaceCoul==baseRGB) basename = "BGR";
	else if (espaceCoul==baseI1I2I3) basename = "I1I2I3";
	else if (espaceCoul==baseIrg) basename = "Irg";
	else if (espaceCoul==baseLab) basename = "Lab";
	else if (espaceCoul==basergbN) basename = "rgbN";
	else if (espaceCoul==baseYCrCb) basename = "YCrCb";
	else if (espaceCoul==baseHSV) basename = "HSV";
	else basename = "";
	
	ficout << "<basename>\n";
	ficout << basename << endl;
	ficout << "</basename>\n";

	/* Ecriture du modele (histogramme) */
	ficout << "<dim>" <<endl;
	ficout << dimhist << endl;
	ficout << "</dim>" <<endl;
	
	ficout << "<size>\n";  
	for(int i=0;i<dimhist;i++) ficout << tailleshist[i] << "\t";
	ficout << endl;
	ficout << "</size>\n";
	
	ficout << "<data>\n";
	switch(dimhist) {
		case 2 :
			for(int i=0;i<tailleshist[0];i++)
				for(int j=0;j<tailleshist[1];j++)
					ficout << cvGetReal2D(hist->bins,i,j) << " ";
			break;
		case 3 :
			for(int i=0;i<tailleshist[0];i++)
				for(int j=0;j<tailleshist[1];j++)
					for(int k=0;k<tailleshist[2];k++)
						ficout << cvGetReal3D(hist->bins,i,j,k) << " ";
			break;
	}
	ficout << endl;
	ficout << "</data>\n";
	
	ficout.close();
	
	return;
}


/** pour étiqueter une séquence d'images :
 *
 * prend en entrée la série d'images à étiqueter (droite et gauche comme pour un suivi)
 * le fichier de sortie sera le fichier "etiquetage.gest" dans le même dossier que les images
 */
void etiquetage_sequence(char** argv) {
	
	FILE* fichier;
	char cheminfichier[256];
	
	printf("\n\nMODE ETIQUETAGE\n");
	
	//traqueur 1 <dirname> <fileprefix> <extensiong> <start number> <end number> <number of digits> <extensiond>
	//chemin = argv[2] ; nomImage = argv[3] %4d numimg argv[4g ou 8d]
	
	/* ouverture du fichier */
	sprintf(cheminfichier,"%s/etiquetage.gest",argv[2]);
	fichier = fopen(cheminfichier, "w");
	if (!fichier) {
		printf("impossible d'ouvrir ou de créer le fichier %s\n",cheminfichier);
		return;
	}
	fprintf(fichier,"etiquetage.gest\nformat du fichier (-1 = cible non suivie):\nimage 'côté' ('nom')\n'cible':'attribut1=...,attribut2=...'\n");
	
	numimg = atoi(argv[5]);
	vid = new videoimgseq(argv[2],argv[3],argv[4],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image gauche
	vidd = new videoimgseq(argv[2],argv[3],argv[8],atoi(argv[5]),atoi(argv[6]),atoi(argv[7]));//image droite
	
	if(vid->init() && vidd->init()) cout << "Video ouverte\n";
	else {
		cout << "Erreur lors de l'initialisation de la video\n";
		return;
	}
	
	w=256;
	h=192;
	imgg=cvCreateImage(cvSize(w,h),8,3);
	imgd=cvCreateImage(cvSize(w,h),8,3);
	
	/* création de la fenètre */
	cvNamedWindow("etiquetage",1);
	cvMoveWindow("etiquetage",0,0);
	cvSetMouseCallback("etiquetage",on_mouse);//pour qu'opencv gère les évènement souris
	
	printf("\nc'est parti!!\n\n");
	fprintf(fichier,"\ndébut de la séquence\n");
	
	/* boucle principale */
	while(numimg<=atoi(argv[6])) {//tant qu'il reste des images
		
		/* affichage de l'image suivante */
		cvResize(vid->next(), imgg, CV_INTER_LINEAR);//On redimensionne
		cvResize(vidd->next(), imgd, CV_INTER_LINEAR);//On redimensionne
		
		/* 1) image gauche */
		cvShowImage("etiquetage",imgg);
		sprintf(cheminfichier,"%s%04d.%s",argv[3],numimg,argv[4]);
		printf("image gauche (%s)\n",cheminfichier);
		fprintf(fichier,"image gauche (%s)\n",cheminfichier);
		/* tête */
		evenement = false;
		printf("étiqueter la tête\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgg, cvPoint(X,Y), margeT, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgg);
			fprintf(fichier,"T:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"T:-1\n");
		/* MG */
		evenement = false;
		printf("étiqueter la main gauche\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgg, cvPoint(X,Y), margeM, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgg);
			fprintf(fichier,"MG:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"MG:-1\n");
		/* MD */
		evenement = false;
		printf("étiqueter la main droite\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgg, cvPoint(X,Y), margeM, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgg);
			fprintf(fichier,"MD:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"MD:-1\n");
		
		/* 2) image droite */
		cvShowImage("etiquetage",imgd);
		sprintf(cheminfichier,"%s%04d.%s",argv[3],numimg,argv[8]);
		printf("image droite (%s)\n",cheminfichier);
		fprintf(fichier,"image droite (%s)\n",cheminfichier);
		/* tête */
		evenement = false;
		printf("étiqueter la tête\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgd, cvPoint(X,Y), margeT, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgd);
			fprintf(fichier,"T:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"T:-1\n");
		/* MG */
		evenement = false;
		printf("étiqueter la main gauche\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgd, cvPoint(X,Y), margeM, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgd);
			fprintf(fichier,"MG:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"MG:-1\n");
		/* MD */
		evenement = false;
		printf("étiqueter la main droite\n");
		while (!evenement) cvWaitKey(100);
		if (X!=-1) {
			cvCircle( imgd, cvPoint(X,Y), margeM, CV_RGB(255,0,0), 1 );
			cvShowImage("etiquetage",imgd);
			fprintf(fichier,"MD:x=%d,y=%d\n",X,Y);
		} else fprintf(fichier,"MD:-1\n");
		
		numimg++;
	}
	
	fprintf(fichier,"fin de la séquence");
	printf("\nFIN\n");
	
	/* on relache de la mémoire */
	cvReleaseImage(&imgg);
	cvReleaseImage(&imgd);
	
	cvDestroyWindow("etiquetage");
	
	fclose(fichier);

/* Inital rectangle must be selected before continuing *
while(!track_object) {
	cvCopyImage(imgIn,imgTmp);	    
	if(select_region) cvRectangle(imgTmp,ptULinit,ptLRinit,CV_RGB(255,0,0),2); 
	cvShowImage("fenetre",imgTmp);
	cvWaitKey(100);
}
*/

}


/**********************************************************************/
/*************************** sous fonctions ***************************/
/**********************************************************************/

/** fonction qui gère les évènements souris (générés par opencv) */
void on_mouse( int event, int x, int y, int flags, void* unused ) {
	
/*	if( select_region ) {
		if(x<origin.x) {
			ptULinit.x = x;
			ptLRinit.x = origin.x;
		} else {
			ptLRinit.x = x;
			ptULinit.x = origin.x;
		}
	
		if(y<origin.y) {
			ptULinit.y = y;
			ptLRinit.y = origin.y;
		} else {
			ptLRinit.y = y;
			ptULinit.y = origin.y;
		}

		cvRectangle(imgTmp,ptULinit,ptLRinit,CV_RGB(255,0,0),2);
		//  printf("%d %d %d %d", ptULinit.x, ptLRinit.x, ptULinit.y, ptLRinit.y);
	}*/
	
	switch (event) {
	
		case CV_EVENT_LBUTTONDOWN:
			evenement = true;
			X=x;
			Y=y;
			break;
/*		case CV_EVENT_LBUTTONUP:
			select_region = 0;
			track_object = 1;
			break;*/
		case CV_EVENT_RBUTTONDOWN:
			evenement = true;
			X=-1;
			Y=-1;
			break;
	}
}

/** fonction qui compare le résultat de gest à celui de l'étiquetage :
 * 
 * vectEtat, etat : résultats du traqueur
 * numImage : numéro de l'image traitée (-1 = initialisation , -2 = finalisation)
 */
void comp_etiquetage(int* vectEtat, ETAT_GEST etat, int numImage) {
	
	char cheminfichier[256];
	char *chartemp;
	CvPoint point2dgaucheT, point2ddroiteT;
	CvPoint point2dgaucheMG, point2ddroiteMG;
	CvPoint point2dgaucheMD, point2ddroiteMD;
	projection *proj;
	
	bool OKT=false;
	bool OKMG=false;
	bool OKMD=false;
	bool FLT=false;//faux label
	bool FLMG=false;
	bool FLMD=false;
	
	/* initialisation */
	if (numImage==-1) {
		/* ouverture du fichier */
		sprintf(cheminfichier,"%s/etiquetage.gest",cheminTravail);
		fichier = fopen(cheminfichier, "r");
		if (!fichier) {
			printf("impossible d'ouvrir le fichier %s\n",cheminfichier);
			return;
		}
		/* recherche du début de la séquence */
		while ( strcmp(cheminfichier,"début de la séquence\n") ) {//tant qu'on n'est pas au début
			if (!fgets(cheminfichier, 256, fichier)) {//on sort la ligne du fichier
				printf("ERREUR : fin du fichier d'étiquetage\n");
				return;
			}
			printf("%s",cheminfichier);
		}
		scoreT=0;
		scoreMG=0;
		scoreMD=0;
		scoreFLT=0;
		scoreFLMG=0;
		scoreFLMD=0;
		proj = new projection();
		return;
	}
	
	/* finalisation */
	if (numImage==-2) {
		printf("\nscore final : T %d | MG %d | MD %d\n",scoreT,scoreMG,scoreMD);
		printf("nb faux labels : T %d | MG %d | MD %d\n",scoreFLT,scoreFLMG,scoreFLMD);
		fclose (fichier);
		delete(proj);
		return;
	}
	
	
	/* 0) projection du résultat */
	point2dgaucheT.x = -1;
	point2dgaucheT.y = -1;
	point2ddroiteT.x = -1;
	point2ddroiteT.y = -1;
	point2dgaucheMG.x = -1;
	point2dgaucheMG.y = -1;
	point2ddroiteMG.x = -1;
	point2ddroiteMG.y = -1;
	point2dgaucheMD.x = -1;
	point2dgaucheMD.y = -1;
	point2ddroiteMD.x = -1;
	point2ddroiteMD.y = -1;
	switch (etat) {
		case RIEN:
			break;
		case TOUT:
			proj->projettePoint (vectEtat, &point2dgaucheT, &point2ddroiteT);		//T
			proj->projettePoint (&vectEtat[3], &point2dgaucheMG, &point2ddroiteMG);		//MG
			proj->projettePoint (&vectEtat[12], &point2dgaucheMD, &point2ddroiteMD);	//MD
			break;
		case PASmAINgAUCHE:
			proj->projettePoint (vectEtat, &point2dgaucheT, &point2ddroiteT);		//T
			proj->projettePoint (&vectEtat[12], &point2dgaucheMD, &point2ddroiteMD);	//MD
			break;
		case PASmAINdROITE:
			proj->projettePoint (vectEtat, &point2dgaucheT, &point2ddroiteT);		//T
			proj->projettePoint (&vectEtat[3], &point2dgaucheMG, &point2ddroiteMG);		//MG
			break;
		case AUCUNEmAIN:
			proj->projettePoint (vectEtat, &point2dgaucheT, &point2ddroiteT);		//T
			break;
		case PAStETE:
			proj->projettePoint (&vectEtat[3], &point2dgaucheMG, &point2ddroiteMG);		//MG
			proj->projettePoint (&vectEtat[12], &point2dgaucheMD, &point2ddroiteMD);	//MD
			break;
		case PAStETEnImAINgAUCHE:
			proj->projettePoint (&vectEtat[12], &point2dgaucheMD, &point2ddroiteMD);	//MD
			break;
		case PAStETEnImAINdROITE:
			proj->projettePoint (&vectEtat[3], &point2dgaucheMG, &point2ddroiteMG);		//MG
			break;
	}
	
	
	/* 1) comparaison image gauche */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	/* tête */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "T:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2dgaucheT.x>X-margeT)&&(point2dgaucheT.x<X+margeT)&&(point2dgaucheT.y>Y-margeT)&&(point2dgaucheT.y<Y+margeT)) { printf("OK\n"); OKT = true; }
		if ((point2dgaucheMG.x>X-margeM)&&(point2dgaucheMG.x<X+margeM)&&(point2dgaucheMG.y>Y-margeM)&&(point2dgaucheMG.y<Y+margeM)) { printf("Faux Label MG\n"); FLMG = true; }
		if ((point2dgaucheMD.x>X-margeM)&&(point2dgaucheMD.x<X+margeM)&&(point2dgaucheMD.y>Y-margeM)&&(point2dgaucheMD.y<Y+margeM)) { printf("Faux Label MD\n"); FLMD = true; }
	} else {
		if (point2dgaucheT.x==-1) { printf("OK\n"); OKT = true; }
		else printf("NON OK\n");
	}
	/* MG */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "MG:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2dgaucheT.x>X-margeT)&&(point2dgaucheT.x<X+margeT)&&(point2dgaucheT.y>Y-margeT)&&(point2dgaucheT.y<Y+margeT)) { printf("Faux Label T\n"); FLT = true; }
		if ((point2dgaucheMG.x>X-margeM)&&(point2dgaucheMG.x<X+margeM)&&(point2dgaucheMG.y>Y-margeM)&&(point2dgaucheMG.y<Y+margeM)) { printf("OK\n"); OKMG = true; }
		if ((point2dgaucheMD.x>X-margeM)&&(point2dgaucheMD.x<X+margeM)&&(point2dgaucheMD.y>Y-margeM)&&(point2dgaucheMD.y<Y+margeM)) { printf("Faux Label MD\n"); FLMD = true; }
	} else {
		if (point2dgaucheMG.x==-1) { printf("OK\n"); OKMG = true; }
		else printf("NON OK\n");
	}
	/* MD */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "MD:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2dgaucheT.x>X-margeT)&&(point2dgaucheT.x<X+margeT)&&(point2dgaucheT.y>Y-margeT)&&(point2dgaucheT.y<Y+margeT)) { printf("Faux Label T\n"); FLT = true; }
		if ((point2dgaucheMG.x>X-margeM)&&(point2dgaucheMG.x<X+margeM)&&(point2dgaucheMG.y>Y-margeM)&&(point2dgaucheMG.y<Y+margeM)) { printf("Faux Label MG\n"); FLMG = true; }
		if ((point2dgaucheMD.x>X-margeM)&&(point2dgaucheMD.x<X+margeM)&&(point2dgaucheMD.y>Y-margeM)&&(point2dgaucheMD.y<Y+margeM)) { printf("OK\n"); OKMD = true; }
	} else {
		if (point2dgaucheMD.x==-1) { printf("OK\n"); OKMD = true; }
		else printf("NON OK\n");
	}
	printf("résultat OK : T=%d, MG=%d, MD=%d\n",OKT,OKMG,OKMD);
	printf("résultat FL : T=%d, MG=%d, MD=%d\n",FLT,FLMG,FLMD);
	
	/* 2) comparaison image droite */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	/* tête */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "T:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2ddroiteT.x>X-margeT)&&(point2ddroiteT.x<X+margeT)&&(point2ddroiteT.y>Y-margeT)&&(point2ddroiteT.y<Y+margeT)) { printf("OK\n"); if (FLT) FLT=false; }//si c'était bon, ça reste bon, autrement c'est pas bon
		else OKT = false;
		if ((point2ddroiteMG.x>X-margeM)&&(point2ddroiteMG.x<X+margeM)&&(point2ddroiteMG.y>Y-margeM)&&(point2ddroiteMG.y<Y+margeM)) printf("Faux Label MG\n");
		if ((point2ddroiteMD.x>X-margeM)&&(point2ddroiteMD.x<X+margeM)&&(point2ddroiteMD.y>Y-margeM)&&(point2ddroiteMD.y<Y+margeM)) printf("Faux Label MD\n");
	} else {
		if (point2ddroiteT.x==-1) { printf("OK\n"); OKT = true; }
		else { printf("NON OK\n"); OKT=false; FLT=false; }
	}
	/* MG */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "MG:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2ddroiteT.x>X-margeT)&&(point2ddroiteT.x<X+margeT)&&(point2ddroiteT.y>Y-margeT)&&(point2ddroiteT.y<Y+margeT)) printf("Faux Label T\n");
		if ((point2ddroiteMG.x>X-margeM)&&(point2ddroiteMG.x<X+margeM)&&(point2ddroiteMG.y>Y-margeM)&&(point2ddroiteMG.y<Y+margeM)) { printf("OK\n"); if (FLMG) FLMG=false; }//si c'était bon, ça reste bon, autrement c'est pas bon
		else OKMG = false;
		if ((point2ddroiteMD.x>X-margeM)&&(point2ddroiteMD.x<X+margeM)&&(point2ddroiteMD.y>Y-margeM)&&(point2ddroiteMD.y<Y+margeM)) printf("Faux Label MD\n");
	} else {
		if (point2ddroiteMG.x==-1) { printf("OK\n"); OKMG = true; }
		else { printf("NON OK\n"); OKMG=false; FLMG=false; }
	}
	/* MD */
	if (!fgets(cheminfichier, 256, fichier)) { printf("ERREUR : fin du fichier d'étiquetage\n"); return; }
	printf("%s",cheminfichier);
	chartemp = strstr(cheminfichier, "MD:");
	if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
	chartemp = strstr(cheminfichier, "-1");
	if (chartemp==NULL) {//la cible est suivie
		chartemp = strstr(cheminfichier, "x=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		X=atoi(chartemp+sizeof("x=")-1);
		chartemp = strstr(cheminfichier, "y=");
		if (chartemp==NULL) { printf("ERREUR : mauvais format\n"); return; }
		Y=atoi(chartemp+sizeof("y=")-1);
		if ((point2ddroiteT.x>X-margeT)&&(point2ddroiteT.x<X+margeT)&&(point2ddroiteT.y>Y-margeT)&&(point2ddroiteT.y<Y+margeT)) printf("Faux Label T\n");
		if ((point2ddroiteMG.x>X-margeM)&&(point2ddroiteMG.x<X+margeM)&&(point2ddroiteMG.y>Y-margeM)&&(point2ddroiteMG.y<Y+margeM)) printf("Faux Label MG\n");
		if ((point2ddroiteMD.x>X-margeM)&&(point2ddroiteMD.x<X+margeM)&&(point2ddroiteMD.y>Y-margeM)&&(point2ddroiteMD.y<Y+margeM)) { printf("OK\n"); if (FLMD) FLMD=false; }//si c'était bon, ça reste bon, autrement c'est pas bon
		else OKMD = false;
	} else {
		if (point2ddroiteMD.x==-1) { printf("OK\n"); OKMD = true; }
		else { printf("NON OK\n"); OKMD=false; FLMD=false; }
	}
	
	printf("résultat OK : T=%d, MG=%d, MD=%d\n",OKT,OKMG,OKMD);
	printf("résultat FL : T=%d, MG=%d, MD=%d\n",FLT,FLMG,FLMD);
	
	if (OKT) scoreT++;
	if (OKMG) scoreMG++;
	if (OKMD) scoreMD++;
	if (FLT) scoreFLT++;
	if (FLMG) scoreFLMG++;
	if (FLMD) scoreFLMD++;
	if ((!OKT)||(!OKMG)||(!OKMD)) FRp++;
	if ((FLT)||(FLMG)||(FLMD)) FRl++;
}

/** fait la balance des blancs :
 * 
 * imgg,imgd : images à modifier
 * couleurDeRefBalanceDesBlancs  : couleur de référence de la balance des blancs (3 si la référence est la moyenne globale)
 * 
 * en fait on normalise le bleu et le rouge et on multiplie par la moyenne du vert
 */
void faitBalanceDesBlancs(IplImage *imgg, IplImage *imgd, int couleurDeRefBalanceDesBlancs) {
	
	IplImage *imageg[3];
	IplImage *imaged[3];
	CvScalar  moyenneg;
	CvScalar  moyenned;
	
	moyenneg = cvAvg(imgg);
	moyenned = cvAvg(imgd);
	for (int i=0; i<3; i++) imageg[i] = cvCreateImage(cvSize(imgg->width,imgg->height),imgg->depth,1);
	for (int i=0; i<3; i++) imaged[i] = cvCreateImage(cvSize(imgd->width,imgd->height),imgd->depth,1);
		
	if (couleurDeRefBalanceDesBlancs==3) {
		cvSplit(imgg,imageg[0],imageg[1],imageg[2],NULL);
		cvConvertScale(imageg[0],imageg[0],moyenneg.val[0]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvConvertScale(imageg[1],imageg[1],moyenneg.val[1]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvConvertScale(imageg[2],imageg[2],moyenneg.val[2]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvMerge(imageg[0],imageg[1],imageg[2],NULL,imgg);
			
		cvSplit(imgd,imaged[0],imaged[1],imaged[2],NULL);
		cvConvertScale(imaged[0],imaged[0],moyenned.val[0]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvConvertScale(imaged[1],imaged[1],moyenned.val[1]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvConvertScale(imaged[2],imaged[2],moyenned.val[2]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvMerge(imaged[0],imaged[1],imaged[2],NULL,imgd);
	} else {
		cvSplit(imgg,imageg[0],imageg[1],imageg[2],NULL);
		cvConvertScale(imageg[0],imageg[0],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[0]);
		cvConvertScale(imageg[1],imageg[1],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[1]);
		cvConvertScale(imageg[2],imageg[2],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[2]);
		cvMerge(imageg[0],imageg[1],imageg[2],NULL,imgg);
			
		cvSplit(imgd,imaged[0],imaged[1],imaged[2],NULL);
		cvConvertScale(imaged[0],imaged[0],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[0]);
		cvConvertScale(imaged[1],imaged[1],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[1]);
		cvConvertScale(imaged[2],imaged[2],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[2]);
		cvMerge(imaged[0],imaged[1],imaged[2],NULL,imgd);
	}
}

