/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* SkinBlobDetector.cpp
 * Auteur : BURGER Brice
 * Date : 03/2007
 */
#include "SkinBlobDetector.h"


/* Constructeur */
SkinBlobDetector::SkinBlobDetector() {

	CvSize imgSize=cvSize(w,h);
	
	probapeau = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	contour = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	
	/* images de travail */
	imgP1 = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	imgP2 = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	imgP3 = cvCreateImage(imgSize,IPL_DEPTH_8U,1);
	images = new IplImage*[3];
	images[0] = imgP1;
	images[1] = imgP2;
	images[2] = imgP3;
	
	contourSeq = 0;
	
	if (!ChargeModeleCouleur()) { printf("ERREUR FATALE\n"); exit(0); }//si on bloque ici, c'est toute l'initialisation qui est morte, impossible donc de démarrer
	
	/* allocation de la mémoire */
	storage = cvCreateMemStorage(0);
}


/* Destructeur */
SkinBlobDetector::~SkinBlobDetector() {
	
	cvReleaseImage(&probapeau);
	cvReleaseImage(&contour);
	cvReleaseMemStorage(&storage);
	
	cvReleaseImage(&imgP1);
	cvReleaseImage(&imgP2);
	cvReleaseImage(&imgP3);
	delete [] images;
}


/* Charge le modèle de couleur suivant l'espace de couleur courant
 * 
 * retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
 */
bool SkinBlobDetector::ChargeModeleCouleur() {
	
	/* Charge le modèle de couleur : baseRGB 0 ; baseBGR 1; baseYCrCb 2; baseHSV 3; baseI1I2I3 4; baseIrg 5; basergbN 6; baseLab 7 */
	switch (espaceCoul) {
		case baseRGB:
			skinModel.load(GEST_FICHIER_MODELEPEAU_RGB);
			dimhist = 3;
			break;
		case baseLab:
			skinModel.load(GEST_FICHIER_MODELEPEAU_LAB);
			limiteMin = 5;//...il faudrait que ces limites soient intégrer dans le fichier modèle
			limiteMax = 250;
			dimhist = 2;
			break;
		case baseYCrCb:
			skinModel.load(GEST_FICHIER_MODELEPEAU_YCRCB);
			limiteMin = 10;
			limiteMax = 245;
			dimhist = 2;
			break;
		case baseBGR:
		case baseI1I2I3:
		case basergbN:
		case baseHSV:
		case baseIrg:
		default:
			printf("ATTENTION : espace de couleur (%d) non traité!!\n",espaceCoul);
			printf("ATTENTION : chargement du modèle de couleur annulé!!\n");
			return false;
	}
	
	return true;
}


/* Remise À Zéro d'une zone de l'image de proba peau
 * 
 */
void SkinBlobDetector::RAZzone(CvRect rect) {
	
	/* vérifications (ce que opencv ne gère pas) */
	if (rect.width < 5) rect.width = 5;	//il faut rect.w et rect.h >= 5
	if (rect.height < 5) rect.height = 5;
	if (rect.x < 0) rect.x = 0;		//il faut rect.x et rect.y >= 0 et < w et h
	if (rect.x >= w) rect.x = w-1;
	if (rect.y < 0) rect.y = 0;
	if (rect.y >= h) rect.y = h-1;
	
	cvSetImageROI(probapeau, rect);
	cvZero(probapeau);
	cvResetImageROI(probapeau);
}


/* Remise À Zéro de l'image de proba peau
 * 
 * cette fonction est nécessaire quand on travail sur des images avec ROI :
 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
 * (c'est donc essentiellement pour l'affichage qu'il faut faire une RAZ)
 */
void SkinBlobDetector::RAZ() {
	
	cvResetImageROI(probapeau);
	cvZero(probapeau);
}


/* calcul de l'image de probabilité peau et des blobs 2D en découlant
 *
 * img : image sur laquelle on calcule
 * on revoie :
 * moy : liste des moyennes (~= centres) des blobs (doit avoir été précédemment aloué)
 * cov : liste des covariances correspondantes (doit avoir été précédemment aloué)
 * le nombre de blobs détectés est également renvoyé
 * 
 * si moy ou cov == NULL, on calcule seulement l'image de proba peau (pas de recherche de blobs), on retourne alors -1
 * 
 * les ROI sont prises en compte
 */
int SkinBlobDetector::calculProbaPeau(IplImage* img, int **moy, double **cov) {

	/* si img a une ROI, les images de travail doivent avoir les mêmes durant le calcul */
	cvSetImageROI(imgP1, cvGetImageROI(img));
	cvSetImageROI(imgP2, cvGetImageROI(img));
	cvSetImageROI(imgP3, cvGetImageROI(img));
	cvSetImageROI(probapeau, cvGetImageROI(img));
	cvSetImageROI(contour, cvGetImageROI(img));


	/******** Calcul de l'image de proba peau ***********/
	
	/* Séparation en trois plans (BGR) */
	cvSplit(img,imgP3,imgP2,imgP1,NULL);

	/* Segmentation par projection de l'histogramme */
	//...cvCalcBackProject(images,probapeau,skinModel.hist_thresh);//...seulement si on a fait skinModel.threshold(...);
	cvCalcBackProject(images,probapeau,skinModel.hist);
	
	/* suppression de certains pixels en fonction de leur luminosité */
	if (dimhist==2) {//il faut supprimmer les cases ayant une luminosité trop forte (blanc) ou trop faible (noir) (si dimhist==2, c'est que imgP3 est la luminosité)
		cvThreshold(imgP3,imgP3,limiteMin,255,CV_THRESH_TOZERO);
		cvThreshold(imgP3,imgP3,limiteMax,255,CV_THRESH_TOZERO_INV);
		cvThreshold(imgP3,imgP3,1,255,CV_THRESH_BINARY);//il faut que le masque soit binaire, sinon not met à 0 tout ce qui est < 255
		cvNot(imgP3,imgP3);
		cvSet(probapeau,cvScalar(0),imgP3);//on met à 0 les pixels désignés par le masque
	}

	/* Réduction du bruit */
	cvSmooth(probapeau,probapeau);

	/* série de dilations/érosions pour obtenir des blobs plus elliptiques (on comble les trous, palme les mains, ...) */
	if (nbfermeture) {
		cvDilate(probapeau,probapeau,NULL, nbfermeture);
		cvErode(probapeau,probapeau,NULL, nbfermeture);
	}
	
	/* Seuillage de l'image de proba pour supprimer les pixels de probabilité trop faible */
	cvThreshold(probapeau,probapeau,seuilbas,255,CV_THRESH_TOZERO);
	
	if ((moy==NULL)||(cov==NULL)) return -1;
	
	/******** Selection des blobs en fonction d'un seuil ***********/

	/* Regroupement des pixels en régions connexes */
	cvClearMemStorage(storage);//RAZ de la mémoire
	cvCopyImage(probapeau, contour);
	cvFindContours(contour, storage, &contourSeq, sizeof(CvContour), CV_RETR_EXTERNAL, CV_CHAIN_APPROX_SIMPLE );
	
	nbBlobs=0;//nombre de blob détectés
	cvResetImageROI(contour);//imageData doit représenter la vrai image (sans ROI)
	imageData = probapeau->imageData;//les moyennes et covariances sont calculées sur l'images de proba peau
	for (contourTemp = contourSeq, i=0; contourTemp != 0; contourTemp = contourTemp->h_next, i++ ) {
		
		cvSetImageROI(contour, cvGetImageROI(img));//cvDrawContours doit toujours se faire dans la ROI de départ
		cvDrawContours(contour, contourTemp, CV_RGB(1,1,1), CV_RGB(1,1,1), -1, CV_FILLED, 8 );
		
		/* calcul du rectangle englobant, ces coordonnées doivent être dans l'image réelle */
		blobRect = cvBoundingRect(contourTemp,1);//blobRect est dans le repère du ROI de départ!!!
		blobRect.x += cvGetImageROI(img).x;//on rajoute la position du coin supérieur gauche de la ROI pour la prendre en compte
		blobRect.y += cvGetImageROI(img).y;

		/* on désigne une ROI pour ne compter que dans blobRect */
		cvSetImageROI(contour, blobRect);
		if(cvCountNonZero(contour)>surfaceminblob) {//si la surface n'est pas trop petite
			
			/* RAZ */
			memset(moy[nbBlobs],0,2*sizeof(int));
			memset(cov[nbBlobs],0,4*sizeof(double));
			probaTotale = 0;
			
			/* calcul de la moyenne */
			for (y=blobRect.y ; y< blobRect.y+blobRect.height ; y++) {
				for (x=blobRect.x ; x< blobRect.x+blobRect.width ; x++) {
					proba = ((uchar*)(imageData+y*w))[x];
					moy[nbBlobs][0] += proba*x;
					moy[nbBlobs][1] += proba*y;		
					probaTotale += proba;
				}
			}
			
			// Normalisation (les probas sont des entiers [0 255] ici et probaTotale!=1)
			moy[nbBlobs][0] /= probaTotale;
			moy[nbBlobs][1] /= probaTotale;
			
			/* calcul de la covariance (centrée) */
			for (y=blobRect.y ; y< blobRect.y+blobRect.height ; y++) {
				for (x=blobRect.x ; x< blobRect.x+blobRect.width ; x++) {
					proba = ((uchar*)(imageData+y*w))[x];
					cov[nbBlobs][0] += pow(x-moy[nbBlobs][0],2)*proba;
					cov[nbBlobs][1] += (x-moy[nbBlobs][0])*(y-moy[nbBlobs][1])*proba;
					cov[nbBlobs][3] += pow(y-moy[nbBlobs][1],2)*proba;
				}
			}
			
			// Normalisation (les probas sont des entiers [0 255] ici et probaTotale!=1)
			cov[nbBlobs][0]/=(double)probaTotale;
			cov[nbBlobs][1]/=(double)probaTotale;
			cov[nbBlobs][3]/=(double)probaTotale;
			cov[nbBlobs][2] = cov[nbBlobs][1];//matrice symétrique!
			
			nbBlobs++;
			if (nbBlobs==BLOBS_PEAU_MAX) {
				printf("ATTENTION: SkinBlobDetector.cpp : trop de blobs\n");
				break;
			}
		}
	}
	
	/* pour ne pas perturber le reste du programme, on ne conserve pas les ROIs */
	cvResetImageROI(imgP1);
	cvResetImageROI(imgP2);
	cvResetImageROI(imgP3);
	cvResetImageROI(probapeau);
	cvResetImageROI(contour);
	
	return nbBlobs;
}



















