/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef SKINBLOBDETECTOR_H
#define SKINBLOBDETECTOR_H

/* 
 * \author BURGER Brice
 * \since 03/2007
 */

#include <cv.h>
#include <highgui.h>
#include "skincolormodel.h"
#include "detectionParams.h"




class SkinBlobDetector {
	
	/* images de travail */
	IplImage* contour;//image de contours
	IplImage **images;
	IplImage *imgP1;
	IplImage *imgP2;
	IplImage *imgP3;
	
	/* pour la détection de contours */
	CvMemStorage *storage;//mémoire pour opencv
	CvSeq *contourSeq;//séquence de contours déduite de l'image
	CvSeq *contourTemp;//contours en cours
	
	/* blobs peau */
	CvRect blobRect;//rectangle englobant du blob
	int nbBlobs;//nombre de blob détectés
	int proba;//proba du pixel en cours
	int probaTotale;//somme des probas du blob en cours (sert à la normalisation)
	SkinColorModel skinModel;//modèle de couleur peau
	
	/* divers */
	int i;//itération
	int x,y;//position dans l'image
	char *imageData;//accès aux informations de l'image
	int dimhist;//dimension de l'histogramme utilisé pour la rétro-projection
	int limiteMin,limiteMax;//limites de luminosité au-delà desquelles on ne regarde pas la couleur


	public:
	
	/** image calculée */
	IplImage* probapeau;//image de probabilité peau

	/** Constructeur */
	SkinBlobDetector();

	/** Destructeur */
	~SkinBlobDetector();
	
	/** Charge le modèle de couleur suivant l'espace de couleur courant
	 * 
	 * \return retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
	 */
	bool ChargeModeleCouleur();
	
	/** Remise À Zéro d'une zone de l'image de proba peau
	 * 
	 */
	void RAZzone(CvRect rect);
	
	/** Remise À Zéro d'une image
	 * 
	 * cette fonction est nécessaire quand on travail sur des images avec ROI :
	 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
	 * (c'est donc essentiellement pour l'affichage qu'il faut faire une RAZ)
	 */
	void RAZ();
	
	/** calcul de l'image de probabilité peau et des blobs 2D en découlant
	 *
	 * \param img : image sur laquelle on calcule
	 * \return on revoie :
	 * \return moy : liste des moyennes (~= centres) des blobs (doit avoir été précédemment aloué)
	 * \return cov : liste des covariances correspondantes (doit avoir été précédemment aloué)
	 * \return le nombre de blobs détectés est également renvoyé
	 * 
	 * si moy ou cov == NULL, on calcule seulement l'image de proba peau (pas de recherche de blobs), on retourne alors -1
	 * 
	 * les ROI sont prises en compte
	 */
	int calculProbaPeau(IplImage* img, int **moy, double **cov);

};
#endif
