/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* detectionBlobPeau.cpp
 * Auteur : BURGER Brice
 * Date : 11/2006
 */
#include "detectionBlobPeau.h"




/* Constructeur */
detectionBlobPeau::detectionBlobPeau(IplImage* imgG, IplImage* imgD) {
	
	detecteurBlobG = new SkinBlobDetector();
	detecteurBlobD = new SkinBlobDetector();
	projecteur = new projection();//on a besoin de pouvoir projeter
	
	moyG = new int*[BLOBS_PEAU_MAX];
	covG = new double*[BLOBS_PEAU_MAX];
	moyD = new int*[BLOBS_PEAU_MAX];
	covD = new double*[BLOBS_PEAU_MAX];
	cov3D = new double*[BLOBS_PEAU_MAX];
	p3D = new int*[BLOBS_PEAU_MAX];
	for (int i=0; i<BLOBS_PEAU_MAX; i++) {
		moyG[i] = new int[2];
		covG[i] = new double[4];
		moyD[i] = new int[2];
		covD[i] = new double[4];
		cov3D[i] = new double[9];
		p3D[i] = new int[3];
	}

	Ut = cvCreateMat( 3, 3, CV_64F );
	W = cvCreateMat( 3, 3, CV_64F );
	A = cvCreateMat( 3, 3, CV_64F );
	
	/* ne servent qu'en mode bavard : */
	Ut2D = cvCreateMat( 2, 2, CV_64F );
	W2D = cvCreateMat( 2, 2, CV_64F );
	A2D = cvCreateMat( 2, 2, CV_64F );
}


/* destructeur */
detectionBlobPeau::~detectionBlobPeau() {

	delete detecteurBlobG;
	delete detecteurBlobD;
	delete projecteur;
	
	for (int i=0; i<BLOBS_PEAU_MAX; i++) {
		delete[] moyG[i];
		delete[] covG[i];
		delete[] moyD[i];
		delete[] covD[i];
		delete[] cov3D[i];
		delete[] p3D[i];
	}
	delete[] moyG;
	delete[] covG;
	delete[] moyD;
	delete[] covD;
	delete[] cov3D;
	delete[] p3D;
	
	cvReleaseMat(&Ut);
	cvReleaseMat(&W);
	cvReleaseMat(&A);
	
	cvReleaseMat(&Ut2D);
	cvReleaseMat(&W2D);
	cvReleaseMat(&A2D);
}


/* mise à jour du modèle de couleur
 * 
 * retourne true si tout c'est bien passé, false autrement (mauvais espace de couleur)
 */
bool detectionBlobPeau::MAJModeleCouleur() {
	
	if (!detecteurBlobG->ChargeModeleCouleur()) return false;
	if (!detecteurBlobD->ChargeModeleCouleur()) return false;
	
	return true;
}

/* obtention de l'image de proba peau Gauche dont on s'est servi pour la détection
 * 
 */
IplImage* detectionBlobPeau::getImgProbaPeauG() {
	
	return detecteurBlobG->probapeau;
}

/* obtention de l'image de proba peau Droite dont on s'est servi pour la détection
 * 
 */
IplImage* detectionBlobPeau::getImgProbaPeauD() {
	
	return detecteurBlobD->probapeau;
}

/* efface une zone rectangulaire dans les images de proba peau
 * 
 */
void detectionBlobPeau::effaceZone(int* _centre, int _rayon) {
	
	
	rayon = int (_rayon*intrinsicsLeft[2]/_centre[2]);
	projecteur->projettePoint (_centre, &point2dgauche, &point2ddroite);//projection du centre
	
	detecteurBlobG->RAZzone(cvRect(point2dgauche.x-rayon, point2dgauche.y-rayon, rayon*2, rayon*2));
	detecteurBlobD->RAZzone(cvRect(point2ddroite.x-rayon, point2ddroite.y-rayon, rayon*2, rayon*2));
}

/* Remise À Zéro des images de proba peau
 * 
 * cette fonction est nécessaire quand on travail sur des images avec ROI :
 * on ne calcule alors que sur certaines parties de l'image, le reste de l'image ne correspondant donc à rien
 */
void detectionBlobPeau::RAZ() {
	
	detecteurBlobG->RAZ();
	detecteurBlobD->RAZ();
}


/* détection des blobs couleurs 3D
 * 
 * c'est le détecteur de base, il recherche tous les blobs contenus dans les images données en entrée
 * limitation : les blobs trop éloignés ou trop proches (au delà de DISTMAXRECHERCHE ou en deçà de DISTMINRECHERCHE) sont éliminés
 * 
 * On peut aussi ne rechercher les blobs peau que dans une zone cubique (dont les projections sur les images sont données en entrée)
 * 
 * bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
 * imgG et imgD sont les images a traiter (gauche et droite)
 * zones : zones 3D de recherche des blobs (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
 * strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
 * 
 * ellipsoides est la liste des ellipsoides renvoyees
 * on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyees
 * 
 * si ellipsoides==NULL, on calcule seulement les images de proba peau (pas de triangulation), on retourne alors -1
 */
int detectionBlobPeau::detecte(bool bavard, IplImage* imgG, IplImage* imgD, int** ellipsoides, CvRect* zones, bool strict) {
	
	if (bavard) cout << "***** detectionBlobPeau.cpp : DEBUT de la detection d'ellipsoides peau" << endl;
	
	/* image gauche */
	if (zones) cvSetImageROI(imgG, zones[0]);
	if (ellipsoides==NULL) detecteurBlobG->calculProbaPeau(imgG, NULL, NULL);
	else nbBlobG = detecteurBlobG->calculProbaPeau(imgG, moyG, covG);
	if (zones) cvResetImageROI(imgG);
	
	/* image droite */
	if (zones) cvSetImageROI(imgD, zones[1]);
	if (ellipsoides==NULL) detecteurBlobD->calculProbaPeau(imgD, NULL, NULL);
	else nbBlobD = detecteurBlobD->calculProbaPeau(imgD, moyD, covD);
	if (zones) cvResetImageROI(imgD);
	
	/* si aucune détection, on arrête *///mais il fallait tout de même calculer l'image de proba peau!
	if (ellipsoides==NULL) {
		if (bavard) cout << "***** detectionBlobPeau.cpp : FIN du calcul d'image de proba peau" << endl;
		return -1;
	}
	
	if (nbBlobD==0) {
		if (bavard) cout << "***** detectionBlobPeau.cpp : FIN de la détection d'ellipsoides peau, aucun blob détecté à droite" << endl;
		return 0;
	}
	if (nbBlobG==0) {
		if (bavard) cout << "***** detectionBlobPeau.cpp : FIN de la détection d'ellipsoides peau, aucun blob détecté à gauche" << endl;
		return 0;
	}
	
	if (bavard) {
		parle2D();
		cout << "ETAPE 1 : on a repéré les blobs couleurs et généré les ellipses correspondantes" << endl;
	}	
	
	/* triangulation et correction pour obtenir les ellipsoïdes 3D correspondantes */
	nbpt = trianguleEtCorrige (ellipsoides, bavard, strict);
	
	if (bavard) cout << "***** detectionBlobPeau.cpp : FIN de la detection d'ellipsoides peau" << endl;
	return nbpt;
}


/* détection des blobs couleurs 3D avec limitations
 * 
 * Il s'agit de la même fonction que détecte, mais avec des limitations. La fonction est séparée pour accélérer les calculs.
 * 
 * bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
 * imgG et imgD sont les images a traiter (gauche et droite)
 * ellipsoides est la liste des ellipsoides renvoyees
 * on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyees
 * strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
 * 
 * Limitations possibles :
 * rectG et rectD : liste des rectangles dans lesquels on ne cherche pas de blobs 2D dans l'image gauche ou droite
 * pour interdire une zone 3D, il suffit de la projetter et ainsi de mettre l'interdiction sous la forme de d'un rectangle dans chaque image
 */
int detectionBlobPeau::detecteAvecLimitations(bool bavard, IplImage* imgG, IplImage* imgD, int** ellipsoides, listeRect* rectG, listeRect* rectD, bool strict) {

	if (bavard) cout << "***** detectionBlobPeau.cpp : DEBUT de la detection d'ellipsoides peau" << endl;	
	
	nbBlobG = detecteurBlobG->calculProbaPeau(imgG, moyG, covG);
	nbBlobD = detecteurBlobD->calculProbaPeau(imgD, moyD, covD);
	
	/* Suppression des blobs peau non désirés dans l'image de gauche */
	for(i=0;i<rectG->nbRect;i++) {//pour tous les rectangles de la liste
		rect = rectG->rect[i];
		nbsuppr = 0;
		/* suppression des blobs 2D */
		for (j=0 ; j<nbBlobG ; j++) {
			if (moyG[j][0] > rect.x) if (moyG[j][0] < rect.x+rect.width) 
			if (moyG[j][1] > rect.y) if (moyG[j][1] < rect.y+rect.height) {
				for (k=j+1 ; k<nbBlobG-nbsuppr ; k++) {
					moyG[k-1][0] = moyG[k][0];
					moyG[k-1][1] = moyG[k][1];
					covG[k-1][0] = covG[k][0];
					covG[k-1][1] = covG[k][1];
				}
				nbsuppr++;
			}
		}
		nbBlobG-=nbsuppr;
	}
	/* Suppression des blobs peau non désirés dans l'image de droite */
	for(i=0;i<rectD->nbRect;i++) {//pour tous les rectangles de la liste
		/* extraction des rectangles proprement dit */
		rect = rectD->rect[i];
		nbsuppr = 0;
		/* suppression des blobs 2D */
		for (j=0 ; j<nbBlobD ; j++) {
			if (moyD[j][0] > rect.x) if (moyD[j][0] < rect.x+rect.width) 
			if (moyD[j][1] > rect.y) if (moyD[j][1] < rect.y+rect.height) {
				for (k=j+1 ; k<nbBlobD-nbsuppr ; k++) {
					moyD[k-1][0] = moyD[k][0];
					moyD[k-1][1] = moyD[k][1];
					covD[k-1][0] = covD[k][0];
					covD[k-1][1] = covD[k][1];
				}
				nbsuppr++;
			}
		}
		nbBlobD-=nbsuppr;
	}
	
	if (bavard) {
		parle2D();
		cout << "ETAPE 1 : on a repéré les blobs couleurs et généré les ellipses correspondantes" << endl;
	}	
	
	/* triangulation et correction pour obtenir les ellipsoïdes 3D correspondantes */
	nbpt = trianguleEtCorrige (ellipsoides, bavard, strict);
	
	if (bavard) cout << "***** detectionBlobPeau.cpp : FIN de la detection d'ellipsoides peau" << endl;
	return nbpt;
}


/*****************************************************************************/
/***************************** fonctions internes ****************************/
/*****************************************************************************/
/* Cette fonction affiche les résultats de la recherche de blobs 2D
 * 
 * on affiche les ellipses obtenues à partir des matrices de covariance
 */
void detectionBlobPeau::parle2D() {
	
	for (i=0;i<nbBlobG;i++) {//gauche
		Ut2D = cvCreateMat( 2, 2, CV_64F );
		W2D = cvCreateMat( 2, 2, CV_64F );
		A2D = cvCreateMat( 2, 2, CV_64F );
		A2D->data.db = covG[i];//A=covG
		
		cvSVD(A2D, W2D, Ut2D, NULL, CV_SVD_U_T);//A=U.W.Ut

		alpha = -180/3.1415*atan2(Ut2D->data.db[1],Ut2D->data.db[0]);
		centre.x = (int) moyG[i][0];
		centre.y = (int) moyG[i][1];
		axes.width = int (3*sqrt(W2D->data.db[0]));
		axes.height = int (3*sqrt(W2D->data.db[3]));

		cout << "ellipse gauche = centre,axes,angle = " << centre.x << " " << centre.y << " , " << axes.width << " " << axes.height << " , " << alpha << endl;
	}
	
	for (i=0;i<nbBlobD;i++) {//droite
		Ut2D = cvCreateMat( 2, 2, CV_64F );
		W2D = cvCreateMat( 2, 2, CV_64F );
		A2D = cvCreateMat( 2, 2, CV_64F );
		A2D->data.db = covD[i];//A=covD

		cvSVD(A2D, W2D, Ut2D, NULL, CV_SVD_U_T);//A=U.W.Ut
	
		alpha = -180/3.1415*atan2(Ut2D->data.db[1],Ut2D->data.db[0]);
		centre.x = (int) moyD[i][0];
		centre.y = (int) moyD[i][1];
		axes.width = int (3*sqrt(W2D->data.db[0]));
		axes.height = int (3*sqrt(W2D->data.db[3]));

		cout << "ellipse droite = centre,axes,angle = " << centre.x << " " << centre.y << " , " << axes.width << " " << axes.height << " , " << alpha << endl;
	}
}


/* Cette fonction triangule les blobs 2D, convertie les blobs 3D obtenus en ellipsoïde 3D, puis corrige le résultat
 * 
 * bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
 * strict : si true, on rejète tous les blobs dont le volume est trop faible ou trop élevé (par rapport au volume théorique d'une main)
 * 
 * ellipsoides est la liste des ellipsoides renvoyées
 * on renvoie egalement le nombre d'ellipsoides contenus dans la liste des ellipsoides renvoyées
 */
int detectionBlobPeau::trianguleEtCorrige(int** ellipsoides, bool bavard, bool strict) {
	
	/*********** triangulation pour obtenir les ellipsoides 3D qui correspondent aux ellipses précédentes ******************/
	nbpt3D = triangulationBlobs(bavard, nbBlobG,moyG,covG, nbBlobD,moyD,covD, p3D,cov3D);
	
	/********** on rempli la liste d'ellipsoides ***********************/
	nbpt=0;
	for (i=0;i<nbpt3D;i++) {
		p3d = p3D[i];
		if ((p3d[2]<DISTMAXRECHERCHE)&&(p3d[2]>DISTMINRECHERCHE)) {//on ne regarde pas à plus de DISTMAXRECHERCHE ni à moins de DISTMINRECHERCHE
			
			// on en déduit les coordonnées de l'ellipsoide
			A->data.db = cov3D[i];//A=cov3d
			
			cvSVD(A, W, Ut, NULL, CV_SVD_U_T);//A=U.W.Ut
			//cvTranspose( Ut, U );//on peut le faire soi même juste avec ce qu'on utilise
			
			/* mise en conformité du blob : étant donné la proximité des caméras, 
			 * la triangulation donne une taille en profondeur énorme, or on sait qu'elle n'est pas réaliste, 
			 * on la corrige en considérant que les deux autres demis-axes sont justes et que le volume d'une main reste à peu près constant
			 */
			
			/* le volume est considéré comme constant -> on s'en sert pour déterminer la profondeur */
			//ax = int( 3*sqrt(W->data.db[0]) );//peu importe l'ordre dans W : les angles le remettent en place, ici ax est toujours dans le sens de la profondeur (le plus grand)
			ay = int( 3*sqrt(W->data.db[4]) );
			az = int( 3*sqrt(W->data.db[8]) );
			if ((az==0)||(ay==0)) continue;//taille impossible, on passe
			ax = demivolumeMain / (ay * az);
			
			if (strict) if ((ax<tailleBlobMin)||(ax>tailleBlobMax)) {//la taille du blob est contrôlée à partir de sa profondeur
				if (bavard) printf ("ellipsoïde rejetée : ax,ay,az = %d,%d,%d\n",ax,ay,az);
				continue;
			}
			
			/* calcul des angles : theta et psi sont compris entre -180 et 180 degrés, phi entre -90 et 90 */
			theta = int( 180/3.1415*atan2(Ut->data.db[5],Ut->data.db[8]) );//angles en degres (on n'oublie pas qu'on part de Ut et pas U)
			phi = int( 180/3.1415*asin(-Ut->data.db[2]) );//angles en degres
			psi = int( 180/3.1415*atan2(Ut->data.db[1],Ut->data.db[0]) );//angles en degres
			
			/* on remplit le résultat */
			ellipsoides[nbpt][0]=p3d[0];
			ellipsoides[nbpt][1]=p3d[1];
			ellipsoides[nbpt][2]=p3d[2];
			ellipsoides[nbpt][3]=ax;//peu importe l'ordre dans W : les angles le remettent en place, ici ax est toujours dans le sens de la profondeur
			ellipsoides[nbpt][4]=ay;
			ellipsoides[nbpt][5]=az;
			ellipsoides[nbpt][6]=theta;//angles en degres (on n'oublie pas qu'on part de Ut et pas U)
			ellipsoides[nbpt][7]=phi;//angles en degres
			ellipsoides[nbpt][8]=psi;//angles en degres
			
			
			if (bavard) {
				cout << "ellipsoide numero " << nbpt << " detectee : ";
				for (j=0;j<9;j++) cout << ellipsoides[nbpt][j] << " ";
				cout << endl;
			}
			
			nbpt++;//on compte le nb de blobs assez proches
		}
	}
	return nbpt;
}

