/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef GESTPROJECTION_H
#define GESTPROJECTION_H


/** 
 * 
 * Fonctions de projections des objets 3D sur le plan image
 * 
 * \author BURGER Brice
 * \since 11/2006
 */



#include "math.h"
#include "cv.h"
#include "detectionParams.h"



class projection
{

	public:
	
	/** Constructeur */
	projection();
	
	/** Destructeur */
	~projection();
	
	/** Cette fonction projette une ellipsoïde 3D dans le plan
	 * \param particule : paramètres de l'ellipsoïde (3 translations, taille des demi-axes, 3 angles)=(tx, ty, tz, ax, ay, az, alpha, betha, gamma)
	 * \param resultat : paramètres de l'ellipse obtenue par cette projection (angle, taille des demi-axes)=(theta, ax, ay)
	 * 
	 * ATTENTION : le résultat est en mm, comme l'entrée, et non en pixels
	 */
	void projectionEllipsoide (int* particule, int* resultat);

	/** pour projetter un point 3D sur chaque image en fonction des paramètres intrinsèques */
	void projettePoint (int* point3d, CvPoint* point2dgauche, CvPoint* point2ddroite);

	/** pour mettre à l'échelle (en pixels) les axes projettés précédemment */
	void projetteAxes (int* axesproj, CvSize* axesg, CvSize* axesd);

	/** mêmes fonctions de projections de point et mise a l'échelle des axes, mais uniquement du côté gauche */
	void projettePointgauche (int* point3d, CvPoint* point2dgauche);
	void projetteAxesgauche (int* axesproj, CvSize* axesg);
	/** même fonction, mais en donnant la matrice intrinsèque soi-même -> permet de projeter dans une autre caméra (ou un autre repère) que celle de GEST */
	void projettePointgaucheAutrepart (int* point3d, CvPoint* point2dgauche, int u0, int v0, int au, int av);
	
	private:
	
	/* projection d'ellipsoïde */
	
	int tx,ty,tz,ax,ay,az;//particule
	double alpha,betha,gamma;
	double g,h,i,j,k,l,m,n,o;//matrice de rotation
	double p,q,r;//translations dans la matrice de passage inversée
	int axcarre,aycarre,azcarre;//on essaye de diminuer le nombre de calculs
	double gp,kp,op,sp,lp,pp,tp,qp,up,vp;//matrice de l'ellipsoïde après changement de repère
	double a,b,c,d,e,f;//equation de l'ellipse dans l'espace : ax²+2bxy+cy²+2dx+2ey+f=0
	double theta;//on en déduit les paramètres de l'ellipse dont on a besoin
	double ap,cp,dp,ep;//equation de l'ellipse dans l'espace sans rotations : a'x²+c'y²+2d'x+2e'y+f=0
	double fpp;//on essaye de diminuer le nombre de calculs
	
	/* autres projections */
	double* point3dd;
	double* axesprojd;
};

#endif
