/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* triangulation.cpp
 * Auteur : BURGER Brice
 * Date : 11/2006
 */
#include "triangulation.h"




static int*** distG;
static int*** distD;

static int** possibiliteappariementGD;//possibilités d'appariements gauche avec droite
static int** possibiliteappariementDG;//possibilités d'appariements droite avec gauche
static int* appariementGD;//appariement provisoires gauche avec droite
static int* appariementDG;//appariement provisoires droite avec gauche
static int* appariement;//appariement complet (les points non appariés sont marqués -1)

/* Cette fonction initialise l'espace de travail de la fonction de triangulation :
 * 
 */
void initTriangulation() {
	
	distG = new int**[BLOBS_PEAU_MAX];
	distD = new int**[BLOBS_PEAU_MAX];
	possibiliteappariementGD = new int*[BLOBS_PEAU_MAX];
	possibiliteappariementDG = new int*[BLOBS_PEAU_MAX];
	appariementGD = new int[BLOBS_PEAU_MAX];
	appariementDG = new int[BLOBS_PEAU_MAX];
	appariement = new int[BLOBS_PEAU_MAX];
	
	for (int i=0 ; i<BLOBS_PEAU_MAX ; i++) {
		distG[i] = new int*[BLOBS_PEAU_MAX];
		distD[i] = new int*[BLOBS_PEAU_MAX];
		possibiliteappariementGD[i] = new int[BLOBS_PEAU_MAX];
		possibiliteappariementDG[i] = new int[BLOBS_PEAU_MAX];
		for (int j=0 ; j<BLOBS_PEAU_MAX ; j++) {
			distG[i][j] = new int[2];
			distD[i][j] = new int[2];
			memset(distG[i][j],0,sizeof(int)*2);//les distances i=j ne sont pas calculées (distance d'un point avec lui même)
			memset(distD[i][j],0,sizeof(int)*2);
		}
	}
}


/* Cette fonction relache l'espace de travail de la fonction de triangulation :
 * 
 */
void finTriangulation() {

	for (int i=0 ; i<BLOBS_PEAU_MAX ; i++) {
		for (int j=0 ; j<BLOBS_PEAU_MAX ; j++) {
			delete[] distG[i][j];
			delete[] distD[i][j];
		}
		delete[] distG[i];
		delete[] distD[i];
		delete[] possibiliteappariementGD[i];
		delete[] possibiliteappariementDG[i];
	}
	delete[] distG;
	delete[] distD;
	delete[] possibiliteappariementGD;
	delete[] possibiliteappariementDG;
	delete[] appariementGD;
	delete[] appariementDG;
	delete[] appariement;
	
}


/* Cette fonction apparie et triangule des blobs :
 * nbBlobsG : nombre de blobs 2D côté gauche
 * nbBlobsD : nombre de blobs 2D côté droit
 * moyG : moyennes des blobs 2D du côté gauche
 * moyD : moyennes des blobs 2D du côté droit
 * covG : covariances des blobs 2D du côté gauche
 * covD : covariances des blobs 2D du côté droit
 * bavard : si FALSE, la fonction ne dit rien, sinon elle est bavarde
 * 
 * on renvoie le nombre de blobs triangulés, ainsi que moy3D et cov3D : moyennes et covariances des blobs triangulés
 * 
 * pour ne trianguler que des points (sans covariances), il suffit d'entrer NULL pour covG, covD et cov3D
 */
int triangulationBlobs (bool bavard, int nbBlobsG, int** moyG, double** covG, int nbBlobsD, int** moyD, double** covD, int** moy3D, double** cov3D) {

	if ((nbBlobsG==0)||(nbBlobsD==0)) return 0;
	
	register int i;
	register int j;
	int k;
	float diffdet;//quotient des déterminants des covariances (pour pouvoir comparer la surface des blobs)
	bool avecCov;
	
	if (covG==NULL || covD==NULL || cov3D==NULL) avecCov=false;
	else avecCov=true;
	
	/* 1 - Calcul des distances aux voisins */
	/****************************************/

	/* pour les points côté gauche */
	for (i=0; i<nbBlobsG; i++) for (j=i+1; j<nbBlobsG; j++) {//i!=j
		distG[i][j][0] = moyG[i][0] - moyG[j][0];
		distG[i][j][1] = moyG[i][1] - moyG[j][1];

		distG[j][i][0] = -distG[i][j][0];
		distG[j][i][1] = -distG[i][j][1];
	}

	/* pour les points côté droit */
	for (i=0; i<nbBlobsD; i++) for (j=i+1; j<nbBlobsD; j++) {//i!=j
		distD[i][j][0] = moyD[i][0] - moyD[j][0];
		distD[i][j][1] = moyD[i][1] - moyD[j][1];
		
		distD[j][i][0] = -distD[i][j][0];
		distD[j][i][1] = -distD[i][j][1];
	}
	
	
	/* 2 - construction des listes de possibilité d'appariement */
	/************************************************************/
	/* cette liste est construite grâce à quelques connaissances à priori :
	 * - l'image gauche est à gauche de l'image droite, cecla a des conséquences sur la position d'un même objet repéré dans les 2 images
	 * - pour des blobs, leur taille (et donc leur covariance) ne peut pas être trop différente d'une image à l'autre
	 */
	
	/* possibilités d'appariement gauche avec droite */
	for (i=0; i<nbBlobsG; i++) {
		k=0;
		possibiliteappariementGD[i][0] = -1;//par défaut, aucun appariement n'est possible
		
		for (j=0; j<nbBlobsD; j++) {
		
			if (moyG[i][0] >= moyD[j][0] - distSecuTriangulation) //un point sur l'image de gauche sera toujours plus à droite que dans l'image de droite
				if (moyG[i][1] >= moyD[j][1] - distSecuTriangulation) //un point sur l'image de gauche sera toujours à peu près à la même hauteur que dans l'image de droite
					if (moyG[i][1] <= moyD[j][1] + distSecuTriangulation) {//vérification sur la hauteur dans l'autre sens
						if (avecCov) {//vérification que la surface des blobs est proche
							diffdet = (covG[i][0]*covG[i][3] - covG[i][2]*covG[i][1]) / (covD[j][0]*covD[j][3] - covD[j][2]*covD[j][1]);
							if ((diffdet<0.19)||(diffdet>5.06)) continue;//on veut (2/3)² < rapport des surfaces < (3/2)² (pour que ni w, ni h n'ai une différence de rapport > 50%) <=> (2/3)^4 < rapport des déterminants < (3/2)^4 ,car surface = W*H = 3*racine(lamda1) * 3*racine(lamda2) = 9*racine(det)
						}
						possibiliteappariementGD[i][k]=j;
						k++;
						possibiliteappariementGD[i][k]=-1;//on indique la fin de la liste
					}
		}
	}

	/* possibilités d'appariement droite avec gauche */
	for (i=0; i<nbBlobsD; i++) {
		k=0;
		possibiliteappariementDG[i][0] = -1;//par défaut, aucun appariement n'est possible
		
		for (j=0; j<nbBlobsG; j++) {
		
			if (moyD[i][0] <= moyG[j][0] + distSecuTriangulation) //un point sur l'image de gauche sera toujours plus à droite que dans l'image de droite
				if (moyD[i][1] <= moyG[j][1] + distSecuTriangulation) //un point sur l'image de gauche sera toujours à peu près à la même hauteur que dans l'image de droite
					if (moyD[i][1] >= moyG[j][1] - distSecuTriangulation) {//vérification sur la hauteur dans l'autre sens
						if (avecCov) {//vérification que la surface des blobs est proche
							diffdet = (covD[i][0]*covD[i][3] - covD[i][2]*covD[i][1]) / (covG[j][0]*covG[j][3] - covG[j][2]*covG[j][1]);
							if ((diffdet<0.19)||(diffdet>5.06)) continue;//on veut (2/3)² < rapport des surfaces < (3/2)² (pour que ni w, ni h n'ai une différence de rapport > 50%) <=> (2/3)^4 < rapport des déterminants < (3/2)^4 ,car surface = W*H = 3*racine(lamda1) * 3*racine(lamda2) = 9*racine(det)
						}
						possibiliteappariementDG[i][k]=j;
						k++;
						possibiliteappariementDG[i][k]=-1;//on indique la fin de la liste
					}
		}
	}


	/* 3 - appariement + triangulation   */
	/*************************************/

	int nb3D = 0;

	creerAppariement(appariementGD, distG,nbBlobsG, distD,nbBlobsD, possibiliteappariementGD);
	creerAppariement(appariementDG, distD,nbBlobsD, distG,nbBlobsG, possibiliteappariementDG);
	
	recouperAppariements(appariement, appariementGD,nbBlobsG, appariementDG,nbBlobsD);
	
	for (i=0; i<nbBlobsG; i++) {//les appariement se font par rapport à la liste de gauche, il ne peut donc pas y avoir plus de nbBlobsG points appariés
		
		if (appariement[i]!=-1) {
			if (avecCov) 
				triangulate(moy3D[nb3D], NULL,
					moyG[i][0],moyG[i][1],
					intrinsicsLeft[0],intrinsicsLeft[1],intrinsicsLeft[2],intrinsicsLeft[3],
					moyD[appariement[i]][0],moyD[appariement[i]][1],
					intrinsicsRight[0],intrinsicsRight[1],intrinsicsRight[2],intrinsicsRight[3],
					Mdg,
					covG[i],covD[appariement[i]],cov3D[nb3D]);
			else 
				triangulate(moy3D[nb3D], NULL,
					moyG[i][0],moyG[i][1],
					intrinsicsLeft[0],intrinsicsLeft[1],intrinsicsLeft[2],intrinsicsLeft[3],
					moyD[appariement[i]][0],moyD[appariement[i]][1],
					intrinsicsRight[0],intrinsicsRight[1],intrinsicsRight[2],intrinsicsRight[3],
					Mdg,
					NULL,NULL,NULL);
			nb3D++;
		}
	}
	
	/* si mode bavard */
	if (bavard) {
		cout << "*** resultats de la triangulation :" << endl;
		for (i=0; i<nbBlobsG; i++) cout << "moyG[" << i << "] = " << moyG[i][0] << " " << moyG[i][1] << endl;
		for (i=0; i<nbBlobsD; i++) cout << "moyD[" << i << "] = " << moyD[i][0] << " " << moyD[i][1] << endl;
		for (i=0; i<nbBlobsG; i++) cout << "covG[" << i << "] = " << covG[i][0] << " " << covG[i][1] << " " << covG[i][2] << " " << covG[i][3] << " " << (covG[i][0]*covG[i][3] - covG[i][2]*covG[i][1]) << endl;
		for (i=0; i<nbBlobsD; i++) cout << "covD[" << i << "] = " << covD[i][0] << " " << covD[i][1] << " " << covD[i][2] << " " << covD[i][3] << " " << (covD[i][0]*covD[i][3] - covD[i][2]*covD[i][1]) << endl;
		
		/*
		diffCov = (covG[i][0]*covG[i][1])/(float)(covD[j][0]*covD[j][1]);
							printf("diffcov=%f\n",diffCov);
*/
		for (i=0; i<nbBlobsG; i++) {
			k=0;
			cout << "possibiliteappariementGD[" << i << "] : ";
			while (possibiliteappariementGD[i][k]!=-1) {
				cout << possibiliteappariementGD[i][k] << " ";
				k++;
			}
			cout << endl;
		}
		for (i=0; i<nbBlobsD; i++) {
			k=0;
			cout << "possibiliteappariementDG[" << i << "] : ";
			while (possibiliteappariementDG[i][k]!=-1) {
				cout << possibiliteappariementDG[i][k] << " ";
				k++;
			}
			cout << endl;
		}
		
		for (i=0; i<nbBlobsG; i++) cout << "appariementGD[" << i << "] = " << appariementGD[i] << endl;
		for (i=0; i<nbBlobsD; i++) cout << "appariementDG[" << i << "] = " << appariementDG[i] << endl;
		
		for (i=0; i<nbBlobsG; i++) cout << "appariement[" << i << "] = " << appariement[i] << endl;
		
		for (i=0; i<nb3D; i++) cout << "moy3D[" << i << "] = " << moy3D[i][0] << " " << moy3D[i][1] << " " << moy3D[i][2] << endl;
		
		cout << "*** fin de la triangulation :" << endl;
	}
	
	return nb3D;
}



/************************************************/
/*  fonctions utilisées pour la triangulation : */
/************************************************/  

/* Cette fonction apparie deux listes de points :
 * dist1 : liste de distances entres voisins du côté 1
 * dist2 : liste de distances entres voisins du côté 2
 * nb1 : nombre de points du côté 1
 * nb2 : nombre de points du côté 2
 * possibiliteappariement : possibilités d'appariements
 * 
 * on renvoie la liste des appariements du côté 1 avec le 2
 */
void creerAppariement(int *appariement, int ***dist1, int nb1, int ***dist2, int nb2, int **possibiliteappariement) {

	int distance;
	int distanceMin;
	int indexMin;
	unsigned int j,k;

	for (int i=0; i<nb1; i++) {
		indexMin = -1;
		k=0;
		distanceMin = 99999999;
		while (possibiliteappariement[i][k]!=-1) {//on ne calcule de distance que pour les points qu'il est possible d'apparier
			j = possibiliteappariement[i][k];
			k++;

			distance = getNeighBorDistance(i,dist1[i],nb1, j,dist2[j],nb2);
			
			if (distance < distanceMin || indexMin==-1) {
				indexMin = j;
				distanceMin = distance;
			}
		}
		appariement[i] = indexMin;// i.e. p2[indexMin] est le plus proche voisin de p1[i]
	}
}


/* Cette fonction calcule la distance des voisins de p1[I] par rapport au voisins de p2[J] via leurs listes de distances entre voisins :
 * I : numéro du point étudié côté 1
 * J : numéro du point étudié côté 2
 * dist1i : liste de distances de p1[i] à ses voisins
 * dist2j : liste de distances de p2[j] à ses voisins
 * nb1 : nombre de points du côté 1
 * nb2 : nombre de points du côté 2
 * 
 * on renvoie la distance calculée
 */
int getNeighBorDistance(int I, int **dist1i, int nb1, int J, int **dist2j, int nb2) {

	int distance = 0;
	double distanceCarre = 0;
	double distanceMinimaleCarre = 0; 
	int indexMin;
	
		
	/* cas où les points sont trop peu nombreux */
	if ((nb1==1) && (nb2==1)) return 0;
	
	if (nb1==1) {
		indexMin = -1;
		/* Recherche du voisin de B le plus proche de A */
		for (int j=0; j<nb2; j++) {
			if (j==J) continue;//pour éviter un faux zéro
			distanceCarre = pow( (dist2j[j][0]) ,2) + pow( (dist2j[j][1]) ,2);

			if (distanceCarre < distanceMinimaleCarre || indexMin == -1) {
				distanceMinimaleCarre = distanceCarre; 
				indexMin = j;
			}
		}
		distance = (int)sqrt(distanceMinimaleCarre);
		return distance;
	}
		
	if (nb2==1) {
		for (int i=0; i<nb1; i++) distance += (int)sqrt( pow( (dist1i[i][0]) ,2) + pow( (dist1i[i][1]) ,2) );
		return distance;
	}
	
	/* cas "normal" (le nombre de points suffit) */
	for (int i=0; i<nb1; i++) {
		indexMin = -1;
		/* Recherche du voisin de B le plus proche du i-eme voisin de A */
		for (int j=0; j<nb2; j++) {
			if ((i==I)&&(j==J)) continue;//pour éviter un faux zéro
			distanceCarre = pow( (dist1i[i][0] - dist2j[j][0]) ,2) + pow( (dist1i[i][1] - dist2j[j][1]) ,2);

			if (distanceCarre < distanceMinimaleCarre || indexMin == -1) {
				distanceMinimaleCarre = distanceCarre; 
				indexMin = j;
			}
		}
		distance += (int)sqrt(distanceMinimaleCarre);
	}

	return distance;// i.e. distance des voisins de p1[i] par rapport au voisins de p2[j]
}


/* Cette fonction recoupe deux liste d'appariements :
 * appariement1 : liste d'appariement d'un côté
 * appariement2 : liste d'appariement de l'autre côté
 * nb1 : taille de appariement1
 * nb2 : taille de appariement2
 * 
 * on renvoie la liste des appariements complets appariements
 */
void recouperAppariements(int *appariements, int *appariement1, int nb1, int *appariement2, int nb2) {

	for (int i=0; i<nb1; i++) {
		appariements[i] = -1;//par défaut, le iièmé pt de gauche n'est pas apparié
		for (int j=0; j<nb2; j++) {
			if ( (i==appariement2[j]) && (appariement1[i]==j) ) {//si les appariements g->d et d->g correspondent, l'appariement deviens entier (g<->d)
				appariements[i] = j;//le iièmé pt de gauche est apparié au jiéme pt de droite
				break;
			}
		}
	}
}


