/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* detectionVisage.cpp
 * Auteur : BURGER Brice
 * Date : 02/2007
 */

#include "detectionVisage.h"


/* Constructeur */
detectionVisage::detectionVisage() {//plus tard, on pourra peut-être choisir le type de détection, mais pour l'instant, çà n'a aucun intérêt

	detecteurVisageG = new faceDetector("haarcascade_frontalface_alt2.xml");
	detecteurVisageD = new faceDetector("haarcascade_frontalface_alt2.xml");
	/*  Possibilités :
	(visage de face)
	haarcascade_frontalface_alt2.xml
	haarcascade_frontalface_alt_tree.xml
	haarcascade_frontalface_alt.xml
	haarcascade_frontalface_default.xml
	(visage de coté)
	haarcascade_profileface.xml
	(corps)
	haarcascade_fullbody.xml
	haarcascade_lowerbody.xml
	haarcascade_upperbody.xml
	*/
	//...je garde default, parcequ'il repère plus facilement les visages et est le plus rapide, mais il fait des fausses détections parfois
	
	/* pour la triangulation */
	p1 = new int*[VISAGE_MAX];
	p2 = new int*[VISAGE_MAX];
	p3D = new int*[VISAGE_MAX];
	for (i=0; i<VISAGE_MAX; i++) {
		p1[i] = new int[2];
		p2[i] = new int[2];
		p3D[i] = new int[3];
	}
}


/* Destructeur */
detectionVisage::~detectionVisage() {
	
	delete detecteurVisageG;
	delete detecteurVisageD;
	
	for (i=0; i<VISAGE_MAX; i++) {
		delete[] p1[i];
		delete[] p2[i];
		delete[] p3D[i];
	}
	delete[] p1;
	delete[] p2;
	delete[] p3D;
}


/* détection de visages 3D 
 * 
 * c'est le détecteur de base, il recherche tous les visages contenus dans les images données en entrée
 * 
 * imgG et imgD sont les images a traiter (gauche et droite)
 * 
 * visages est la liste des visages renvoyees (position du centre de la sphère)
 * on renvoie également le nombre de visages contenus dans la liste de visages renvoyée
 */
int detectionVisage::detecte(IplImage* imgG, IplImage* imgD, int** visages) {
	
	detecteurVisageG->trouve(imgG, &visagesG);
	if (visagesG.nbRect==0) return 0;
	
	detecteurVisageD->trouve(imgD, &visagesD);
	if (visagesD.nbRect==0) return 0;
	
	/* calcul de la position du centre des visages */
	for (i=0;i<visagesG.nbRect;i++) {//pour tous les visages repérés à gauche
		rect = &(visagesG.rect[i]);
		p1[i][0] = rect->x + rect->width/2;
		p1[i][1] = rect->y + rect->height/2;
	}
	for (i=0;i<visagesD.nbRect;i++) {//pour tous les visages repérés à droite
		rect = &(visagesD.rect[i]);
		p2[i][0] = rect->x + rect->width/2;
		p2[i][1] = rect->y + rect->height/2;
	}
	
	/* triangulation */
	nbpt3D = triangulationBlobs(false, visagesG.nbRect,p1,NULL, visagesD.nbRect,p2,NULL, p3D,NULL);
	j=0;
	for (i=0;i<nbpt3D;i++) {
		if ((p3D[i][2]<DISTMAXRECHERCHE)&&(p3D[i][2]>DISTMINRECHERCHE)) {
			visages[j][0] = p3D[i][0];
			visages[j][1] = p3D[i][1];
			visages[j][2] = p3D[i][2];
			j++;
		}
	}
	
	return j;
}

/* détection de visages 3D : 
 * 
 * on recherche ici dans une zone
 * 
 * imgG et imgD sont les images a traiter (gauche et droite)
 * zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
 * 
 * visages est la liste des visages renvoyees (position du centre de la sphère)
 * on renvoie également le nombre de visages contenus dans la liste de visages renvoyée
 */
int detectionVisage::detecteAvecZones(IplImage* imgG, IplImage* imgD, int** visages, CvRect* zones) {
	
	/* image gauche */
	if (zones) cvSetImageROI(imgG, zones[0]);
	detecteurVisageG->trouve(imgG, &visagesG);//recherche de visage
	if (zones) cvResetImageROI(imgG);//pour ne pas perturber le reste du programme, on ne conserve pas cette ROI
	if (visagesG.nbRect==0) return 0;
	
	/* image droite */
	if (zones) cvSetImageROI(imgD, zones[1]);
	detecteurVisageD->trouve(imgD, &visagesD);//recherche de visage
	if (zones) cvResetImageROI(imgD);//pour ne pas perturber le reste du programme, on ne conserve pas cette ROI
	if (visagesD.nbRect==0) return 0;
	
	/* calcul de la position du centre des visages */
	for (i=0;i<visagesG.nbRect;i++) {//pour tous les visages repérés à gauche
		rect = &(visagesG.rect[i]);
		p1[i][0] = rect->x + zones[0].x + rect->width/2;//on rajoute la position du coin supérieur gauche de la ROI pour la prendre en compte
		p1[i][1] = rect->y + zones[0].y + rect->height/2;
	}
	for (i=0;i<visagesD.nbRect;i++) {//pour tous les visages repérés à droite
		rect = &(visagesD.rect[i]);
		p2[i][0] = rect->x + zones[1].x + rect->width/2;//on rajoute la position du coin supérieur gauche de la ROI pour la prendre en compte
		p2[i][1] = rect->y + zones[1].y + rect->height/2;
	}
	
	/* triangulation */
	nbpt3D = triangulationBlobs(false, visagesG.nbRect,p1,NULL, visagesD.nbRect,p2,NULL, p3D,NULL);
	j=0;
	for (i=0;i<nbpt3D;i++) {
		if ((p3D[i][2]<DISTMAXRECHERCHE)&&(p3D[i][2]>DISTMINRECHERCHE)) {
			visages[j][0] = p3D[i][0];
			visages[j][1] = p3D[i][1];
			visages[j][2] = p3D[i][2];
			j++;
		}
	}
	
	return j;
}

/* détection de visages 3D : 
 * 
 * on recherche ici dans une zone qui est sensée ne contenir qu'un seul visage (si plusieurs sont détectés, seul le premier sera renvoyé)
 * 
 * imgG et imgD sont les images a traiter (gauche et droite)
 * zones : zones 3D de recherche du visage (sous forme des rectangles projetés sur les images) (NULL pour calculer sur toute l'image)
 * 
 * visage est le visage renvoyé (position du centre de la sphère)
 * On renvoie également si un visage a été détecté ou non.
 */
bool detectionVisage::detecteUnSeul(IplImage* imgG, IplImage* imgD, int* visage, CvRect* zones) {
	
	/* image gauche */
	if (zones) cvSetImageROI(imgG, zones[0]);
	detecteurVisageG->trouve(imgG, &visagesG);//recherche de visage
	if (zones) cvResetImageROI(imgG);//pour ne pas perturber le reste du programme, on ne conserve pas cette ROI
	if (visagesG.nbRect==0) return false;
	
	/* image droite */
	if (zones) cvSetImageROI(imgD, zones[1]);
	detecteurVisageD->trouve(imgD, &visagesD);//recherche de visage
	if (zones) cvResetImageROI(imgD);//pour ne pas perturber le reste du programme, on ne conserve pas cette ROI
	if (visagesD.nbRect==0) return false;
	
	/* calcul de la position du centre des visages */
	for (i=0;i<visagesG.nbRect;i++) {//pour tous les visages repérés à gauche
		rect = &(visagesG.rect[i]);
		p1[i][0] = rect->x + zones[0].x + rect->width/2;//on rajoute la position du coin supérieur gauche de la ROI pour la prendre en compte
		p1[i][1] = rect->y + zones[0].y + rect->height/2;
	}
	for (i=0;i<visagesD.nbRect;i++) {//pour tous les visages repérés à droite
		rect = &(visagesD.rect[i]);
		p2[i][0] = rect->x + zones[1].x + rect->width/2;//on rajoute la position du coin supérieur gauche de la ROI pour la prendre en compte
		p2[i][1] = rect->y + zones[1].y + rect->height/2;
	}
	
	/* triangulation */
	nbpt3D = triangulationBlobs(false, visagesG.nbRect,p1,NULL, visagesD.nbRect,p2,NULL, p3D,NULL);
	
	/* on triangule tout, mais on ne prend que le premier visage qui est cohérent (s'il y en a d'autres, on les oublie) */
	for (i=0;i<nbpt3D;i++) {
		if ((p3D[i][2]<DISTMAXRECHERCHE)&&(p3D[i][2]>DISTMINRECHERCHE)) {
			visage[0] = p3D[i][0];
			visage[1] = p3D[i][1];
			visage[2] = p3D[i][2];
			return true;
		}
	}
	
	return false;//si on n'a rien trouvé de cohérent
}

