/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* evaluation.cpp
 * 
 * Ce fichier contient les fonction permettant d'évaluer la qualité des particules générée par le traqueur.
 * 
 * On travaille ici en 2D!
 * 
 * Auteur : BURGER Brice
 * Date : 01/2007
 */


#include "evaluation.h"


/* Constructeur */
evaluation::evaluation() {
	
	pts = new CvPoint[nbPtEchantillonnageEllipse];
	mesurecoul1 = new MesureCouleur();
	mesurecoul2 = new MesureCouleur();
	mesurecoulREF = new MesureCouleur();
}


/* Destructeur */
evaluation::~evaluation() {
	
	delete[] pts;
	delete mesurecoul1;
	delete mesurecoul2;
	delete mesurecoulREF;
}


/* Cette fonction initialise les fonctions d'évaluation et les tableaux et classes utilisés
 * 
 */
void evaluation::init(IplImage* img, CvPoint centre, int rayon) {
	
	//...pour l'instant, un seul histogramme de référence pour gauche et droite (on pourrait faire la moyenne des deux, ou un truc dans le genre)
	//...à changer : on devrait faire 2 instances d'evaluation (1 pour chaque image (gauche et droite)) ça éviterais ce aaaaaaa
	mesurecoulREF->CalcFromCercle(img, centre, rayon);//mesurecoulREF->histo est maintenant l'histogramme de référence
	aaaaaaaaa = 0;
}


/* calcule la vraisemblance d'une ellipse
 * 
 * on commence par discrétiser l'ellipse en nbPtEchantillonnageEllipse points de contour,
 * puis on calcule l'ensemble des vraissemblances qu'on veut prendre en compte
 * 
 * imgprobapeau : l'image de probabilité de peau (en niveaux de gris donc)
 * imgmvt : l'image de mouvement
 * imgdist : l'image de distance
 * centre,axes et angle : ellipse à évaluer
 * les unités sont le pixel et le degré
 * 
 * retourne la vraissemblance calculée
 * 
 * Si une image = NULL, ce type de vraissemblance ne sera pas prise en compte
 */
double evaluation::evalEllipse(IplImage* imgprobapeau, IplImage* imgmvt, IplImage* imgdist, CvPoint centre, CvSize axes, double angle) {
	
	/* discretisation de l'ellipse */
	nbpt = discretise_ellipse(centre,axes,angle,nbPtEchantillonnageEllipse);
	if (nbpt==0) return 0;

	/* calculs de vraissemblance */
	eval = 1;
	if (imgprobapeau) {
		eval *= ( 1 + proximitepeau(imgprobapeau, nbpt, centre) );//vraissemblance couleur peau
		if (eval==1) return 0;//si proba peau == 0, le reste des mesures ne comptent pas (le reste c'est du bonus)
	}
	if (imgmvt) eval *= ( 1 + tauxMouvement(imgmvt, nbpt, centre) );//on pondère (par le haut) par le taux de mouvement
	if (imgdist) eval *= ( 1 + 0.01*distanceContour(imgdist, nbpt, centre) );
	
	return eval;
}


/* calcule la vraisemblance d'un cercle
 * 
 * on commence par discrétiser le cercle en nbPtEchantillonnageCercle points de contour,
 * puis on calcule l'ensemble des vraissemblances qu'on veut prendre en compte
 * 
 * imgprobapeau : l'image de probabilité de peau (en niveaux de gris donc)
 * imgcoul : image en cours
 * imgdist : l'image de distance
 * centre,rayon : cercle à évaluer
 * les unités sont le pixel et le degré
 * 
 * retourne la vraissemblance calculée
 * 
 * Si imgcoul ou imgprobapeau = NULL, ce type de vraissemblance ne sera pas prise en compte
 */
double evaluation::evalCercle(IplImage* imgprobapeau, IplImage* imgcoul, IplImage* imgdist, CvPoint centre, int rayon) {
	
	/* discretisation de l'ellipse */
	nbpt = discretise_cercle(centre,rayon,nbPtEchantillonnageCercle);
	if (nbpt==0) return 0;
	
	/* calculs de vraissemblance */
	eval = 1;
	if (imgcoul) {
		if (aaaaaaaaa == 0) { mesurecoul = mesurecoul1; aaaaaaaaa++; }
		else { mesurecoul = mesurecoul2; aaaaaaaaa--;  }
		
		mesurecoul->CalcFromCercle(imgcoul, centre, rayon);//calcul de l'histogramme de couleur du cercle
		eval *= ( 1 + mesurecoul->BhattaDistance(mesurecoulREF->histo, sigmacouleur) );//vraissemblance liée à la couleur
		if (eval==1) return 0;//si l'histogramme est complètement faux (le reste c'est du bonus)
	}
	if (imgprobapeau) eval *= ( 1 + 0.5*proximitepeau(imgprobapeau, nbpt, centre) );//on pondère (par le haut) par la vraissemblance couleur peau (la tête peut être de dos)
	if (imgdist) eval *= ( 1 + 0.01*distanceContour(imgdist, nbpt, centre) );
	
	//... la mesure de distance au visage détecté se fait en 3D et est donc dans traqueurDeDeuxMains.cpp
	
	return eval;
}


/* met à jour ce qui peut l'être : (pour l'instant que l'histogramme de couleur)
 * 
 */
void evaluation::MAJeval() {
	
	mesurecoulREF->MiseAJour(mesurecoul->histo, tauxMAJcoul);
	mesurecoul1->RAZmasque();
	mesurecoul2->RAZmasque();
	//mesurecoul->Disp("coul", 1);
	//mesurecoulREF->Disp("ref", 1);
}


/* fonction calculant le poids à donner à une particule en fonction de sa vraissemblance et de celle des autres particules :
 * 
 * V : vraissemblance de la particule
 * Vmax : vraissemblance maximale
 */
double evaluation::poids(double V, double Vmax) {

	if (V==0) return 0;

	Poids = 2*(Vmax - V)/V;
	Poids = exp( -Poids*Poids );
	
	return Poids;
}





/**********************/
/* fonctions internes */
/**********************/

/* calcule la vraisemblance d'une forme discrétisée selon une méthode inspirée de THAYANANTHAN
 * 
 * on calcule la moyenne des probabilités de peau vers l'intérieur et vers l'extérieur.
 * au final on cherche à avoir une proba maximale à l'intérieur de l'ellipse et minimale à l'extérieur
 * 
 * img : l'image de probabilité de peau (en niveaux de gris donc)
 * N : nombre de points discrétisés
 * centre : centre de la forme
 * les unités sont le pixel et le degré
 * 
 * retourne g = exp( 1 - 1/moyenne des probas peau intérieures ) * exp( moyenne des probas peau extérieures / (moyenne des probas peau extérieures-1) )
 */
double evaluation::proximitepeau(IplImage* img, int N, CvPoint centre) {
	
	//int Pint,Pext;//somme des probas peau intérieures et extérieures
	//int J,K;//coordonnees du point en cours
	//int deltaX, deltaY;//distance du point de contour au centre suivant l'axe x et y
	//unsigned int dX, dY;//valeur absolue de deltaX, deltaY
	
	moyenneint=0;
	moyenneext=0;

	/* pour tous les points de contour (pts) */
	for (register int i=0 ; i<N ; i++) {
		J = pts[i].x;
		K = pts[i].y;
		Pint=0;

		/* on avance vers le centre de la forme */
		for (j=1;j<distMaxEval;j++) {//distMaxEval limite la zone de recherche

			/* on détermine le vecteur de pts[i] vers le centre */
			deltaX = centre.x-J;
			deltaY = centre.y-K;
			if (deltaX==0) { if (deltaY==0) break; }//si on a atteint le centre
			dX = abs(deltaX);
			dY = abs(deltaY);

			/* on avance vers le centre */
			if (MAX(dX,dY)==dX) {//on cherche le plus grand
				if ((dY/(double)dX)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J++;
					else J--;
					if (deltaY>0) K++;
					else K--;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaX>0) J++;
					else J--;
				}
			}
			else {
				if ((dX/(double)dY)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J++;
					else J--;
					if (deltaY>0) K++;
					else K--;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaY>0) K++;
					else K--;
				}
			}
			
			/* on verifie qu'on est bien dans l'image, sinon on arrête le décompte là */
			if (J<0) break;
			if (J>=img->width) break;
			if (K<0) break;
			if (K>=img->height) break;

			/* on récupère la proba peau de ce pixel */
			Pint += *( (unsigned char*)img->imageData+K*img->widthStep+J );
		}
		moyenneint += Pint/(double)j;
		
		J = pts[i].x;
		K = pts[i].y;
		Pext=0;
		
		/* on avance vers l'extérieur de la forme */
		for (j=1;j<distMaxEval;j++) {//distMaxEval limite la zone de recherche

			/* on détermine le vecteur de pts[i] vers le centre */
			deltaX = centre.x-J;
			deltaY = centre.y-K;
			if (deltaX==0) { if (deltaY==0) break; }//si on a atteint le centre
			dX = abs(deltaX);
			dY = abs(deltaY);

			/* on avance à l'encontre du centre */
			if (MAX(dX,dY)==dX) {//on cherche le plus grand
				if ((dY/(double)dX)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J--;
					else J++;
					if (deltaY>0) K--;
					else K++;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaX>0) J--;
					else J++;
				}
			}
			else {
				if ((dX/(double)dY)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J--;
					else J++;
					if (deltaY>0) K--;
					else K++;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaY>0) K--;
					else K++;
				}
			}
			
			/* on verifie qu'on est bien dans l'image, sinon on arrête le décompte là */
			if (J<0) break;
			if (J>=img->width) break;
			if (K<0) break;
			if (K>=img->height) break;

			/* on récupère la proba peau de ce pixel */
			Pext += *( (unsigned char*)img->imageData+K*img->widthStep+J );
		}
		moyenneext += Pext/(double)j;
	}
	
	if ((moyenneint==0)||(moyenneext==N*255)) g=0;
	else g = exp( 1 - N*255/moyenneint ) * exp( moyenneext / (moyenneext-N*255) );//facteur 255 : on remet les probas entre 0 et 1 (elles étaient entre 255 et 0 dans l'image de proba peau)
//cout << "moy ext = "  << moyenneext/N << " , moy int = " << moyenneint/N << " , g = " << g << endl;
	return g;
}


/* calcule le "taux de mouvement" dans une forme discrétisée
 * 
 * on calcule la moyenne de la valeur des pixels (sur les orthogonales à la forme aux points discrétisés) sur l'image de mouvement.
 * cette moyenne est considérée comme étant le "taux de mouvement"
 * 
 * img : l'image de mouvement
 * N : nombre de points discrétisés
 * centre : centre de la forme
 * les unités sont le pixel et le degré
 * 
 * retourne g = 1 - exp( - ( moyenne mouvement intérieur / (1 - moyenne mouvement intérieur) ) / sigmamvt )
 */
double evaluation::tauxMouvement(IplImage* img, int N, CvPoint centre) {
	
	//int Mint;//somme des valeurs des pixels intérieurs
	//int J,K;//coordonnees du point en cours
	//int deltaX, deltaY;//distance du point de contour au centre suivant l'axe x et y
	//unsigned int dX, dY;//valeur absolue de deltaX, deltaY
	
	moyenneint=0;

	/* pour tous les points de contour (pts) */
	for (register int i=0 ; i<N ; i++) {
		J = pts[i].x;
		K = pts[i].y;
		Mint=0;

		/* on avance vers le centre de la forme */
		for (j=1;j<distMaxEval;j++) {//distMaxEval limite la zone de recherche

			/* on détermine le vecteur de pts[i] vers le centre */
			deltaX = centre.x-J;
			deltaY = centre.y-K;
			if (deltaX==0) { if (deltaY==0) break; }//si on a atteint le centre
			dX = abs(deltaX);
			dY = abs(deltaY);

			/* on avance vers le centre */
			if (MAX(dX,dY)==dX) {//on cherche le plus grand
				if ((dY/(double)dX)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J++;
					else J--;
					if (deltaY>0) K++;
					else K--;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaX>0) J++;
					else J--;
				}
			}
			else {
				if ((dX/(double)dY)>=0.5) {//on avance vers le centre suivant x et y
					if (deltaX>0) J++;
					else J--;
					if (deltaY>0) K++;
					else K--;
				}
				else {//on avance vers le centre suivant la direction du plus grand
					if (deltaY>0) K++;
					else K--;
				}
			}
			
			/* on verifie qu'on est bien dans l'image, sinon on arrête le décompte là */
			if (J<0) break;
			if (J>=img->width) break;
			if (K<0) break;
			if (K>=img->height) break;

			/* on récupère la valeur de ce pixel */
			Mint += *( (unsigned char*)img->imageData+K*img->widthStep+J );
		}
		moyenneint += Mint/(double)j;
	}
	
	if (moyenneint==0) g=0;
	else g = 1 - exp(-moyenneint/( sigmamvt*(255*N-moyenneint) ));//facteur 255 : on remet entre 0 et 1 (c'étaient entre 255 et 0 dans l'image de mouvement)
	//=1 - exp(-(g/(1-g))/sigmamvt) avec g = moyenneint/(255*N)
	
	return g;
}


/* calcule la distance aux contours d'une forme discrétisée
 * 
 * on calcule la moyenne de la valeur des pixels discrétisés sur l'image de distance.
 * 
 * img : l'image de distance
 * N : nombre de points discrétisés
 * 
 * retourne g = exp( - moyenne distance au carré / sigmacontour );
 */
double evaluation::distanceContour(IplImage* img, int N, CvPoint centre) {
	
	dist=0;
	/* pour tous les points de contour (pts) */
	for (register int i=0 ; i<N ; i++) {
		J = pts[i].x;//coordonnees du point en cours
		K = pts[i].y;

		/* on récupère la valeur de ce pixel */
		dist += *( (unsigned char*)img->imageData+K*img->widthStep+J );
	}
	
	dist /= N;//distance moyenne de la forme au contour le plus proche
//cout << "dist = " << dist << ", exp(-dist*dist/sigmacontour) = " << exp(-dist*dist/sigmacontour) << endl;
	return exp(-dist*dist/sigmacontour);
}


/* Discrétisation de l'ellipse : 
 * on la découpe suivant des plans horizontaux pour en extraire des points de contours
 *
 * centre, axes et angle : caractéristiques de l'ellipse
 * nb_pt : nombre de points a obtenir
 * retourne le nombre de points effectivement créés
 */
int evaluation::discretise_ellipse(CvPoint centre, CvSize axes, double angle, int nb_pt) {

	a=axes.width;
	b=axes.height;
	c=cos(3.1415*angle/180);//angle est en degrés!
	s=sin(3.1415*angle/180);//angle est en degrés!

	A = (c*c)/(a*a)+(s*s)/(b*b); 
	B = -2*c*s*(1/(a*a) - 1/(b*b)); 
	C = (s*s)/(a*a)+(c*c)/(b*b);

	pDelta = B*B - A*C*4;
	A4 = A*4;
	inv2A = 1/(2*A);
	
	D = sqrt(A4/-pDelta);
	pas = 4*D/nb_pt;//pas de discrétisation (2*delta=hauteur de l'ellipse suivant l'axe vertical)

	
	/* extrémité haute */
	pts[0].y = -(int)D+centre.y;//extrémité haute (y le plus petit)
	pts[0].x = int ( (-B*(pts[0].y-centre.y))*inv2A + centre.x );
	
	N=1;
	I=-D+centre.y;
	if (I>h) return 0;//si on est deja trop bas pour pts[0], alors aucun point ne sera a l'interieur de l'image
	if (I<0) N--;//si pts[0] sort de l'image (trop haut), on le supprime
	else if ( pts[0].x<0 || pts[0].x>w ) N--;//si on sort de l'image, on supprime le point. Le else est la pour empecher de faire -- deux fois
	
	/* on parcoure l'ellipse suivant l'axe vertical pour calculer les points qui marchent par deux */
	for (register int i=1 ; i<nb_pt-1 ; i+=2) {
		I += pas;
		
		if (I<0) continue;//si on sort de l'image (on est trop haut), on supprime les deux points de cette ligne et on passe a la ligne suivante
		if (I>h) return N;//si on sort de l'image (on est trop bas), il n'y aura plus d'autres points
		
		Y = I-centre.y;
		delta = sqrt(pDelta*Y*Y + A4);
		
		pts[N].y= (int)I;
		pts[N].x= int ( (-B*Y - delta)*inv2A + centre.x );
		
		if ( pts[N].x<0 || pts[N].x>w ) N--;//si on sort de l'image, on supprime le point
		if (N>1) if (pts[N].x==pts[N-2].x) if (pts[N].y==pts[N-2].y) N--;//pour ne pas dupliquer de point (risque d'arriver quand pas<1)
		if (N>0) if (pts[N].x==pts[N-1].x) if (pts[N].y==pts[N-1].y) N--;//pour ne pas dupliquer de point (risque d'arriver quand pas<1)
		
		pts[N+1].y= (int)I;
		pts[N+1].x= int ( (-B*Y + delta)*inv2A + centre.x );
		
		if ( pts[N+1].x<0 || pts[N+1].x>w ) N--;//si on sort de l'image, on supprime le point
		if (N>0) if (pts[N+1].x==pts[N-1].x) if (pts[N+1].y==pts[N-1].y) N--;//pour ne pas dupliquer de point (risque d'arriver quand pas<1)
		if (N>=0) if (pts[N+1].x==pts[N].x) if (pts[N+1].y==pts[N].y) N--;//pour ne pas dupliquer de point (risque d'arriver quand pas<1)

		N+=2;
	}
	
	/* extrémité basse */
	pts[N].y = (int)D+centre.y;//extrémité basse (y le plus grand)
	pts[N].x = int ( (-B*(pts[N].y-centre.y))*inv2A + centre.x );
	if (pts[N].y<0) return 0;//si on est trop haut pour le dernier point, alors aucun point ne sera a l'interieur de l'image
	if (pts[N].y>h) N--;//si on sort de l'image, on supprime le point
	else if ( pts[N].x<0 || pts[N].x>w ) N--;//si on sort de l'image, on supprime le point. Le else est la pour empecher de faire -- deux fois

	return N+1;
}


/* Discrétisation d'un cercle : 
 *
 * centre, rayon : caractéristiques du cercle
 * nb_pt : nombre de points a obtenir
 * retourne le nombre de points effectivement créés
 */
int evaluation::discretise_cercle(CvPoint centre, int rayon, int nb_pt) {

	pas = 6.283/(float)nb_pt;//6.283 ~= 2*pi

	/* extrémité haute */
	pts[0].x = centre.x;
	pts[0].y = centre.y - rayon;//extrémité haute (y le plus petit)

	N=1;
	if (pts[0].y>h) return 0;//si on est déja trop bas pour pts[0], alors aucun point ne sera a l'interieur de l'image
	if (pts[0].y<0) N--;//si pts[0] sort de l'image (trop haut), on le supprime
	else if ( pts[0].x<0 || pts[0].x>w ) N--;//si on sort de l'image, on supprime le point. Le else est la pour empecher de faire -- deux fois
	
	/* on parcoure le cercle suivant theta *///theta en radians
	for (theta=pas ; theta<6.280 ; theta+=pas) {//6.280 = un peu moins que 2*pi
		
		pts[N].x = centre.x + (int)(rayon*sin(theta));
		pts[N].y = centre.y - (int)(rayon*cos(theta));
		
		if (pts[N].y<0) continue;//si on sort de l'image (on est trop haut), on supprime le point et on passe au suivant
		if (pts[N].y>h) return N;//si on sort de l'image (on est trop bas), il n'y aura plus d'autres points
		
		if (pts[N].x<0 || pts[N].x>w) continue;//si on sort de l'image, on supprime le point

		N++;
	}

	return N;
}



