/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef GEST_PARAMS_H
#define GEST_PARAMS_H

/** gestParams.h
 * 
 * Ce fichier contient les paramètres par défaut du traqueur.
 * 
 * Toute nouvelle variable globale doit être déclarée en extern dans la première partie du fichier, puis déclarée normalement et initialisée dans la seconde.
 * 
 * les distances sont exprimées en mm et les angles en degrés
 * 
 * \author BURGER Brice
 * \since 01/2007
 */

#include <string>
using std::string;
#include <cv.h>
#include <highgui.h>
#include <iostream>

/* inclusion des autres fichiers de paramètres */
#include "detectionParams.h"



#define nbvarEllipsoide 9 //nombre de variables d'une ellipsoide 3D (3 translations, taille des demi-axes, 3 angle)=(x, y, z, ax, ay, az, alpha, betha, gamma)
#define nbvarSphere 3 //nombre de variables d'une sphère (3 translations)=(x, y, z) (le rayon est fixe)
#define INTERDICTIONS_MAX 10 //nombre de zones interdites maximum
#define CHEMIN_SAUV_DEFAUT "/tmp/gestImages/"
#define NIVEAU_REINIT_MAX 4


/** état du système */
typedef enum ETAT_GEST
{
	RIEN=0,			//rien n'est traqué
	TOUT=1,			//tout est traqué
	PASmAINgAUCHE=2,	//tout est traqué sauf la main gauche
	PASmAINdROITE=3,	//tout est traqué sauf la main droite
	AUCUNEmAIN=4,		//seule la tête est traquée
	PAStETE=5,		//tout est traqué sauf la tête
	PAStETEnImAINgAUCHE=6,	//seule la main droite est traquée
	PAStETEnImAINdROITE=7	//seule la main gauche est traquée
}ETAT_GEST;

/** type de particule */
typedef enum TYPE_PARTICULE_GEST
{
	TETE=0,
	MAINgAUCHE=1,
	MAINdROITE=2
}TYPE_PARTICULE_GEST;


/** structure contenant les infos importantes récupérées de pom */
typedef struct RECUP_POMPOS_GEST
{
	double sensorToMain[6];
	double mainToOrigin[6];
}RECUP_POMPOS_GEST;


#ifndef GLOBALE //si on n'est pas dans le main ou dans le RunTaskCodels, les variables sont déclarées externes

/* paramètres du filtre particulaire */
extern int nbpart;//nombre de particules du filtre
extern float prop;//Proportion de particules choisies d'après la détection
extern int distMaxEval;//distance maximale (en pixels) de recherche autour d'une ellipse pour son évaluation
extern int nbPtEchantillonnageEllipse;//nb de points de l'échantillonnage d'une ellipse
extern int nbPtEchantillonnageCercle;//nb de points de l'échantillonnage d'un cercle

/* matrices de variance *///i.e. distance à laquelle on multiplie une gaussienne pour obtenir la marche aléatoire
extern int VARDynellipsoide[nbvarEllipsoide];//en mm : en fait on aura une evolution comprise dans +-2.5*valeur (parcequ'on fait 3 sigma)
extern int VARDetectellipsoide[nbvarEllipsoide];//le 0 est interdit!!!
extern int VARDynsphere[nbvarSphere];//idem pour une sphère
extern int VARDetectsphere[nbvarSphere];//idem pour une sphère

/* rayon de la tête */
extern int RayonTete;

/* rayon des interdictions */
extern int RayonInterdictionT;
extern int RayonInterdictionM;

/* option : si on calcule l'image de distance ou non *///l'image de distance aide pour avoir une bonne forme, mais prend du temps
extern bool CalculeImgDist;

/* paramètres des mesures */
extern double sigmamvt;
extern double sigmavisage;
extern double tauxMAJcoul;//taux de mise à jour des histogrammes de couleur
extern double sigmacouleur;
extern double sigmacontour;

/* infos importantes récupérées de pom */
extern RECUP_POMPOS_GEST recupPompos;



#else/*GLOBALE*/ //si on est dans le main ou dans le RunTaskCodels, les variables sont déclarées et initialisées



/* paramètres du filtre particulaire */
int nbpart = 100;//nombre de particules du filtre
float prop = 0.4;//Proportion de particules choisies d'après la détection
int distMaxEval = 10;//distance maximale (en pixels) de recherche autour d'une ellipse pour son évaluation
int nbPtEchantillonnageEllipse = 20;//nb de points de l'échantillonnage d'une ellipse
int nbPtEchantillonnageCercle = 20;//nb de points de l'échantillonnage d'un cercle

/* matrices de variance *///i.e. distance à laquelle on multiplie une gaussienne pour obtenir la marche aléatoire
int VARDynellipsoide[nbvarEllipsoide] = {70,70,70 , 30,30,30 , 10,10,10};//en mm : en fait on aura une evolution comprise dans +-2.5*valeur (parcequ'on fait 3 sigma)
int VARDetectellipsoide[nbvarEllipsoide] = {7,7,7 , 3,3,3 , 1,1,1};//le 0 est interdit!!!
int VARDynsphere[nbvarSphere] = {70,70,70};//en mm : en fait on aura une évolution comprise dans +-2.5*valeur (parcequ'on fait 3 sigma)
int VARDetectsphere[nbvarSphere] = {7,7,7};//le 0 est interdit!!!

/* rayon de la tête */
int RayonTete = 140;//en mm

/* rayon des interdictions */
int RayonInterdictionT = RayonTete;
int RayonInterdictionM = 100;

/* option : si on calcule l'image de distance ou non *///l'image de distance aide pour avoir une bonne forme, mais prend du temps
bool CalculeImgDist = true;

/* paramètres des mesures */
double sigmamvt = 0.2;
double sigmavisage = 40000;//(20 cm)² = (200 mm)² = 40000
double tauxMAJcoul = 0.01;//taux de mise à jour des histogrammes de couleur
double sigmacouleur = 0.04;
double sigmacontour = 10;

/* infos importantes récupérées de pom */
RECUP_POMPOS_GEST recupPompos;


#endif/*GLOBALE*/

#endif
