/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/* IDMOT.cpp
 * 
 * Ce fichier contient l'implémentation de l'algorithme IDMOT et de toutes les améliorations construites autour.
 * 
 * Auteur : BURGER Brice
 * Date : 04/2007
 */
#include "IDMOT.h"


//...!!! pour l'instant, le facteur de déséquilibre historique a été supprimé, mais il mérite de nouvelles évaluations. Pour le remettre en place :
//...!!! changer les /*!!!*/ en //!!!!!!//
//...!!! et les //!!!// en /*!!!*/


static int **particulesI, **particulesJ;//tableau des particules générées ce tour pour la cible
static int *particuleFinaleI, *particuleFinaleJ;//tableau contenant la particule finale de la cible (en réalité celle du tour précédent, puisqu'on est au tout début du pas)
static int *particuleFinalePrecI, *particuleFinalePrecJ;//tableau contenant la particule finale précédente de la cible (en réalité celle du tour deux fois précédent, puisqu'on est au tout début du pas)

static float phi1;//"fonction de répulsion magnétique"
static float phi2;//"fonction d'inertie"
static float bethahistI, bethahistJ;//déséquilibre créé en fonction du respect de la dynamique : la cible qui la respecte le mieux sera moins affectée par phi1 et phi2
static int dIJcarre;//distance entre l'estimée temporaire de l'objet i et l'estimée temporaire de l'objet j
//...!!! static int dIpreccarre, dJpreccarre;//distance entre l'estimée temporaire de l'objet (resp. i et j) et la particule finale de l'objet (resp. i et j) du tour précédent//...!!!rajout maison

static int** V;//vecteurs vitesse du tour précédent
static int* Vnorme;//normes des vecteurs de V

static bool recalculegmax;

static int dncarre;//distance entre la iième particule de la cible I et l'estimée temporaire de la cible J

static int* Vn;//vecteur vitesse de la particule en cours
static int Vnormen;//norme de Vn
static float theta;//angle entre le vecteur vitesse de la particule en cours et celui du tour précédent
static int deltaVcarre;//différence de norme entre le vecteur de vitesse de la particule en cours et celle du tour précédent

/* çà a sa place dans params.h je pense */
static float unsurbetha1 = 0.75;//terme de normalisation de phi1
static float unsurbetha2 = 0.5;//terme de normalisation de phi2
//...!!! static float unsurbetha3 = 1;//terme de normalisation de phi3
static int K = 4;//nb d'itérations du correcteur
static float thetaMAXcarre = 2.4;//en radians (au carré) ~=(pi/2)²//...
static int deltaVMAXcarre = 200*200;//en mm (au carré)//...
static int dIJMAXcarre = 3* (70*4 * 70*4);//l'influence commence à cette distance//...cette distance devrait être liées aux matrices de variance : 
					//...dMAXcarre = (dist max parcourable par une particule)² = (3*sigma1)² + (3*sigma2)² + (3*sigma3)²;
static int dMAXcarre = dIJMAXcarre/16;//en mm (au carré)

static evaluation *eval;



/* Cette fonction initialise l'espace de travail :
 * 
 * nbcibles : nombre total de cible qu'il est possible de suivre
 */
void initIDMOT(int nbcibles) {

	Vn = new int[3];
	Vnorme = new int[nbcibles];
	V = new int*[nbcibles];
	for (int i=0 ; i<nbcibles ; i++) V[i] = new int[3];
	eval = new evaluation();
}


/* Cette fonction relache l'espace de travail :
 * 
 * nbcibles : nombre total de cible qu'il est possible de suivre
 */
void finIDMOT(int nbcibles) {

	for (int i=0 ; i<nbcibles ; i++) delete[] V[i];
	delete[] Vn;
	delete[] Vnorme;
	delete[] V;
	delete eval;
}




/* Cette fonction est un préambule à l'algorithme IDMOT : 
 * elle calcule les vecteurs vitesse des cibles suivies ainsi que leurs normes
 * elle doit être éxécutée en tout début d'un nouveau pas (avant que la particule finale du tour deux fois précédent n'ai disparue)
 * 
 * cibles : liste des poids non normalisés des particules de la première cible
 * nbcibles : tableau des particules générées ce tour pour la première cible
 * suivi : tableau de booléen indiquant pour chaque cible si elle est suivie ou non
 * 
 * elle renvoie les vitesses calculées (les normes via Vnorme et les vecteurs via V)
 */
void preambuleIDMOT(Particules** cibles, int nbcibles, bool* suivi) {
	
	for (int i=0 ; i<nbcibles ; i++) {
		if (suivi[i]) Vnorme[i] = calculVitesse(cibles[i]->particuleFinale, cibles[i]->particuleFinalePrec, V[i]);
		else Vnorme[i] = 0;//si on ne connait pas particuleFinale, on ne peut pas calculer de vitesse ni tout ce qui y est lié
	}
}


/* Cette fonction est l'implémentation de l'algorithme IDMOT : 
 * elle influence le poids des particules afin d'empécher la fusion ou l'échange de ces cibles
 * elle doit être éxécutée après un nouveau pas, après la sélection, mais avant le rééchantillonnage
 * 
 * cibles : liste des poids non normalisés des particules de la première cible
 * nbcibles : tableau des particules générées ce tour pour la première cible
 * suivi : tableau de booléen indiquant pour chaque cible si elle est suivie ou non
 * 
 * suivi est également mis à jour en fonction des modifications apportées par l'algorithme
 */
void IDMOT(Particules** cibles, int nbcibles, bool* suivi, double** g, double* gmax) {
	
	for (int i=0 ; i<nbcibles-1 ; i++) for (int j=i+1 ; j<nbcibles ; j++) //pour chaque paire de cibles (i!=j)
		if (suivi[i]&suivi[j]) IDMOTdeuxcibles(cibles[i], g[i], &gmax[i], &suivi[i], Vnorme[i], V[i], 
							cibles[j], g[j], &gmax[j], &suivi[j], Vnorme[j], V[j]);
}






/**********************/
/* fonctions internes */
/**********************/

/* Ceci est une sous-fonction du préambule à l'algorithme IDMOT : 
 * elle calcule le vecteur vitesse d'une cible ainsi que sa norme
 * 
 * particuleFinale : tableau contenant la particule finale (en réalité celle du tour précédent, puisqu'on est au tout début du pas)
 * particuleFinalePrec : tableau contenant la particule finale précédente (en réalité celle du tour deux fois précédent, puisqu'on est au tout début du pas)
 * 
 * on renvoie la vitesse calculée (la norme directement, le vecteur via V)
 */
int calculVitesse(int* particuleFinale, int* particuleFinalePrec, int* V) {

	//en fait, on ne calcule pas une vitesse, mais en supposant que le temps d'aquisition est à peu près constant, çà revient au même pour la suite
	V[0] = particuleFinale[0] - particuleFinalePrec[0];
	V[1] = particuleFinale[1] - particuleFinalePrec[1];
	V[2] = particuleFinale[2] - particuleFinalePrec[2];
	return (int) sqrt( V[0]*V[0] + V[1]*V[1] + V[2]*V[2] );
}


/* Cette fonction est l'implémentation de l'algorithme IDMOT pour deux cibles : (modifié)
 * elle influence le poids des particules afin d'empécher la fusion ou l'échange de ces cibles
 * c'est une sous-fonction de IDMOT()
 * 
 * modifications apportées par apport à l'algo d'origine :
 * - rajout d'un facteur de déséquilibre historique (on avantage une particule par rapport à l'autre en fonction de leur respect de l'historique)
 * - modif du facteur angulaire dans le calcul de phi2 (si la vitesse est faible, l'angle n'a que peu d'importance)
 * 
 * cibleI : première cible à traiter
 * gI : liste des vraissemblances non normalisés des particules de la première cible
 * gmaxI : maximum des gI (son adresse en fait, pour pouvoir le modifier)
 * VI : vecteur vitesse voulue pour la cible I (vitesse du tour précédent)
 * VnormeI : norme de VI
 * 
 * cibleJ : seconde cible à traiter
 * gJ : liste des vraissemblances non normalisés des particules de la seconde cible
 * gmaxJ : maximum des gJ (son adresse en fait, pour pouvoir le modifier)
 * VJ : vecteur vitesse voulue pour la cible J (vitesse du tour précédent)
 * VnormeJ : norme de VJ
 * 
 * on renvoie CI et CJ qui indiquent si les cibles sont encore suivies (des addresses doivent être données en entrée)
 */
void IDMOTdeuxcibles(Particules* cibleI, double* gI, double *gmaxI, bool* CI, int VnormeI, int* VI, 
			Particules* cibleJ, double* gJ, double *gmaxJ, bool* CJ, int VnormeJ, int* VJ) {

	/* cible I */
	particulesI = cibleI->particules;//tableau des particules générées ce tour pour la cible
	particuleFinaleI = cibleI->particuleFinale;//tableau contenant la particule finale de la cible (en réalité celle du tour précédent, puisqu'on est au tout début du pas)
	particuleFinalePrecI = cibleI->particuleFinalePrec;//tableau contenant la particule finale précédente de la cible (en réalité celle du tour deux fois précédent, puisqu'on est au tout début du pas)

	/* cible J */
	particulesJ = cibleJ->particules;//tableau des particules générées ce tour pour la cible
	particuleFinaleJ = cibleJ->particuleFinale;//tableau contenant la particule finale de la cible (en réalité celle du tour précédent, puisqu'on est au tout début du pas)
	particuleFinalePrecJ = cibleJ->particuleFinalePrec;//tableau contenant la particule finale précédente de la cible (en réalité celle du tour deux fois précédent, puisqu'on est au tout début du pas)

	/* calcul de la distance séparant les différentes particules finales temporaires et correction des poids si nécessaire */
	dIJcarre = (particuleFinaleI[0]-particuleFinaleJ[0])*(particuleFinaleI[0]-particuleFinaleJ[0])
			+ (particuleFinaleI[1]-particuleFinaleJ[1])*(particuleFinaleI[1]-particuleFinaleJ[1])
			+ (particuleFinaleI[2]-particuleFinaleJ[2])*(particuleFinaleI[2]-particuleFinaleJ[2]);

	if (dIJcarre < dIJMAXcarre) {//si les deux objets sont proches, on corrige le poids pour tenir compte d'une "force de répulsion magnétique"
	
//printf("dIJcarre(%d) < dIJMAXcarre(%d)\n",dIJcarre, dIJMAXcarre);
//printf("particuleI : %d %d %d\n",particuleFinaleI[0],particuleFinaleI[1],particuleFinaleI[2]);
//printf("particuleJ : %d %d %d\n",particuleFinaleJ[0],particuleFinaleJ[1],particuleFinaleJ[2]);
//printf("particuleFinalePrecI : %d %d %d\n",particuleFinalePrecI[0],particuleFinalePrecI[1],particuleFinalePrecI[2]);
//printf("particuleFinalePrecJ : %d %d %d\n",particuleFinalePrecJ[0],particuleFinalePrecJ[1],particuleFinalePrecJ[2]);
	
		/* pour chaque particule */
		for(int i=0;i<nbpart;i++) {
			
			//ce facteur n'apporte pas grand chose, entend le virer -> pas sûr! faut refaire des évaluations sur des données diverses
			//!!!///* calcul du facteur de déséquilibre historique */
			//!!!//dIpreccarre = (particulesI[i][0]-particuleFinalePrecI[0])*(particulesI[i][0]-particuleFinalePrecI[0])
			//!!!//	+ (particulesI[i][1]-particuleFinalePrecI[1])*(particulesI[i][1]-particuleFinalePrecI[1])
			//!!!//	+ (particulesI[i][2]-particuleFinalePrecI[2])*(particulesI[i][2]-particuleFinalePrecI[2]);
			//!!!//dJpreccarre = (particulesJ[i][0]-particuleFinalePrecJ[0])*(particulesJ[i][0]-particuleFinalePrecJ[0])
			//!!!//	+ (particulesJ[i][1]-particuleFinalePrecJ[1])*(particulesJ[i][1]-particuleFinalePrecJ[1])
			//!!!//	+ (particulesJ[i][2]-particuleFinalePrecJ[2])*(particulesJ[i][2]-particuleFinalePrecJ[2]);
			//!!!//bethahistI = unsurbetha3 * exp(-dJpreccarre/(float)dIpreccarre);
			//!!!//bethahistJ = unsurbetha3 * exp(-dIpreccarre/(float)dJpreccarre);
			//printf("bethahist= %3.3f / %3.3f\n",bethahistI,bethahistJ);
			
			for (int k=0 ; k<K ; k++) {
				
				/* cible I */
				phi1 = Phi1(particulesI[i], particuleFinaleJ, bethahistI);
				if (VnormeI!=0) phi2 = Phi2(particulesI[i], particuleFinalePrecI, bethahistI, VnormeI, VI);
				else phi2 = 1;
				*CI = correction(cibleI, gI, i, gmaxI, phi1, phi2);
				
//cout << "I : phi1=" << phi1 << "phi2 = " << phi2 << endl;
				
				/* cible J */
				phi1 = Phi1(particulesJ[i], particuleFinaleI, bethahistJ);
				if (VnormeJ!=0) phi2 = Phi2(particulesJ[i], particuleFinalePrecJ, bethahistJ, VnormeJ, VJ);
				else phi2 = 1;
				*CJ = correction(cibleJ, gJ, i, gmaxJ, phi1, phi2);
				
//cout << "J : phi1=" << phi1 << "phi2 = " << phi2 << endl;
				
				if (!*CI || !*CJ) return;//si on perd une cible, continuer n'a plus de sens
			}
		}
	}
}


/* Cette fonction applique une correction calculée à une particule et à sa cible : 
 * elle modifie dans un premier temps la vraissemblance de la particule
 * elle modifie ensuite le poids des particules et fait une sélection
 * 
 * cible : cible à traiter
 * g : vraissemblances des particules de la cible
 * i : numéro de la particule à corriger
 * gmax : maximum des vraissemblances des particules de la cible (l'adresse de la variable, en fait, pour pouvoir la modifier)
 * phi1 : correction par la "fonction de répulsion magnétique"
 * phi2 : correction par la "fonction d'inertie"
 * 
 * retourne si la cible reste suivie ou non (résultat de la sélection)
 */
bool correction(Particules* cible, double* g, int i, double *gmax, float phi1, float phi2) {
	
	/* correction */
	if (*gmax==g[i]) recalculegmax = true;//s'il s'agissait du maximum, il faut peut-être recalculer gmax
	else recalculegmax = false;
	g[i] *= phi1*phi2;
	//!!!//printf("phi1*phi2 = %3.3f (g=%3.3f)\n",phi1*phi2,g[i]);
	if (recalculegmax) {
		if (*gmax>g[i]) {//le maximum n'est plus vrai, il faut le retrouver
			*gmax = 0;
			for(int j=0;j<nbpart;j++) if (*gmax<g[j]) *gmax=g[j];
		}
		else *gmax=g[i];//le maximum a simplement augmenté
	}
	else if (*gmax<g[i]) {//g[i] devient le nouveau maximum
		*gmax=g[i];
		recalculegmax = true;
	}
	
	/* nouveau poids, nouvelle particule finale */
	if (recalculegmax) for(int j=0;j<nbpart;j++) cible->poids[j] = eval->poids(g[j],*gmax);//si gmax a été modifié, il faut recalculer tous les poids!
	return cible->selectionne();
}


/* phi1 : "fonction de répulsion magnétique" : 
 * elle est fonction de la distance entre la iième particule de la cible I et la particule finale de la cible J
 * 
 * particuleIi : iième particule de la cible I
 * particuleFinaleJ : particule finale de la cible J
 * bethahistI : fonction de déséquilibre historique (rajout par rapport à IDMOT de base)
 * 
 * retourne la correction a appliquer
 */
float Phi1(int *particuleIi, int *particuleFinaleJ, float bethahistI) {

	/* calcul de la distance I[i]/J */
	dncarre= (particuleIi[0]-particuleFinaleJ[0])*(particuleIi[0]-particuleFinaleJ[0])
		+ (particuleIi[1]-particuleFinaleJ[1])*(particuleIi[1]-particuleFinaleJ[1])
		+ (particuleIi[2]-particuleFinaleJ[2])*(particuleIi[2]-particuleFinaleJ[2]);
	/*!!!*/return ( 1 -  unsurbetha1 * exp(-dncarre/dMAXcarre) );//IDMOT de base
	//!!!//return ( 1 - bethahistI * (unsurbetha1 * exp(-dncarre/dMAXcarre)) );//IDMOT + facteur historique
}


/* phi2 : "fonction d'inertie" : 
 * elle est fonction de la distance entre de la continuité en vitesse et en angle de la particule à traiter
 * 
 * particulei : iième particule de la cible à traiter
 * particuleFinalePrec : particule finale de la cible à traiter
 * bethahist : fonction de déséquilibre historique (rajout par rapport à IDMOT de base)
 * Vnorme : norme du vecteur vitesse du tour précédent
 * V : vecteur vitesse du tour précédent
 * 
 * retourne la correction a appliquer
 */
float Phi2(int *particulei, int *particuleFinalePrec, float bethahist, int Vnorme, int* V) {
	
	/* calcul des vecteurs vitesse */
	Vn[0] = particulei[0] - particuleFinalePrec[0];
	Vn[1] = particulei[1] - particuleFinalePrec[1];
	Vn[2] = particulei[2] - particuleFinalePrec[2];
	
	/* calcul de la norme des vecteurs vitesses */
	Vnormen = (int) sqrt( Vn[0]*Vn[0] + Vn[1]*Vn[1] + Vn[2]*Vn[2] );
	
	/* calcul des différences de vitesses */
	deltaVcarre = (Vnorme - Vnormen) * (Vnorme - Vnormen);
	
	/* calcul de l'angle entre les vecteurs vitesse */
	if (Vnorme*Vnormen==0) theta = 0;//très rare, mais si deux images identiques se suivent çà deviens possible
	else theta = acos( (Vn[0]*V[0] + Vn[1]*V[1] + Vn[2]*V[2]) /  (Vnorme*Vnormen) );
	
	//return 1;
	//return ( unsurbetha2 * exp(-theta*theta/thetaMAXcarre) * exp(-deltaVcarre/deltaVMAXcarre) );//IDMOT de base
	/*!!!*/return ( 1 + unsurbetha2 * exp(-theta*theta/thetaMAXcarre * Vnorme*Vnorme/deltaVMAXcarre) * exp(-deltaVcarre/deltaVMAXcarre) );//IDMOT légèrement modifié
	//!!!//return ( 1 + (1-bethahist) * unsurbetha2 * exp(-theta*theta/thetaMAXcarre * Vnorme*Vnorme/deltaVMAXcarre) * exp(-deltaVcarre/deltaVMAXcarre) );//IDMOT légèrement modifié + facteur historique
}










