/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

#ifndef TRAQUEUR_H
#define TRAQUEUR_H

/** traqueur.h
 * 
 * Programme principal, il fait l'interface entre le main ou genom et le reste du code.
 * Ici sont implémentés les fonctions de haut niveau du traqueur 3D à filtre particulaire.
 * Le but de ce programme est de traquer une main dans l'espace (forme ellipsoidale) à partir des images d'un banc stéréo.
 * 
 * Il a été modifié pour suivre en priorité une tête, dont il déduit la main. Les modification aportées sont annotées avec //§§§ .
 * 
 * ATTENTION : ce fichier n'est plus à jour! (il devrait même être supprimé)
 * 
 * \author BURGER Brice
 * \since 01/2007
 */


#include "particules.h"
#include "evaluation.h"
#include "detectionBlobPeau.h"
#include "detectionVisage.h"
#include "gestParams.h"
#include "imgdisplay.h"




class Traqueur
{

	public:
	
	/**************************************/
	/* fonctions générales :              */
	/**************************************/
	
	/** constructeur du traqueur 3D :
	 * 
	 * \param imgG et imgD : images représentatives de celles qui serviront ensuite durant la traque (même taille, même convention de couleur, ...)
	 */
	Traqueur(IplImage* imgG, IplImage* imgD);


	/** destructeur du traqueur 3D :
	 *
	 */
	~Traqueur();
	
	
	/** fonction d'initialisation globale du traqueur :
	 * 
	 * \param imgG,imgD : images à traiter (gauche et droite)
	 * \param type : type d'initialisation :
	 * \param -	**0 : renvoie le premier blob couleur peau détecté
	 * \param -	**1 : avec zone de recherche précisée (si l'on connait une position approximative de la main à traquer)
	 * \param -	**2 : ici on initialise sur la main droite à partir d'une détection de tête
	 * \param -		La main droite est un blob peau situé à moins de 1.5 m de la tête et se situant à gauche de celle-ci (et en bas de la tête).
	 * \param -		ATTENTION : il ne doit y avoir qu'un seul visage !! Sinon c'est le premier qui est pris.
	 * \param x,y,z,rayon : cube dans lequelle on cherche un blob peau pour le type 1
	 * 
	 * \return retourne la particule initiale trouvée ou NULL si on a rien trouvé
	 */
	double* INIT(IplImage* imgG, IplImage* imgD, int type, double x, double y, double z, double rayon);


	/** fonction de réinitialisation du traqueur : 
	 * 
	 * \param avantSelection : précise si la sélection a déjà eu lieu ou non (pour savoir si la particuleFinale à déjà été écrasée ou non)
	 * 
	 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
	 * 
	 * \return retourne la particule initiale trouvée ou NULL si on a rien trouvé
	 */
	double* REINIT(IplImage* imgG, IplImage* imgD, bool avantSelection);
	
	
	/** fonction de réinitialisation du traqueur (tête) : 
	 * 
	 * \param avantSelection : précise si la sélection a déjà eu lieu ou non (pour savoir si la particuleFinale à déjà été écrasée ou non)
	 * 
	 * Ici on essaie de sauver un pas défectueux pour éviter une réinitialisation globale (via INIT) faisant perdre la continuité de la traque.
	 * 
	 * \return retourne true si la réinitialisation s'est bien passée, false autrement
	 */
	bool REINITtETE(IplImage* imgG, IplImage* imgD, bool avantSelection);
	
	
	/** fonction évaluant une particule via les fonctions d'évaluation 2D :
	 * 
	 * \param particule : la particule à évaluer
	 * 
	 * \return renvoie son évaluation (entre 0 et 1)
	 */
	double EVAL(double *particule);
	
	/** fonction évaluant une particule via les fonctions d'évaluation 2D :
	 * 
	 * \param particule : la particule à évaluer
	 * 
	 * \return renvoie son évaluation (entre 0 et 1)
	 */
	double EVALtete(double *particule);
	
	
	/** pour avancer d'un pas dans le filtre particulaire :
	 * 
	 */
	double* PAS(IplImage* imgG, IplImage* imgD);
	
	
	
	/*****************************************/
	/* fonctions d'affichage :               */
	/*****************************************/

	/** affichage sur l'ecran du robot : 2 images l'une en dessous de l'autre */
	void affiche (IplImage* imgg, IplImage* imgd);


	/** affichage sur l'ecran du robot : 3 images l'une en dessous de l'autre */
	void affiche (IplImage* imgg, IplImage* imgd, IplImage* img3);
	
	
	
	/*****************************************/
	/* fonctions de modification d'obtions : */
	/*****************************************/
	
	/**
	 * fonctions pour changer la matrice de covariance de la dynamique
	 * 
	 * \param typeParticule : type de particule qu'on veut affecter par cette modification
	 * \param cov : nouvelle matrice de covariance (ATTENTION : la taille de la matrice donnée doit correspondre à celle modifiée)
	 * 
	 * \return retourne true s'il n'y a pas eu de problème, false autrement
	 */
	bool changecov(TYPE_PARTICULE_GEST typeParticule, double *cov);
	
	/** si l'on veut utiliser une interface graphique :
	 * 
	 */
	void avec_interface(bool oui);

	/** si l'on veut afficher les particules :
	 * 
	 */
	void affiche_particules(bool oui);

	/** si l'on veut sauvegarder les images obtenues :
	 * 
	 */
	void sauvegarde(bool oui);
	
	
	private :
	
	/**************************************/
	/* variables internes :               */
	/**************************************/
	
	/* images temporaires pour le calcul */
	IplImage* imgTmpg2;
	IplImage* imgTmpd2;
	
	/* images temporaires pour l'affichage */
	IplImage* imgTmpg;
	IplImage* imgTmpd;
	IplImage* img;
	ImgDisplay* afficheur;//pour afficher une image sur l'ecran du robot
	CvRect* rect;//rectangle qui nous intéresse
	IplImage *imageNG[3];
	
	/* adresse temporaire */
	double* temp;
	
	/* pour sauvegarder les images */
	char* nomsauvegarde;
	int numero;//pour sauvegarder des images
	
	/* particules */
	Particules *particules;
	Particules *particulesTETE;
	
	/* détecteurs */
	detectionBlobPeau *detecteurBlob;
	detectionVisage *detecteurVisage;
	/* drapeau d'options : */
	bool AVEC_INTERFACE;//avec ou sans interface graphique
	bool AFFICHE_PARTICULES;//si on affiche les particules ou non
	bool SAUVEGARDE;//si on sauvegarde les images ou non
	/* tableaux de visages */
	listeRect rectG;
	listeRect rectD;
	/* pour la triangulation de points */
	flottant **p1, **p2;
	
	/* pour la projection */
	double* projection;
	CvPoint point2dgauche, point2ddroite;
	CvSize axesg, axesd;
	double angleg, angled;
	/* pour l'évaluation des particules */
	double *g, *gTETE;
	double gmax, gmaxTETE, lamda, lamdaTETE;

};
#endif
