/* Copyright (c) Mathias Fontmarty (LAAS-CNRS) */
#include "mathutils.h"
#include <iomanip>
#include <cmath>
#include <cstring>


C3DPoint::C3DPoint()
{
  C3DPoint(0,0,0,0,1,0);
}
C3DPoint::C3DPoint(flottant x, flottant y, flottant z, 
		   int id, flottant taille)
{
  C3DPoint(x,y,z,0,1,0,id,taille);
}
C3DPoint::C3DPoint(flottant x, flottant y, flottant z, 
		   flottant nx, flottant ny, flottant nz, 
		   int id, flottant taille)
{
  this->x = x;
  this->y = y;
  this->z = z;
  this->nx = nx;
  this->ny = ny;
  this->nz = nz;
  this->id = id;
  this->taille = taille;
}







void matrix4Display(flottant * mat, char* nom)
{
  matrixDisplay(mat,4,4,nom);
}

void matrixDisplay(flottant *mat, int nbLignes, int nbColonnes, char *nom)
{
  std::cout << nom << "\n";
  for (int j=0; j<nbLignes; j++)
    {
      for (int i=0; i<nbColonnes; i++)
	std::cout << std::setw(12) << mat[j*nbColonnes+i] << " ";
      std::cout << "\n";
    }
}



void matrix4Transpose(flottant *vOut, flottant *vIn)
{
  vOut[0] = vIn[0];
  vOut[1] = vIn[4];
  vOut[2] = vIn[8];
  vOut[3] = vIn[12];
  vOut[4] = vIn[1];
  vOut[5] = vIn[5];
  vOut[6] = vIn[9];
  vOut[7] = vIn[13];
  vOut[8] = vIn[2];
  vOut[9] = vIn[6];
  vOut[10] = vIn[10];
  vOut[11] = vIn[14];
  vOut[12] = vIn[3];
  vOut[13] = vIn[7];
  vOut[14] = vIn[11];
  vOut[15] = vIn[15];
  
}

void matrixTranspose(flottant *vOut, flottant *vIn, int dimIn1, int dimIn2)
{
  for (int j=0; j<dimIn1; j++)
    for (int i=0; i<dimIn2; i++)
      vOut[i*dimIn1+j] = vIn[j*dimIn2+i];
}


void matrix4Add(flottant *vOut, flottant *vIn1, flottant *vIn2)
{
  vOut[0] = vIn1[0] + vIn2[0];
  vOut[1] = vIn1[1] + vIn2[1];
  vOut[2] = vIn1[2] + vIn2[2];
  vOut[3] = vIn1[3] + vIn2[3];
  vOut[4] = vIn1[4] + vIn2[4];
  vOut[5] = vIn1[5] + vIn2[5];
  vOut[6] = vIn1[6] + vIn2[6];
  vOut[7] = vIn1[7] + vIn2[7];
  vOut[8] = vIn1[8] + vIn2[8];
  vOut[9] = vIn1[9] + vIn2[9];
  vOut[10] = vIn1[10] + vIn2[10];
  vOut[11] = vIn1[11] + vIn2[11];
  vOut[12] = vIn1[12] + vIn2[12];
  vOut[13] = vIn1[13] + vIn2[13];
  vOut[14] = vIn1[14] + vIn2[14];
  vOut[15] = vIn1[15] + vIn2[15];
}

void matrixAdd(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim1, int dim2)
{
  for (int j=0; j<dim1; j++)
    for (int i=0; i<dim2; i++)
      vOut[j*dim2+i] = vIn1[j*dim2+i]+vIn2[j*dim2+i];
}

void matrix4Mult(flottant *out, flottant *in1, flottant *in2)
{
  out[0] = in1[0]*in2[0] + in1[1]*in2[4] + in1[2]*in2[8] + in1[3]*in2[12];
  out[1] = in1[0]*in2[1] + in1[1]*in2[5] + in1[2]*in2[9] + in1[3]*in2[13];
  out[2] = in1[0]*in2[2] + in1[1]*in2[6] + in1[2]*in2[10] + in1[3]*in2[14];
  out[3] = in1[0]*in2[3] + in1[1]*in2[7] + in1[2]*in2[11] + in1[3]*in2[15];

  out[4] = in1[4]*in2[0] + in1[5]*in2[4] + in1[6]*in2[8] + in1[7]*in2[12];
  out[5] = in1[4]*in2[1] + in1[5]*in2[5] + in1[6]*in2[9] + in1[7]*in2[13];
  out[6] = in1[4]*in2[2] + in1[5]*in2[6] + in1[6]*in2[10] + in1[7]*in2[14];
  out[7] = in1[4]*in2[3] + in1[5]*in2[7] + in1[6]*in2[11] + in1[7]*in2[15];

  out[8] = in1[8]*in2[0] + in1[9]*in2[4] + in1[10]*in2[8] + in1[11]*in2[12];
  out[9] = in1[8]*in2[1] + in1[9]*in2[5] + in1[10]*in2[9] + in1[11]*in2[13];
  out[10] = in1[8]*in2[2] + in1[9]*in2[6] + in1[10]*in2[10] + in1[11]*in2[14];
  out[11] = in1[8]*in2[3] + in1[9]*in2[7] + in1[10]*in2[11] + in1[11]*in2[15];

  out[12] = in1[12]*in2[0] + in1[13]*in2[4] + in1[14]*in2[8] + in1[15]*in2[12];
  out[13] = in1[12]*in2[1] + in1[13]*in2[5] + in1[14]*in2[9] + in1[15]*in2[13];
  out[14] = in1[12]*in2[2] + in1[13]*in2[6] + in1[14]*in2[10] + in1[15]*in2[14];
  out[15] = in1[12]*in2[3] + in1[13]*in2[7] + in1[14]*in2[11] + in1[15]*in2[15];
}

void matrixMult(flottant *vOut, flottant *vIn1, flottant *vIn2, int vIn1dim1, int vIn1dim2, int vIn2dim2)
{
  memset(vOut,0,sizeof(flottant)*vIn1dim1*vIn2dim2);

  for (int i=0; i<vIn1dim1; i++)
    for (int j=0; j<vIn2dim2;j++)
      for (int k=0; k<vIn1dim2; k++)
	vOut[i*vIn2dim2+j] += vIn1[i*vIn1dim2+k]*vIn2[k*vIn2dim2+j];
}

void matrixInvert(flottant *inv, flottant *mat, int dim)
{
  flottant *siMat = new flottant[dim*2*dim];

  for(int i=0; i<dim; i++)
    for(int j=0; j<2*dim; j++)
      {
	if(j<dim) siMat[i*2*dim+j] = mat[i*dim+j];
	else siMat[i*2*dim+j] = 0;
	
	if(j==i+dim) siMat[i*2*dim+j] =1.0;
      }

  for(int i=0; i<dim-1; i++)
    for(int k=i+1; k<dim; k++)
      for(int j=2*dim-1; j>=i; j--)
	{
	  siMat[k*2*dim+j] -= siMat[i*2*dim+j] *
	    (siMat[k*2*dim+i]/siMat[i*2*dim+i]);
	}
         
  /* -------------------------------------*/

  for(int i=dim-1; i>0; i--)
    for(int k=i-1; k >=0; k--)
      for(int j=2*dim-1; j>=i; j--)
	{
	  siMat[k*2*dim+j] -= siMat[i*2*dim+j] *
	  (siMat[k*2*dim+i]/siMat[i*2*dim+i]);
	}

  for(int i=0; i<dim; i++)
    for(int j=2*dim-1; j>=0; j--)
      {
	siMat[i*2*dim+j] =siMat[i*2*dim+j]/
	  siMat[i*2*dim+i];
      }

  /* Recopie dans inv */
  for(int s=0; s<dim; s++)
    for(int r=0; r<dim; r++)
      {
	inv[s*dim+r] = siMat[s*2*dim+r+dim];
      }

  delete[] siMat;
}



flottant matrix3Determinant(flottant *mat)
{
  return 
    mat[0]*mat[4]*mat[8]
    + mat[3]*mat[7]*mat[2]
    + mat[6]*mat[1]*mat[5]
    - mat[2]*mat[4]*mat[6]
    - mat[5]*mat[7]*mat[0]
    - mat[8]*mat[1]*mat[3];

}










void matrix4VectorMult(flottant *vOut, flottant *mIn, flottant *vIn)
{
  vOut[0] = mIn[0]*vIn[0] + mIn[1]*vIn[1] + mIn[2]*vIn[2] + mIn[3]*vIn[3];
  vOut[1] = mIn[4]*vIn[0] + mIn[5]*vIn[1] + mIn[6]*vIn[2] + mIn[7]*vIn[3];
  vOut[2] = mIn[8]*vIn[0] + mIn[9]*vIn[1] + mIn[10]*vIn[2] + mIn[11]*vIn[3];
  vOut[3] = mIn[12]*vIn[0] + mIn[13]*vIn[1] + mIn[14]*vIn[2] + mIn[15]*vIn[3];
}

void matrix3VectorMult(flottant *vOut, flottant *mIn, flottant *vIn, int dim)
{
  vOut[0] = mIn[0]*vIn[0] +  mIn[1]*vIn[1] + mIn[2]*vIn[2];
  vOut[1] = mIn[dim]*vIn[0] +  mIn[dim+1]*vIn[1] + mIn[dim+2]*vIn[2];
  vOut[2] = mIn[2*dim]*vIn[0] +  mIn[2*dim+1]*vIn[1] + mIn[2*dim+2]*vIn[2];
}

/* Produit Vectoriel */
/*********************/
void vector3Mult(flottant *vOut, flottant *vIn1, flottant *vIn2)
{
  vOut[0] = vIn1[1]*vIn2[2] - vIn1[2]*vIn2[1];
  vOut[1] = -vIn1[0]*vIn2[2] + vIn1[2]*vIn2[0];
  vOut[2] = vIn1[0]*vIn2[1] - vIn1[1]*vIn2[0];
}



void vectorAdd(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim)
{
  for (int i=0; i<dim; i++)
    vOut[i] = vIn1[i]+vIn2[i];
}
void vectorSub(flottant *vOut, flottant *vIn1, flottant *vIn2, int dim)
{
  for (int i=0; i<dim; i++)
    vOut[i] = vIn1[i]-vIn2[i];
}













/* On cherche l'intersection entre les segments AB et CD 
 * cad le point P tel que:
 * Px = Ax + s*(Bx-Ax) 
 * Py = Ay + s*(By-Ay)
 * et
 * Px = Cx + t*(Dx-Cx)
 * Py = Cy + t*(Dy-Cy)
 * avec s et t appartenant a [0 1] 
 * Attention! on stocke 2 points d'intersection qui ont
 * les memes coordonnees x,y mais qui n'ont pas la meme
 * profondeur!!!!
 ********************************************************/
int segmentIntersect(S2DPoint *A, S2DPoint *B, S2DPoint *C, S2DPoint *D,  S2DPoint *pointIntersection)
{
  flottant denom = (B->x-A->x)*(D->y-C->y) - (B->y-A->y)*(D->x-C->x);
  
  if (denom==0.0) return 0; // AB parallele a CD
  
  flottant num = (D->x-C->x)*(A->y-C->y) - (D->y-C->y)*(A->x-C->x);

  if (num==0.0 || num==denom) return 0; // intersection sur une extremite
  flottant s = num/denom;

  num = (B->x-A->x)*(A->y-C->y) - (B->y-A->y)*(A->x-C->x);

  if (num==0.0 || num==denom) return 0; // intersection sur une extremite
  flottant t = num/denom;

  if (t>0 && t<1 && s>0 && s<1) 
    {
      pointIntersection->x = A->x + s*(B->x-A->x);
      pointIntersection->y = A->y + s*(B->y-A->y);
      pointIntersection->prof = A->prof + s*(B->prof-A->prof);
      pointIntersection->visible = 1;
      
      pointIntersection++;
      pointIntersection->x = A->x + s*(B->x-A->x);
      pointIntersection->y = A->y + s*(B->y-A->y);
      pointIntersection->prof = C->prof + t*(D->prof-C->prof);
      pointIntersection->visible = 1;

      return 1;
    }
  
  return 0;

}




void trierPointsParX(S2DPoint **tableauATrier, S2DPoint **tableauTrie, int tailleTableau)
{
  flottant minX;
  int indexMin;
  flottant epsilon = 0.5;

  for (int i=0; i<tailleTableau; i++)
    {
      indexMin = i;
      minX = tableauATrier[indexMin]->x;
      for (int j=i+1; j<tailleTableau; j++)
	{
	  //on trie en priorite suivant x
	  if (tableauATrier[j]->x<minX)
	    {
	      indexMin = j;
	      minX = tableauATrier[indexMin]->x;
	    }
	  //dans le cas ou les x sont egaux, on trie suivant y
	  else if (absd(tableauATrier[j]->x-minX)<epsilon)
	    {
	      if (tableauATrier[j]->y<tableauATrier[indexMin]->y)
		{
		  indexMin = j;
		}
	    }
	}

      tableauTrie[i] = tableauATrier[indexMin];
      tableauATrier[indexMin] = tableauATrier[i];
      tableauATrier[i] = tableauTrie[i];

    }
}



/*
 * Utility Fontion to clamp integer coords that are out
 * of an image 
 **********************************/
void clamp(int *toBeClamped, int infLimit, int supLimit)
{
  if (*toBeClamped<infLimit) *toBeClamped = infLimit;
  if (*toBeClamped>=supLimit) *toBeClamped = supLimit-1;
}



















/* Fonction permettant de faire la triangulation
 * d'un point repere sur 2 images stereo.
 * u,v,u0,v0,au,av parametres intrinseques des cameras
 * au = f*ku et av = f*kv
 * Mdg matrice de passage du repere de la camera 2 au
 * repere de la camera 1.
 ********************************************************/
void triangulate(int *pointTriangule, flottant *erreur,
		 int ug, int vg, flottant u0g, 
		 flottant v0g, flottant aug, flottant avg,
		 int ud, int vd, flottant u0d, 
		 flottant v0d, flottant aud, flottant avd,
		 flottant *Mdg,
		 flottant *covp1, flottant *covp2,
		 flottant *covP)
{

  /* 1 - Calcul des coordonnees des pixels sur les plans images en m */
  /*******************************************************************/
  flottant Ug = (ug-u0g)/aug;
  flottant Vg = (vg-v0g)/avg;

  flottant Ud = (ud-u0d)/aud;
  flottant Vd = (vd-v0d)/avd;

  /* 2 - Calcul des vecteurs directeurs des droites de vues en m */
  /***************************************************************/
  flottant V1[4] = {Ug,Vg,1,1};
  flottant V2[4] = {Ud,Vd,1,1};
  flottant V2p[4];

  matrix3VectorMult(V2p,Mdg,V2,4);

  /* 3 - Calcul de la normale aux 2 droites de vues */
  /**************************************************/
  flottant N[4] = {0,0,0,1};
  
  vector3Mult(N,V1,V2p);

  /* 4 - Recherche de H1, point d'intersection de P1 et V1  */
  /**********************************************************/

  /* 4.1 - Calcul des coeffs de l'equation du plan P1 (comportant V2p et N) */
  // NP1 = (A1,B1,C1)' = V2p^N
  flottant eq1[4] = {0,0,0,0};
  vector3Mult(eq1,V2p,N);

  // eq1[3] est calcule en appliquant l'equation du plan P1 
  // a un point du plan P1.
  // Ici on choisit ce point comme etant le centre du repere droit
  // D1 = -(A1*xd + B1*yd + C1*zd)
  eq1[3] = -(eq1[0]*Mdg[3] + eq1[1]*Mdg[7] + eq1[2]*Mdg[11]);

  /* 4.2 - Calcul des coords de H1, point d'intersection de P1 et V1 */
  // H1 = (U1*t,V1*t,t)' ou t=-D1/(Ug*A1+Vg*B1+C1)
  flottant h1[3];
  h1[2] = -eq1[3]/(Ug*eq1[0] + Vg*eq1[1] + eq1[2]); 
  h1[0] = h1[2]*Ug;
  h1[1] = h1[2]*Vg;


  /* 5 - Recherche de H2, point d'intersection de P2 et V2  */
  /**********************************************************/

  /* 5.1 - Calcul des coeffs de l'equation du plan P2 (comportant V1 et N) */
  flottant eq2[4] = {0,0,0,0};
  vector3Mult(eq2,V1,N);

  /* 5.2 - Calcul des coords de H2, point d'intersection de P2 et V2 */
  // H2 = (Ud*t,Vd*t,t)' dans le repere de droite
  // H2p = Mdg * H2 dans le repere de gauche
  // H2p = (a*t + Tdgx, b*t +Tdgy, c*t +Tdgz)'
  flottant a = Mdg[0]*Ud + Mdg[1]*Vd +Mdg[2];
  flottant b = Mdg[4]*Ud + Mdg[5]*Vd +Mdg[6];
  flottant c = Mdg[8]*Ud + Mdg[9]*Vd +Mdg[10];

  flottant t = -(eq2[0]*Mdg[3]+eq2[1]*Mdg[7]+eq2[2]*Mdg[11])/(eq2[0]*a+eq2[1]*b+eq2[2]*c);

  flottant h2p[3];
  h2p[0] = a*t + Mdg[3];
  h2p[1] = b*t + Mdg[7];
  h2p[2] = c*t + Mdg[11];

  /* Calcul du milieu de [H1 H2p] */
  pointTriangule[0] = int( 0.5*(h1[0]+h2p[0]) );
  pointTriangule[1] = int( 0.5*(h1[1]+h2p[1]) );
  pointTriangule[2] = int( 0.5*(h1[2]+h2p[2]) );

  if (erreur) *erreur = sqrt(pow(h1[0]-h2p[0],2)
		 + pow(h1[1]-h2p[1],2)
		 + pow(h1[2]-h2p[2],2));





  /* 6 - Calcul de la covariance le cas echeant */
  /**********************************************/
  /*
   * Idee:
   * P = F1 * covp1* F1' + F2 * covp2 * F2'
   * avec F1 matrice 2x3
   * et F2 matrice 2x3 
   * toutes 2 jacobiennes de la fonction de
   * triangulation f.
   **********************************************/
  if (covp1 && covp2 && covP)
    {
      flottant f1[6];
      flottant f2[6];

      /* 6.1 - calcul de f1 */
      /**********************/

      /* Derivees de A1 B1 C1 D1 */
      flottant da1du1 = (pow(V2p[1],2)+pow(V2p[2],2))/aug;
      flottant da1dv1 = -V2p[0]*V2p[1]/avg;

      flottant db1du1 = -V2p[0]*V2p[1]/aug;
      flottant db1dv1 = (pow(V2p[0],2)+pow(V2p[2],2))/avg;
      
      flottant dc1du1 = -V2p[0]*V2p[2]/aug;
      flottant dc1dv1 = -V2p[1]*V2p[2]/avg;

      flottant dd1du1 = -( Mdg[3]*(pow(V2p[1],2)+pow(V2p[2],2))
			   - Mdg[7]*V2p[0]*V2p[1]
			   - Mdg[11]*V2p[0]*V2p[2] )/aug;
      
      flottant dd1dv1 = -( - Mdg[3]*V2p[0]*V2p[1]
			   + Mdg[7]*(pow(V2p[0],2)+pow(V2p[2],2))
			   - Mdg[11]*V2p[1]*V2p[2] )/avg;

      
      /* Derivees de x1 y1 z1 coordonnees de H1 */
      flottant dz1du1 = ( -dd1du1*(V1[0]*eq1[0] + V1[1]*eq1[1] + eq1[2])
			  +eq1[3]*(eq1[0]/aug + da1du1*V1[0] + db1du1*V1[1] + dc1du1) );
      dz1du1 /= pow(V1[0]*eq1[0] + V1[1]*eq1[1] + eq1[2],2);

      flottant dz1dv1 = ( -dd1dv1*(V1[0]*eq1[0] + V1[1]*eq1[1] + eq1[2])
			  +eq1[3]*(da1dv1*V1[0] + eq1[1]/avg + db1dv1*V1[1] + dc1dv1) );
      dz1dv1 /= pow(V1[0]*eq1[0] + V1[1]*eq1[1] + eq1[2],2);

      flottant dx1du1 = dz1du1*V1[0] + h1[2]/aug;
      flottant dx1dv1 = dz1dv1*V1[0];

      flottant dy1du1 = dz1du1*V1[1];
      flottant dy1dv1 = dz1dv1*V1[1] + h1[2]/avg;

      f1[0] = 0.5 * dx1du1;
      f1[1] = 0.5 * dx1dv1;
      f1[2] = 0.5 * dy1du1;
      f1[3] = 0.5 * dy1dv1;
      f1[4] = 0.5 * dz1du1;
      f1[5] = 0.5 * dz1dv1;



      /* 6.2 - calcul de f2 */
      /**********************/

      flottant du2pdu2 = Mdg[0]/aud;
      flottant du2pdv2 = Mdg[1]/avd;
      flottant dv2pdu2 = Mdg[4]/aud;
      flottant dv2pdv2 = Mdg[5]/avg;
      flottant dw2pdu2 = Mdg[8]/aud;
      flottant dw2pdv2 = Mdg[9]/avd;

      /* Derivees de Nx Ny Nz */
      flottant dnxdu2 = V1[1]*dw2pdu2-dv2pdu2;
      flottant dnxdv2 = V1[1]*dw2pdv2-dv2pdv2;
      flottant dnydu2 = -V1[0]*dw2pdu2+du2pdu2;
      flottant dnydv2 = -V1[0]*dw2pdv2+du2pdv2;
      flottant dnzdu2 = V1[0]*dv2pdu2-V1[1]*du2pdu2;
      flottant dnzdv2 = V1[0]*dv2pdv2-V1[1]*du2pdv2;

      /* Derivees de A2 B2 C2 */
      flottant dA2du2 = V1[1]*dnzdu2 - dnydu2;
      flottant dA2dv2 = V1[1]*dnzdv2 - dnydv2;
      flottant dB2du2 = -V1[0]*dnzdu2 + dnxdu2;
      flottant dB2dv2 = -V1[0]*dnzdv2 + dnxdv2;
      flottant dC2du2 = V1[0]*dnydu2 - V1[1]*dnxdu2;
      flottant dC2dv2 = V1[0]*dnydv2 - V1[1]*dnxdv2;

      /* Derivees de a b c */
      flottant dadu2 = Mdg[0]/aud;
      flottant dadv2 = Mdg[1]/avd;
      flottant dbdu2 = Mdg[4]/aud;
      flottant dbdv2 = Mdg[5]/avd;
      flottant dcdu2 = Mdg[8]/aud;
      flottant dcdv2 = Mdg[9]/avd;

      /* Derivee de t */
      flottant dtdu2 = -( 
			 ( (Mdg[3]*dA2du2 + Mdg[7]*dB2du2 + Mdg[11]*dC2du2) 
			   * (eq2[0]*a+eq2[1]*b+eq2[2]*c) ) 
			 - ( (eq2[0]*Mdg[3]+eq2[1]*Mdg[7]+eq2[2]*Mdg[11]) 
			     * (dA2du2*a + dadu2*eq2[0] 
				+ dB2du2*b + dbdu2*eq2[1]
				+ dC2du2*c + dcdu2*eq2[2]) )
			 )
	/ pow((eq2[0]*a+eq2[1]*b+eq2[2]*c),2);
      
      flottant dtdv2 = -( 
			 ( (Mdg[3]*dA2dv2 + Mdg[7]*dB2dv2 + Mdg[11]*dC2dv2) 
			   * (eq2[0]*a+eq2[1]*b+eq2[2]*c) ) 
			 - ( (eq2[0]*Mdg[3]+eq2[1]*Mdg[7]+eq2[2]*Mdg[11]) 
			     * (dA2dv2*a + dadv2*eq2[0] 
				+ dB2dv2*b + dbdv2*eq2[1]
				+ dC2dv2*c + dcdv2*eq2[2]) )
			 )
	/ pow((eq2[0]*a+eq2[1]*b+eq2[2]*c),2);

      /* Derivee de x2p y2p z2p */
      flottant dx2pdu2 = dadu2*t + dtdu2*a;
      flottant dx2pdv2 = dadv2*t + dtdv2*a;

      flottant dy2pdu2 = dbdu2*t + dtdu2*b;
      flottant dy2pdv2 = dbdv2*t + dtdv2*b;

      flottant dz2pdu2 = dcdu2*t + dtdu2*c;
      flottant dz2pdv2 = dcdv2*t + dtdv2*c;

      /* f2 */
      f2[0] = 0.5*dx2pdu2;
      f2[1] = 0.5*dx2pdv2;
      f2[2] = 0.5*dy2pdu2;
      f2[3] = 0.5*dy2pdv2;
      f2[4] = 0.5*dz2pdu2;
      f2[5] = 0.5*dz2pdv2;

      
      /* P = F1*P1*F1' + F2*P2*F2' */
      flottant f1p[6];
      flottant f2p[6];

      flottant temp1[6];
      flottant temp2[9];
      flottant temp3[9];

      matrixTranspose(f1p,f1,3,2);
      matrixTranspose(f2p,f2,3,2);

      matrixMult(temp1,f1,covp1,3,2,2);
      matrixMult(temp3,temp1,f1p,3,2,3);

      matrixMult(temp1,f2,covp2,3,2,2);
      matrixMult(temp2,temp1,f2p,3,2,3);

      matrixAdd(covP,temp2,temp3,3,3);
    }



  
}
