/* Copyright (c) Ludovic Brethes (LAAS-CNRS) */
#include "yuv2rgb.h"

static long int crv_tab[256];
static long int cbu_tab[256];
static long int cgu_tab[256];
static long int cgv_tab[256];
static long int tab_76309[256];
static unsigned char clp[1024];			//for clip in CCIR601





/************************************************/
/*	Sum the input				*/
/*	Input: input, len			*/
/*	Output: input				*/
/*	Algorithm: add			        */
/************************************************/
void YUV2RGB(unsigned char *src, unsigned char *dst_ori,
	     int width, int height)
{      
	unsigned char *src0;
	unsigned char *src1;
	unsigned char *src2;
	unsigned char *dst;
	int y1,u,v; 
	int i,j, c1, c2, c3, c4;


	//Initialization
	src0=src;
	src1=src+width*height;
	src2=src+width*height+width*height/2;

	dst=dst_ori;

 
 	for (j = 0; j < height; j += 1) { 
		for (i = 0; i < width; i += 2) {

			u = *src1++;
			v = *src2++;

			c1 = crv_tab[v];
			c2 = cgu_tab[u];
			c3 = cgv_tab[v];
			c4 = cbu_tab[u];

            y1 = tab_76309[*src0++];	
			*dst++ = clp[384+((y1 + c1)>>16)];  
			*dst++ = clp[384+((y1 - c2 - c3)>>16)];
            *dst++ = clp[384+((y1 + c4)>>16)];

			y1 = tab_76309[*src0++];		
			*dst++ = clp[384+((y1 + c1)>>16)];  
			*dst++ = clp[384+((y1 - c2 - c3)>>16)];
			*dst++ = clp[384+((y1 + c4)>>16)];
		}
	}

}  


void YUV2RGB420(unsigned char *src, unsigned char *dst_ori,
								 int width,int height)
{
	unsigned char *src0;
	unsigned char *src1;
	unsigned char *src2;
	int y1,y2,u,v; 
	unsigned char *py1,*py2;
	int i,j, c1, c2, c3, c4;
	unsigned char *d1, *d2;

	//Initialization
	src0=src;
	src1=src+width*height;
	src2=src+width*height+width*height/4;

	py1=src0;
	py2=py1+width;
	d1=dst_ori;
	d2=d1+3*width;
 	for (j = 0; j < height; j += 2) { 
		for (i = 0; i < width; i += 2) {

			u = *src1++;
			v = *src2++;

			c1 = crv_tab[v];
			c2 = cgu_tab[u];
			c3 = cgv_tab[v];
			c4 = cbu_tab[u];

			//up-left
            y1 = tab_76309[*py1++];	
			*d1++ = clp[384+((y1 + c1)>>16)];  
			*d1++ = clp[384+((y1 - c2 - c3)>>16)];
            *d1++ = clp[384+((y1 + c4)>>16)];

			//down-left
			y2 = tab_76309[*py2++];
			*d2++ = clp[384+((y2 + c1)>>16)];  
			*d2++ = clp[384+((y2 - c2 - c3)>>16)];
            *d2++ = clp[384+((y2 + c4)>>16)];

			//up-right
			y1 = tab_76309[*py1++];
			*d1++ = clp[384+((y1 + c1)>>16)];  
			*d1++ = clp[384+((y1 - c2 - c3)>>16)];
			*d1++ = clp[384+((y1 + c4)>>16)];

			//down-right
			y2 = tab_76309[*py2++];
			*d2++ = clp[384+((y2 + c1)>>16)];  
			*d2++ = clp[384+((y2 - c2 - c3)>>16)];
            *d2++ = clp[384+((y2 + c4)>>16)];
		}
		d1 += 3*width;
		d2 += 3*width;
		py1+=   width;
		py2+=   width;
	}       


}


void YUV2BGR420(unsigned char *src, unsigned char *dst_ori, int width,int height)
{
  unsigned char *src0;
  unsigned char *src1;
  unsigned char *src2;
  int y1,y2,u,v; 
  unsigned char *py1,*py2;
  int i,j, c1, c2, c3, c4;
  unsigned char *d1, *d2;
  
  //Initialization
  src0=src;
  src1=src+width*height;
  src2=src+width*height+width*height/4;
  
  py1=src0;
  py2=py1+width;
  d1=dst_ori;
  d2=d1+3*width;
  for (j = 0; j < height; j += 2) { 
    for (i = 0; i < width; i += 2) {
      
      u = *src1++;
      v = *src2++;
      
      c1 = crv_tab[v];
      c2 = cgu_tab[u];
      c3 = cgv_tab[v];
      c4 = cbu_tab[u];
      
      //up-left
      y1 = tab_76309[*py1++];	
      *d1++ = clp[384+((y1 + c4)>>16)];   
      *d1++ = clp[384+((y1 - c2 - c3)>>16)];
      *d1++ = clp[384+((y1 + c1)>>16)];     

      //down-left
      y2 = tab_76309[*py2++];
      *d2++ = clp[384+((y2 + c4)>>16)];
      *d2++ = clp[384+((y2 - c2 - c3)>>16)];
      *d2++ = clp[384+((y2 + c1)>>16)];        

      //up-right
      y1 = tab_76309[*py1++];
      *d1++ = clp[384+((y1 + c4)>>16)];
      *d1++ = clp[384+((y1 - c2 - c3)>>16)];
      *d1++ = clp[384+((y1 + c1)>>16)];     
   
      //down-right
      y2 = tab_76309[*py2++];
      *d2++ = clp[384+((y2 + c4)>>16)];
      *d2++ = clp[384+((y2 - c2 - c3)>>16)];
      *d2++ = clp[384+((y2 + c1)>>16)];  
    }
    d1 += 3*width;
    d2 += 3*width;
    py1+=   width;
    py2+=   width;
  }  
}

/****************************************************/
/*	Sum the input									*/
/*	Input: input, len								*/
/*	Output: input									*/
/*	Algorithm: add									*/
/****************************************************/
void InitConvtTbl()
{
   long int crv,cbu,cgu,cgv;
   int i,ind;   
     
   crv = 104597; cbu = 132201;  /* fra matrise i global.h */
   cgu = 25675;  cgv = 53279;
  
   for (i = 0; i < 256; i++) {
      crv_tab[i] = (i-128) * crv;
      cbu_tab[i] = (i-128) * cbu;
      cgu_tab[i] = (i-128) * cgu;
      cgv_tab[i] = (i-128) * cgv;
      tab_76309[i] = 76309*(i-16);
   }
	 
   for (i=0; i<384; i++)
	  clp[i] =0;
   ind=384;
   for (i=0;i<256; i++)
	   clp[ind++]=i;
   ind=640;
   for (i=0;i<384;i++)
	   clp[ind++]=255;
}

