/*
GEST : GESTure Tracking
Copyright (C) 2006-2010  Brice Burger (LAAS-CNRS)
Ce code est protéger par une license APP
*/

/**
 ** gestRunTaskCodels.cc
 **
 ** Codels called by execution task gestRunTask
 **
 ** Author: Brice Burger
 ** Date: 03/2007
 **
 **/

# include <portLib.h>
#include "server/gestHeader.h"

#include <cv.h>
#include <highgui.h>

#ifndef WITH_VIAM
#include "cameraClient.h"
#else
#include "viamStruct.h"
#endif


#define GLOBALE//les variables globales sont déclarée dans params.h
#include "gestParams.h"
#include "traqueurDeDeuxMains.h"

/* utilisation de séquences */
#include "videoimgseq.h"


bool ViaSequence;//false en temps normal, true si on utilise une séquence d'images plutôt que la caméra
videoimgseq* vid;
videoimgseq* vidd;
int numimgfin, numimg;

bool BAVARD = false;//s'il doit être bavard durant la traque


/* variables globales */
int init, compteur;
bool initModuleFaite;
int* pfState;//particule renvoyee par le traqueur
int* pfStatecour;//particule renvoyee par le traqueur a ce tour (pour etre sur de la conserver)
int* pfStateprec;//particule renvoyee par le traqueur au tour precedent
int dist;
int nbvar;//nombre de variables du vecteur d'état
ETAT_GEST ETAT;//état courant du geste
bool PAUSE;
int*** MesureCouleur::bins_cub = NULL;//on initialise ici ce tableau (static) car il est très gros (~60Mo), on ne veux donc le créer qu'une seule fois
bool ONESHOT;

/* asservissement de la platine et calcul de la projection du visage pour GAZE */
projection* proj = NULL;//on a besoin de projeter ici
int MintrinsequeG[4];//Matrice intrinsèque gauche de GAZE (peut-être différente de celle de GEST si on n'utilise pas les mêmes caméras)
int point3D[3];//point 3D sur lequel centrer la platine
CvPoint point2D;//point 2D sur lequel centrer la platine (point 3D projeté sur l'image gauche)
bool ASSERV, GAZEdiff;//si l'asservissement de la platine est utilisé et si les caméras utilisées par GAZE sont différentes que celles de GEST
double sep=0.0,set=0.0,epa=0.0,eta=0.0;  // parametre des regulateur de la platine

/** Flux video issu de la camera */
IplImage* deBayer;//image débayerisée
#ifndef WITH_VIAM
IplImage* brute;//image brute
uchar *bufferedCameraImageg, *bufferedCameraImaged;
int tailleImg;
#else
IplImage* bruteg;//image débayerisée de gauche
IplImage* bruted;//image débayerisée de droite
#endif
IplImage* imgg;//image redimensionnée de droite
IplImage* imgd;//image redimensionnée de gauche
IplImage* imgTg;//image redimensionnée de droite (après changement d'espace de couleur)
IplImage* imgTd;//image redimensionnée de gauche (après changement d'espace de couleur)
int codeCouleur_CV;//code couleur d'opencv (pour le changement d'espace de couleur)
int imageWidth, imageHeight;
bool balanceDesBlancs;
int couleurDeRefBalanceDesBlancs;//couleur de référence de la balance des blancs (3 si la référence est la moyenne globale)
bool debayerisee;//si l'image est déjà débayerisée
bool bonneTaille;//si l'image a déjà la bonne taille (pas besoin de faire de cvResize)

/*CAMERA*/
POSTER_ID cameraPosterId;
POSTER_ID cameraPosterIdGAZE;
#ifndef WITH_VIAM
CameraBank cameraBank;
CameraPoster *cameraPoster;
#else
ViamImageBank *cameraPoster;
ViamImageBank *cameraPosterGAZE;
#endif

/* traqueur */
//Traqueur* traqueur;
Traqueur2* traqueur;

/* balance des blancs */
IplImage *imageg[3];
IplImage *imaged[3];
CvScalar  moyenneg;
CvScalar  moyenned;

/* pour le changement de repère : */
double CRg,CRh,CRi,CRj,CRk,CRl,CRm,CRn,CRo,CRalpha,CRbetha,CRgamma,CRxm,CRym,CRzm;

/*******************************************************************/
/*********************** fonctions internes ************************/
/*******************************************************************/

/* fonction de récupération des infos contenues dans cameraPoster :
 * 
 * récupération des images gauche et droite
 * récupération de la configuration du robot (via pom)
 * 
 * renvoie true si tout c'est bien passé, false autrement
 */
bool recupCameraPoster(int *report) {
	
	if (ViaSequence) {
		numimg++;
		if (numimg>numimgfin) {
			printf("fin de la série\n");
			return false;
		}
		printf("image numéro %d\n",numimg);
		cvResize(vid->next(), imgg, CV_INTER_LINEAR);//On redimensionne
		cvResize(vidd->next(), imgd, CV_INTER_LINEAR);//On redimensionne
		
		/* changement d'espace de couleur si nécessaire */
		if (codeCouleur_CV) {
			cvCvtColor( imgg, imgTg, codeCouleur_CV );
			cvCvtColor( imgd, imgTd, codeCouleur_CV );
		}
		
		return true;
	}
	
	/* on prend le poster */
	if (posterTake(cameraPosterId, POSTER_READ) != OK) {
		*report = S_gest_CANNOT_READ_POSTER;
		if (BAVARD) printf("GEST : POSTER NON TROUVE, on arrete\n");
		return false;
	}
	
	/* recupération de l'image */
	#ifndef WITH_VIAM
	memcpy(bufferedCameraImageg, cameraPoster->image.stereo.pixel,tailleImg*sizeof(char));
	memcpy(bufferedCameraImaged, &cameraPoster->image.stereo.pixel[tailleImg],tailleImg*sizeof(char));
	#else
	if (bonneTaille) {
		memcpy(imgg->imageData, cameraPoster->image[0].data+cameraPoster->image[0].dataOffset,cameraPoster->image[0].imageSize);
		memcpy(imgd->imageData, cameraPoster->image[1].data+cameraPoster->image[1].dataOffset,cameraPoster->image[0].imageSize);
	} else {
		memcpy(bruteg->imageData, cameraPoster->image[0].data+cameraPoster->image[0].dataOffset,cameraPoster->image[0].imageSize);
		memcpy(bruted->imageData, cameraPoster->image[1].data+cameraPoster->image[1].dataOffset,cameraPoster->image[0].imageSize);
	}
	#endif
	
	/* récupération de la configuration via pom */
	#ifndef WITH_VIAM
 	SDI_F->config = cameraPoster->image.stereo.pomTag[0];
 	#else
 	SDI_F->config = cameraPoster->image[0].pos;
	#endif
	
	/* c'est fini, on rend le poster */
	posterGive(cameraPosterId);
	
	/* mise à disposition des infos importantes récupérées de pom pour le traqueur */
	recupPompos.sensorToMain[0] = SDI_F->config.sensorToMain.euler.x;
	recupPompos.sensorToMain[1] = SDI_F->config.sensorToMain.euler.y;
	recupPompos.sensorToMain[2] = SDI_F->config.sensorToMain.euler.z;
	recupPompos.sensorToMain[3] = SDI_F->config.sensorToMain.euler.roll;
	recupPompos.sensorToMain[4] = SDI_F->config.sensorToMain.euler.pitch;
	recupPompos.sensorToMain[5] = SDI_F->config.sensorToMain.euler.yaw;
	recupPompos.mainToOrigin[0] = SDI_F->config.mainToOrigin.euler.x;
	recupPompos.mainToOrigin[1] = SDI_F->config.mainToOrigin.euler.y;
	recupPompos.mainToOrigin[2] = SDI_F->config.mainToOrigin.euler.z;
	recupPompos.mainToOrigin[3] = SDI_F->config.mainToOrigin.euler.roll;
	recupPompos.mainToOrigin[4] = SDI_F->config.mainToOrigin.euler.pitch;
	recupPompos.mainToOrigin[5] = SDI_F->config.mainToOrigin.euler.yaw;
	
	
	if (debayerisee) {//si l'image a été débayerisée
		
		/* image de droite */
		#ifndef WITH_VIAM
		uchar *src, *dst;
		src = bufferedCameraImaged;
		if (!bonneTaille) dst = (uchar*)deBayer->imageData;
		else dst = (uchar*)imgd->imageData;
		for(int i=0; i<imageHeight*imageWidth; i++) {//les images opencv prennent du BGR et non pas du RGBA
			*(dst++) = src[2];
			*(dst++) = src[1];
			*(dst++) = src[0];
			src += 4*sizeof(char);
		}
		if (!bonneTaille) cvResize(deBayer, imgd, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		#else
		if (!bonneTaille) cvResize(bruted, imgd, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		#endif
		
		/* image de gauche */
		#ifndef WITH_VIAM
		src = bufferedCameraImageg;
		if (!bonneTaille) dst = (uchar*)deBayer->imageData;
		else dst = (uchar*)imgg->imageData;
		for(int i=0; i<imageHeight*imageWidth; i++) {//les images opencv prennent du BGR et non pas du RGBA
			*(dst++) = src[2];
			*(dst++) = src[1];
			*(dst++) = src[0];
			src += 4*sizeof(char);
		}
		if (!bonneTaille) cvResize(deBayer, imgg, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		#else
		if (!bonneTaille) cvResize(bruteg, imgg, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		#endif
		
	} else {//si l'image n'a pas encore été débayerisée
	
		/* image de droite */
		#ifndef WITH_VIAM
		brute->imageData = (char*)bufferedCameraImaged;//On designe la zone mémoire contenant l'image
		if (bonneTaille) {
			cvCvtColor(brute, imgd, CV_BayerBG2BGR);//On effectue une débayerisation
		} else {
			cvCvtColor(brute, deBayer, CV_BayerBG2BGR);//On effectue une débayerisation
			cvResize(deBayer, imgd, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		}
		#else
		if (bonneTaille) {
			cvCvtColor(bruted, imgd, CV_BayerBG2BGR);//On effectue une débayerisation
		} else {
			cvCvtColor(bruted, deBayer, CV_BayerBG2BGR);//On effectue une débayerisation
			cvResize(deBayer, imgd, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		}
		#endif
		
		/* image de gauche */
		#ifndef WITH_VIAM
		brute->imageData = (char*)bufferedCameraImageg;//On designe la zone mémoire contenant l'image
		if (bonneTaille) {
			cvCvtColor(brute, imgg, CV_BayerBG2BGR);//On effectue une débayerisation
		} else {
			cvCvtColor(brute, deBayer, CV_BayerBG2BGR);//On effectue une débayerisation
			cvResize(deBayer, imgg, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		}
		#else
		if (bonneTaille) {
			cvCvtColor(bruteg, imgg, CV_BayerBG2BGR);//On effectue une débayerisation
		} else {
			cvCvtColor(bruteg, deBayer, CV_BayerBG2BGR);//On effectue une débayerisation
			cvResize(deBayer, imgg, CV_INTER_LINEAR);//On redimensionne (ou on copie si c'est la même taille)
		}
		#endif
	}
	
	/* changement d'espace de couleur si nécessaire */
	if (codeCouleur_CV) {
		cvCvtColor( imgg, imgTg, codeCouleur_CV );
		cvCvtColor( imgd, imgTd, codeCouleur_CV );
	}
	
	return true;
}

/* fait la balance des blancs :
 * 
 * en fait on normalise le bleu et le rouge et on multiplie par la moyenne du vert
 */
void faitBalanceDesBlancs() {

	moyenneg = cvAvg(imgg);
	moyenned = cvAvg(imgd);
		
	if (couleurDeRefBalanceDesBlancs==3) {
		cvSplit(imgg,imageg[0],imageg[1],imageg[2],NULL);
		cvConvertScale(imageg[0],imageg[0],moyenneg.val[0]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvConvertScale(imageg[1],imageg[1],moyenneg.val[1]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvConvertScale(imageg[2],imageg[2],moyenneg.val[2]*3/(moyenneg.val[0]+moyenneg.val[1]+moyenneg.val[2]));
		cvMerge(imageg[0],imageg[1],imageg[2],NULL,imgg);
			
		cvSplit(imgd,imaged[0],imaged[1],imaged[2],NULL);
		cvConvertScale(imaged[0],imaged[0],moyenned.val[0]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvConvertScale(imaged[1],imaged[1],moyenned.val[1]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvConvertScale(imaged[2],imaged[2],moyenned.val[2]*3/(moyenned.val[0]+moyenned.val[1]+moyenned.val[2]));
		cvMerge(imaged[0],imaged[1],imaged[2],NULL,imgd);
	} else {
		cvSplit(imgg,imageg[0],imageg[1],imageg[2],NULL);
		cvConvertScale(imageg[0],imageg[0],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[0]);
		cvConvertScale(imageg[1],imageg[1],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[1]);
		cvConvertScale(imageg[2],imageg[2],moyenneg.val[couleurDeRefBalanceDesBlancs]/moyenneg.val[2]);
		cvMerge(imageg[0],imageg[1],imageg[2],NULL,imgg);
			
		cvSplit(imgd,imaged[0],imaged[1],imaged[2],NULL);
		cvConvertScale(imaged[0],imaged[0],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[0]);
		cvConvertScale(imaged[1],imaged[1],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[1]);
		cvConvertScale(imaged[2],imaged[2],moyenned.val[couleurDeRefBalanceDesBlancs]/moyenned.val[2]);
		cvMerge(imaged[0],imaged[1],imaged[2],NULL,imgd);
	}
}

/* copie le vecteur d'état du traqueur dans le poster :
 * 
 * ATTENTION : les conventions des posters sont : x en avant, z vers le haut !!! (contrairement aux conventions dans la vision)
 */
void setPfState(int* vect) {
	
	/*** Vecteur d'état ***/
	/* tête */
	SDI_F->pos.Tx = vect[2];//x poster = z image !!
	SDI_F->pos.Ty = -vect[0];//y poster = -x image !!
	SDI_F->pos.Tz = -vect[1];//z poster = -y image !!
	/* main gauche */
	SDI_F->pos.MGx = vect[5];//x poster = z image !!
	SDI_F->pos.MGy = -vect[3];//y poster = -x image !!
	SDI_F->pos.MGz = -vect[4];//z poster = -y image !!
	SDI_F->pos.MGax = vect[8];//x poster = z image !!
	SDI_F->pos.MGay = vect[6];//y poster = -x image !!
	SDI_F->pos.MGaz = vect[7];//z poster = -y image !!
	SDI_F->pos.MGalpha = vect[11];//x poster = z image !!
	SDI_F->pos.MGbetha = -vect[9];//y poster = -x image !!
	SDI_F->pos.MGgamma = -vect[10];//z poster = -y image !!
	/* main droite */
	SDI_F->pos.MDx = vect[14];//x poster = z image !!
	SDI_F->pos.MDy = -vect[12];//y poster = -x image !!
	SDI_F->pos.MDz = -vect[13];//z poster = -y image !!
	SDI_F->pos.MDax = vect[17];//x poster = z image !!
	SDI_F->pos.MDay = vect[15];//y poster = -x image !!
	SDI_F->pos.MDaz = vect[16];//z poster = -y image !!
	SDI_F->pos.MDalpha = vect[20];//x poster = z image !!
	SDI_F->pos.MDbetha = -vect[18];//y poster = -x image !!
	SDI_F->pos.MDgamma = -vect[19];//z poster = -y image !!
	
	
	/*** Vecteur d'état dans le repère MAIN -> SENSOR to MAIN ***/
	/* récupération des rotations et translations */
	CRalpha = recupPompos.sensorToMain[3];
	CRbetha = recupPompos.sensorToMain[4];
	CRgamma = recupPompos.sensorToMain[5];
	CRxm = recupPompos.sensorToMain[0]*1000;//*1000 parcequ'on est en millimètres
	CRym = recupPompos.sensorToMain[1]*1000;
	CRzm = recupPompos.sensorToMain[2]*1000;
	/* calcul de la matrice de rotation */
	CRg = cos(CRbetha)*cos(CRgamma);
	CRh = -sin(CRgamma)*cos(CRalpha) + cos(CRgamma)*sin(CRbetha)*sin(CRalpha);
	CRi = sin(CRgamma)*sin(CRalpha) + cos(CRgamma)*cos(CRalpha)*sin(CRbetha);
	CRj = cos(CRbetha)*sin(CRgamma);
	CRk = cos(CRgamma)*cos(CRalpha) + sin(CRgamma)*sin(CRbetha)*sin(CRalpha);
	CRl = -cos(CRgamma)*sin(CRalpha) + sin(CRgamma)*cos(CRalpha)*sin(CRbetha);
	CRm = -sin(CRbetha);
	CRn = sin(CRalpha)*cos(CRbetha);
	CRo = cos(CRbetha)*cos(CRalpha);
	/* transformation */
	SDI_F->posMain.Tx = (CRg*SDI_F->pos.Tx + CRh*SDI_F->pos.Ty + CRi*SDI_F->pos.Tz) + CRxm;
	SDI_F->posMain.Ty = (CRj*SDI_F->pos.Tx + CRk*SDI_F->pos.Ty + CRl*SDI_F->pos.Tz) + CRym;
	SDI_F->posMain.Tz = (CRm*SDI_F->pos.Tx + CRn*SDI_F->pos.Ty + CRo*SDI_F->pos.Tz) + CRzm;
	SDI_F->posMain.MGx = (CRg*SDI_F->pos.MGx + CRh*SDI_F->pos.MGy + CRi*SDI_F->pos.MGz) + CRxm;
	SDI_F->posMain.MGy = (CRj*SDI_F->pos.MGx + CRk*SDI_F->pos.MGy + CRl*SDI_F->pos.MGz) + CRym;
	SDI_F->posMain.MGz = (CRm*SDI_F->pos.MGx + CRn*SDI_F->pos.MGy + CRo*SDI_F->pos.MGz) + CRzm;
	SDI_F->posMain.MDx = (CRg*SDI_F->pos.MDx + CRh*SDI_F->pos.MDy + CRi*SDI_F->pos.MDz) + CRxm;
	SDI_F->posMain.MDy = (CRj*SDI_F->pos.MDx + CRk*SDI_F->pos.MDy + CRl*SDI_F->pos.MDz) + CRym;
	SDI_F->posMain.MDz = (CRm*SDI_F->pos.MDx + CRn*SDI_F->pos.MDy + CRo*SDI_F->pos.MDz) + CRzm;
	//... les axes et angles devraient également êtres modifiées ici, mais on verra ça une autre fois
	
	
	/*** Projection sur l'image gauche ***/
	/* tête */
	if (GAZEdiff) {//ATTENTION : les conventions dans le code sont différentes du poster
		point3D[0] = -SDI_F->posMain.Ty;//x image = -y poster !!
		point3D[1] = -SDI_F->posMain.Tz;//y image = -z poster !!
		point3D[2] = SDI_F->posMain.Tx;//z image = x poster !!
		proj->projettePointgaucheAutrepart (point3D, &point2D, MintrinsequeG[0], MintrinsequeG[1], MintrinsequeG[2], MintrinsequeG[3]);
		if ((point2D.x<0)||(point2D.x>=w)||(point2D.y<0)||(point2D.y>=h)) SDI_F->pos2D.R = -1;//comme l'image n'est plus la même, il y a des risques d'en sortir -> il faut vérifier si on sort de nos propres limites, i.e. w,h
		else SDI_F->pos2D.R = int (RayonTete*MintrinsequeG[2]/point3D[2]);
	} else {
		point3D[0] = vect[0];
		point3D[1] = vect[1];
		point3D[2] = vect[2];
		proj->projettePointgauche (point3D, &point2D);
		SDI_F->pos2D.R = int (RayonTete*intrinsicsLeft[2]/point3D[2]);
	}
	SDI_F->pos2D.Tx = point2D.x;
	SDI_F->pos2D.Ty = point2D.y;
}

/* fonction de changement d'espace de couleur :
 * 
 * renvoie true si tout c'est bien passé, false autrement
 */
bool ChangeEspaceCoul(GEST_COLORSPACE EspaceCoul, int *report) {
	
	switch (EspaceCoul) {
		case baseRGB:
		case baseBGR:
		case basergbN:
			printf("nouvel espace de couleur : RGB/BGR/RGBN\n");
			codeCouleur_CV = 0;//=CV_BGR2BGRA, qui ne servira jamais ici
			balanceDesBlancs = false;
			nbfermeture = 2;
			couleurDeRefBalanceDesBlancs = 1;//couleur de référence de la balance des blancs (3 si la référence est la moyenne globale)
			break;
		case baseHSV:
			printf("nouvel espace de couleur : HSV\n");
			codeCouleur_CV = CV_BGR2HSV;
			balanceDesBlancs = false;
			nbfermeture = 2;
			couleurDeRefBalanceDesBlancs = 1;//couleur de référence de la balance des blancs (3 si la référence est la moyenne globale)
			break;
		case baseYCrCb:
			printf("nouvel espace de couleur : YCrCb\n");
			codeCouleur_CV = CV_BGR2YCrCb;
			balanceDesBlancs = false;//quand on extrait la luminosité, il ne faut pas que l'image ai été triturée avant!
			nbfermeture = 0;
			break;
		case baseLab:
			printf("nouvel espace de couleur : Lab\n");
			codeCouleur_CV = CV_BGR2Lab;
			balanceDesBlancs = false;//quand on extrait la luminosité, il ne faut pas que l'image ai été triturée avant!
			nbfermeture = 0;
			break;
		case baseIrg:
		case baseI1I2I3:
		default:
			printf("espace de couleur (%d) non traité!!\n",EspaceCoul);
			*report = S_gest_BAD_COLOR_SPACE;
			return false;
	}
	
	if (codeCouleur_CV) {
		if (imgTg==NULL) {
			imgTg=cvCreateImage(cvSize(w,h),8,3);
			imgTd=cvCreateImage(cvSize(w,h),8,3);
		}
	} else {//RGB, pas de changement d'espace de couleurs
		if (imgTg!=NULL) {
			cvReleaseImage(&imgTg);
			cvReleaseImage(&imgTd);
			imgTg=NULL;
			imgTd=NULL;
		}
	}
	
	if (!traqueur->ChangeEspaceCouleur(EspaceCoul)) {
		*report = S_gest_BAD_COLOR_SPACE;
		return false;
	}
	
	return true;
}


/*******************************************************************/
/************************ fonctions genom **************************/
/*******************************************************************/

/*------------------------------------------------------------------------
 *
 * gestRunTaskInit  --  Initialization codel (fIDS, ...)
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
gestRunTaskInit(int *report)
{
	/* mise en place des valeurs par defaut dans la SDI_F */
	SDI_F->pos.ETAT = RIEN_GENOM;
	SDI_F->HeadDetected = (GEST_BOOLEEN)false;
	SDI_F->pos2D.R = -1;
	
	/* mise en place des autres valeurs par défaut */
	BASE = GLOBAL_PATH_TO_GEST + string("/code/");//chemin de base du code de suivi
	w=GEST_INTERNAL_WIDTH;
	h=GEST_INTERNAL_HEIGHT;
	distMaxEval = 10;
	seuilbas = 170;
	nbpart = 100;
	surfaceminblob = 20;
	CalculeImgDist = false;
	espaceCoul = baseRGB;//espace de couleur RGB par défaut
	codeCouleur_CV = 0;
	debayerisee = false;
	bonneTaille = false;
	initModuleFaite = false;
	ViaSequence = false;
	PAUSE = false;
	ONESHOT = false;
	ASSERV = false;
	GAZEdiff = false;
	traqueur = NULL;
	SDI_F->WithPlatineControl = (GEST_BOOLEEN)false;
	//nom par défaut des poster de la caméra
	#ifndef WITH_VIAM
	cameraBank=CAMERA_BANK_A;
	snprintf(SDI_F->InitStruct.CameraPosterName, GEST_TAILLE_CHAR_MAX, CAMERA_POSTER_NAME, cameraBank);
	#else
	strncpy(SDI_F->InitStruct.CameraPosterName,"viamBankTop", GEST_TAILLE_CHAR_MAX);
	strncpy(SDI_F->InitStructGAZE.CameraPosterName,"viamBankTop", GEST_TAILLE_CHAR_MAX);
	#endif
	
	return OK;
}

/*------------------------------------------------------------------------
 *
 * gestRunTaskEnd  --  Termination codel
 *
 * Description: 
 * 
 * Returns:    OK or ERROR
 */

STATUS
gestRunTaskEnd(void)
{
	printf ("GEST : -----FIN DU MODULE----\n");
	
	for (int i=0; i<3; i++) cvReleaseImage(&imageg[i]);
	for (int i=0; i<3; i++) cvReleaseImage(&imaged[i]);
	
	/* libération de mémoire */
	#ifndef WITH_VIAM
	if (brute) cvReleaseImage(&brute);
	free(bufferedCameraImageg);
	free(bufferedCameraImaged);
	#else
	cvReleaseImage(&bruteg);
	cvReleaseImage(&bruted);
	#endif
	if (deBayer) cvReleaseImage(&deBayer);
	cvReleaseImage(&imgg);
	cvReleaseImage(&imgd);
	if (codeCouleur_CV) {
		cvReleaseImage(&imgTg);
		cvReleaseImage(&imgTd);
	}
	delete [] pfStateprec;
	delete [] pfStatecour;
	delete traqueur;
	if (ViaSequence) {
		if (vid) delete vid;
		if (vidd) delete vidd;
	}
	if (proj != NULL) delete proj;//pour l'asservissement de la platine

  return OK;
}

/*------------------------------------------------------------------------
 *
 * gestRunTaskExec  --  codel of permanent activity
 *
 * Description: 
 * 
 * Reports:      OK
 * 
 * Returns:    OK or ERROR
 */

STATUS
gestRunTaskExec(int *report)
{
	return OK;
}
/*------------------------------------------------------------------------
 * Init
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_POSTER_NOT_FOUND
 *              S_gest_CANNOT_READ_POSTER
 *              S_gest_BAD_COLOR_SPACE
 */

/* gestInitMain  -  codel EXEC of Init
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gestInitMain(GEST_INIT_STRUCT *InitStruct, int *report)
{
	/*--- INITIALISATION DES IMAGES ET DU SUIVI ---*/
	/* Initialisation de la caméra elle même (contrôle des images) */
	printf("connecting to camera poster '%s'\n",SDI_F->InitStruct.CameraPosterName);
	if (posterFind(SDI_F->InitStruct.CameraPosterName, &cameraPosterId) != OK) {
		printf("ERREUR: gestInitMain: camera posterFind");
		*report = S_gest_POSTER_NOT_FOUND;
		return ETHER;
	}
	if (posterTake(cameraPosterId, POSTER_READ) != OK) {
		printf("ERREUR: gestInitMain: camera posterTake");
		*report = S_gest_CANNOT_READ_POSTER;
		return ETHER;
	}
	#ifndef WITH_VIAM
	cameraPoster = (CameraPoster*) posterAddr(cameraPosterId);
	#else
	cameraPoster = (ViamImageBank*) posterAddr(cameraPosterId);
	#endif
	if (cameraPoster == NULL) {
		printf("ERREUR: gestInitMain: no camera poster");
		*report = S_gest_CANNOT_READ_POSTER;
		return ETHER;
	}
	
	/* taille des images récupérées */
	#ifndef WITH_VIAM
	imageWidth = cameraPoster->width;
	imageHeight = cameraPoster->height;
	#else
	imageWidth = cameraPoster->image[0].width;
	imageHeight = cameraPoster->image[0].height;
	#endif
	if ((imageWidth==w)&&(imageHeight==h)) bonneTaille = true;
	
	/* récupération des matrices intrinsèques *///ATTENTION à l'ordre !
	#ifndef WITH_VIAM
	setintrinsicsLeft(	cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_LEFT][6],//u0
				cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_LEFT][2],//v0
				-cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_LEFT][5],//au
				cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_LEFT][0],//av
				768, 1024 );//taille de l'image d'origine //avec le module camera on a besoin de ces données
	setintrinsicsRight(	cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_RIGHT][6],//u0
				cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_RIGHT][2],//v0
				-cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_RIGHT][5],//au
				cameraPoster->image.stereo.mintrinsic[CAMERA_POSTER_RIGHT][0],//av
				768, 1024 );//taille de l'image d'origine //avec le module camera on a besoin de ces données
	#else
	setintrinsicsLeft(	cameraPoster->image[0].calibration.intrirect[2],//u0
				cameraPoster->image[0].calibration.intrirect[5],//v0
				cameraPoster->image[0].calibration.intrirect[0],//au
				cameraPoster->image[0].calibration.intrirect[4],//av
				imageHeight, imageWidth );//taille de l'image
	setintrinsicsRight(	cameraPoster->image[1].calibration.intrirect[2],//u0
				cameraPoster->image[1].calibration.intrirect[5],//v0
				cameraPoster->image[1].calibration.intrirect[0],//au
				cameraPoster->image[1].calibration.intrirect[4],//av
				imageHeight, imageWidth );//taille de l'image
	#endif
	
	//... on part du principe que les caméras sont parallèles.
	Mdg[3] = cameraPoster->calibration.baseline*1000;//dans viam, c'est en mètres
	Mgd[3] = -Mdg[3];
	printf("baseline is %f\n",Mdg[3]);
	
	posterGive(cameraPosterId);
	
	/* vérifications en cas d'initialisation multiple */
	if (initModuleFaite) {
		#ifndef WITH_VIAM
		if (brute != NULL) { 
			cvReleaseImage(&brute);
			brute = NULL;
		}
		if (bufferedCameraImageg != NULL) { 
			free(bufferedCameraImageg);
			bufferedCameraImageg = NULL;
		}
		if (bufferedCameraImaged != NULL) { 
			free(bufferedCameraImaged);
			bufferedCameraImaged = NULL;
		}
		#else
		if (bruteg != NULL) { 
			cvReleaseImage(&bruteg);
			bruteg = NULL;
		}
		if (bruted != NULL) { 
			cvReleaseImage(&bruted);
			bruted = NULL;
		}
		#endif
		if (deBayer != NULL) { 
			cvReleaseImage(&deBayer);
			deBayer = NULL;
		}
		if (proj != NULL) {
			delete proj;//pour l'asservissement de la platine
			proj=NULL;
		}
	}
	proj = new projection();//pour l'asservissement de la platine
	
	printf("GEST : taille de l'image importée de camera : %d x %d\n", imageWidth, imageHeight);
	deBayer = cvCreateImage(cvSize(imageWidth,imageHeight),8,3);
	#ifndef WITH_VIAM
	if (debayerisee) {//si l'image a déjà été débayerisée
		tailleImg = imageWidth*imageHeight*4;
		brute = NULL;
	} else {//si l'image n'a pas encore été débayerisée
		tailleImg = imageWidth*imageHeight;
		brute=cvCreateImage(cvSize(imageWidth,imageHeight),8,1);
	}
	bufferedCameraImageg = (uchar*)calloc(tailleImg, sizeof(char));
	bufferedCameraImaged = (uchar*)calloc(tailleImg, sizeof(char));
	#else
	bruteg = cvCreateImage(cvSize(imageWidth,imageHeight),8,3);
	bruted = cvCreateImage(cvSize(imageWidth,imageHeight),8,3);
	#endif
	
	if (initModuleFaite) {
		printf ("GEST : gestInitMain : OK\n");
		return ETHER;
	}
	
	/* création du traqueur */
	imgg=cvCreateImage(cvSize(w,h),8,3);
	imgd=cvCreateImage(cvSize(w,h),8,3);
	
	if (traqueur==NULL) traqueur = new Traqueur2(imgg, imgd);
	nbvar = traqueur->GETNBVAR();
	
	pfStateprec = new int[nbvar];
	pfStatecour = new int[nbvar];
	
	/* création des images de travail */
	ChangeEspaceCoul((GEST_COLORSPACE)espaceCoul, report);//mise en place de l'espace de couleur par défaut
	
	if (codeCouleur_CV) {
		if (imgTg==NULL) {
			imgTg=cvCreateImage(cvSize(w,h),8,3);
			imgTd=cvCreateImage(cvSize(w,h),8,3);
		}
	}
	
	for (int i=0; i<3; i++) imageg[i] = cvCreateImage(cvSize(imgg->width,imgg->height),imgg->depth,1);
	for (int i=0; i<3; i++) imaged[i] = cvCreateImage(cvSize(imgd->width,imgd->height),imgd->depth,1);
	
	initModuleFaite = true;
	printf ("GEST : gestInitMain : OK\n");
	
	return ETHER;
}


/*------------------------------------------------------------------------
 * InitForGaze
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_POSTER_NOT_FOUND
 *              S_gest_CANNOT_READ_POSTER
 */

/* gestInitForGaze  -  codel EXEC of InitForGaze
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
gestInitForGaze(GEST_INIT_STRUCT *InitStruct, int *report)
{
	#ifndef WITH_VIAM
	printf("ERREUR : la fonction gestInitForGaze ne peut pas être utilisée avec un autre module que VIAM, ou alors il faut l'implémenter...\n");
	*report = S_gest_POSTER_NOT_FOUND;
	return ETHER;
	#endif
	
	/* Recherche du poster */
	printf("connecting to GAZE camera poster '%s'\n",SDI_F->InitStructGAZE.CameraPosterName);
	if (posterFind(SDI_F->InitStructGAZE.CameraPosterName, &cameraPosterIdGAZE) != OK) {
		printf("ERREUR: gestInitForGaze: camera posterFind");
		*report = S_gest_POSTER_NOT_FOUND;
		return ETHER;
	}
	if (posterTake(cameraPosterIdGAZE, POSTER_READ) != OK) {
		printf("ERREUR: gestInitForGaze: camera posterTake");
		*report = S_gest_CANNOT_READ_POSTER;
		return ETHER;
	}
	cameraPosterGAZE = (ViamImageBank*) posterAddr(cameraPosterIdGAZE);
	if (cameraPosterGAZE == NULL) {
		printf("ERREUR: gestInitForGaze: no camera poster");
		*report = S_gest_CANNOT_READ_POSTER;
		return ETHER;
	}
	
	/* récupération des matrices intrinsèques *///ATTENTION à l'ordre !
	double wtmp = cameraPosterGAZE->image[0].width;
	double htmp = cameraPosterGAZE->image[0].height;
	MintrinsequeG[0] = cameraPosterGAZE->image[0].calibration.intrirect[2]*w/wtmp;//u0
	MintrinsequeG[1] = cameraPosterGAZE->image[0].calibration.intrirect[5]*h/htmp;//v0
	MintrinsequeG[2] = cameraPosterGAZE->image[0].calibration.intrirect[0]*w/wtmp;//au
	MintrinsequeG[3] = cameraPosterGAZE->image[0].calibration.intrirect[4]*h/htmp;//av
	
	GAZEdiff = true;
	
	posterGive(cameraPosterIdGAZE);
	
	printf ("GEST : gestInitForGaze : OK\n");
	
	return ETHER;
}


/*------------------------------------------------------------------------
 * Track
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_FAILED
 *              S_gest_CANNOT_READ_POSTER
 */

/* LancerStart  -  codel Start of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
LancerStart(int *report)
{
	if (PAUSE) PAUSE = false;
	
	if (ViaSequence) {//... à changer, c'est pas bien en dur!!
		//c'est parès un return ça devrait pas
		//src/LeMain/traqueur 4 /home/bburger/truc1/ image g.png 0040 0440 4 d.png
		numimgfin = 310;
		numimg = 198;
		printf("Lancement de la séquence d'images %s%s%04d%s à %d images","/home/bburger/truc1/","image",numimg,"*.png", numimgfin);
		vid = new videoimgseq("/home/bburger/truc1/","image","g.png",numimg,0740,4);//image gauche//...il faut donner plus, sinon, il renvoie NULL avant la fin (plusieurs dizaines d'images avant)!!!
		vidd = new videoimgseq("/home/bburger/truc1/","image","d.png",numimg,0740,4);//image droite
		if(vid->init() && vidd->init()) cout << "Video ouverte\n";
		else cout << "Erreur lors de l'initialisation de la video\n";
	}
	
	init = 0;
	return EXEC;
}

/* LancerExec  -  codel EXEC of Track
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
LancerExec(int *report)
{
 double p=0,t=0,ep,et,pt_maxspeed=0.6,panKp=2,panKi=0,panKd=2,tiltKp=2,tiltKi=0,tiltKd=2; /* platine control*/
	
	/* par défaut, quoi qu'il arrive, la structure de contrôle de la platine doit rester à 0 */
	SDI_F->platineStruct.pan  = 0;
	SDI_F->platineStruct.tilt = 0;

	if (PAUSE) return EXEC;
	
	if (!init) {

		/******************/	
		/* initialisation */
		/******************/
		
		/* récupération des infos (images et compagnie) */
		if (!recupCameraPoster(report)) return END;
		
		/* Balance des blancs */
		if (balanceDesBlancs) faitBalanceDesBlancs();
		
		/* initialisation du traqueur */
		if (codeCouleur_CV) ETAT = traqueur->INIT(imgg,imgd , imgTg,imgTd);
		else ETAT = traqueur->INIT(imgg,imgd , imgg,imgd);
		if (ETAT==0) {//la traque n'a pas démarré
			if (BAVARD) printf("GEST : INITIALISATION IMPOSSIBLE, on recommence\n");
			return EXEC;
		}
		if (BAVARD) printf("GEST : INITIALISATIONcommence\n");
		compteur = 0;
		init = 1;
		
		/* l'initialisation c'est déjà le premier pas */
		pfState = traqueur->RECUP();
		for (int i=0;i<nbvar;i++) pfStatecour[i] = pfState[i];
	
		/* remplissage du poster */
		SDI_F->pos.ETAT = (ETAT_GEST_GENOM)ETAT;
		setPfState(pfStatecour);//copie du vecteur d'état de départ
		SDI_F->HeadDetected = (GEST_BOOLEEN)true;//si on démarre, c'est qu'on a détecté une tête
		
		return EXEC;
	}

	/**********************/
	/* on avance d'un pas */
	/**********************/
	if (BAVARD) printf("GEST : pas numero %d\n",compteur);
	
	/* récupération des infos (images et compagnie) */
	if (!recupCameraPoster(report)) return END;
	
	/* Balance des blancs */
	if (balanceDesBlancs) faitBalanceDesBlancs();
	
	/* le pas */
	for (int i=0;i<nbvar;i++) pfStateprec[i] = pfStatecour[i];//mémoire du pas precedent
	if (codeCouleur_CV) ETAT = traqueur->PAS(imgg,imgd , imgTg,imgTd);
	else ETAT = traqueur->PAS(imgg,imgd , imgg,imgd);
	
	if (ETAT==0) {//plus rien n'est traqué -> il faut réinitialiser entièrement
		init = 0;
		SDI_F->pos.ETAT = RIEN_GENOM;//on indique que plus rien n'est traqué
		return EXEC; 
	}
	
	pfState = traqueur->RECUP();
	for (int i=0;i<nbvar;i++) pfStatecour[i] = pfState[i];
	
	/* remplissage du poster */
	SDI_F->pos.ETAT = (ETAT_GEST_GENOM)ETAT;
	setPfState(pfStatecour);
	if (((ETAT_GEST_GENOM)ETAT==RIEN_GENOM)||((ETAT_GEST_GENOM)ETAT==PAStETE_GENOM)||((ETAT_GEST_GENOM)ETAT==PAStETEnImAINgAUCHE_GENOM)||((ETAT_GEST_GENOM)ETAT==PAStETEnImAINdROITE_GENOM)) SDI_F->pos2D.R = -1;
	
	if (traqueur->TETEDETECTEE()) SDI_F->HeadDetected = (GEST_BOOLEEN)true;
	else SDI_F->HeadDetected = (GEST_BOOLEEN)false;
	
	
	/*********************************/
	/* partie asservissement platine */
	/*********************************/
	compteur++;
	if (ASSERV) {
		
		/* calcul du point image sur lequel se fera l'asservissement */
		//on centre sur le centre de gravité des cibles (en donnant tout de même la priorité à la tête)
		//pour projeter, il faut garder les conventions vision : dans GEST, on a : x à gauche, y vers le haut, z la profondeur
		p=-1;
		switch (ETAT) {
			case RIEN_GENOM:
				printf("devrait être impossible!!!!\n");
				*report = S_gest_FAILED;
				return ETHER;
			case PAStETEnImAINgAUCHE_GENOM://s'il n'y a plus qu'une main, ça ne suffit pas, on ne bouge donc pas
			case PAStETEnImAINdROITE_GENOM:
				p=0;
				t=0;
				printf("pas de mouvement\n");
				break;
			case PAStETE_GENOM://on se met entre les deux mains, mais plus haut pour chercher la tête (en partant du principe que les mains sont le plus souvent en bas, le long du corps)
				point3D[0] = (pfStatecour[3] + pfStatecour[12])/2;
				point3D[1] = (pfStatecour[4] + pfStatecour[13])/2 - 500;//<0 car y vers le haut, on ne tient pas compte de la distance
				point3D[2] = (pfStatecour[5] + pfStatecour[14])/2;
				proj->projettePointgauche (point3D, &point2D);
				break;
			case TOUT_GENOM://centre de gravité, mais avec poids double pour la tête
				point3D[0] = (2*pfStatecour[0] + pfStatecour[3] + pfStatecour[12])/4;
				point3D[1] = (2*pfStatecour[1] + pfStatecour[4] + pfStatecour[13])/4;
				point3D[2] = (2*pfStatecour[2] + pfStatecour[5] + pfStatecour[14])/4;
				proj->projettePointgauche (point3D, &point2D);
				break;
			case PASmAINgAUCHE_GENOM://il y a de grandes chances pour que la main manquante soit de l'autre coté du corps, on centre donc sur la tête (sauf en hauteur)
				point3D[0] = pfStatecour[0];
				point3D[1] = (pfStatecour[1] + pfStatecour[13])/2;
				point3D[2] = pfStatecour[2];
				proj->projettePointgauche (point3D, &point2D);
				break;
			case PASmAINdROITE_GENOM://il y a de grandes chances pour que la main manquante soit de l'autre coté du corps, on centre donc sur la tête (sauf en hauteur)
				point3D[0] = pfStatecour[0];
				point3D[1] = (pfStatecour[1] + pfStatecour[4])/2;
				point3D[2] = pfStatecour[2];
				proj->projettePointgauche (point3D, &point2D);
				break;
			case AUCUNEmAIN_GENOM://on centre sur la tête, mais plus bas pour chercher les mains (en partant du principe que les mains sont le plus souvent en bas, le long du corps)
				point3D[0] = pfStatecour[0];
				point3D[2] = pfStatecour[2];
				if (pfStatecour[2]>1000) point3D[1] = pfStatecour[1] + 300;//on choisi le centre sur le coeur, i.e. la tête - 30cm suivant la hauteur (>0 car y vers le haut)
				else point3D[1] = pfStatecour[1];//sauf si on est trop près de la caméra, alors on centre directement sur la tête pour ne pas la perdre
				proj->projettePointgauche (point3D, &point2D);
				break;
		}
		
		/* PAN TILT PID Controllers */
       	 	if (p==-1) {//rien à recalculer si la platine ne doit pas bouger
			ep=(w/2)-point2D.x;	sep=sep+ep; 	et=(h/2)-point2D.y;	set=set+et; 
			p = panKp*ep/1000  + panKi*sep/10000 + panKd*(ep-epa)/1000; 
			t = tiltKp*et/1000 + tiltKi*set/10000 + tiltKd*(et-eta)/1000; 

			// speeds saturation
			if ((ep>0)&(p>pt_maxspeed))		{ p=pt_maxspeed;  sep=sep-ep; printf("P max");}
			else if ((ep<0)&(p< -pt_maxspeed))	{ p=-pt_maxspeed; sep=sep-ep; printf("P min");}
 			if ((et>0)&(t>pt_maxspeed))		{ t=pt_maxspeed;  set=set-et; printf("T max");}
			else if ((et<0)&(t< -pt_maxspeed))	{ t=-pt_maxspeed; set=set-et; printf("T min");}

 			printf("P= %.2f |U= %.2f |Su= %.1f  |  ",p,ep,sep);
 			printf("T= %.2f |V= %.2f |Sv= %.1f \n ",t,et,set); 
			epa=ep; eta=et;
		}
		
		SDI_F->platineStruct.pan  = -p;	//... il faudra voir avec cette histoire de signe : 
						//...le suivi de la platine plante quand on l'initialise avec "platine::InitDriver /dev/ttyS0 9600 0",
						//...il faut donc démarrer avec "platine::InitDriver /dev/ttyS0 9600 1",
						//...mais ça inverse le sens de rotation en pan !
		SDI_F->platineStruct.tilt = t;
	}
	
	return EXEC;
}

/* LancerEnd  -  codel END of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
LancerEnd(int *report)
{
	init = 0;
	SDI_F->pos.ETAT = RIEN_GENOM;
	SDI_F->HeadDetected = (GEST_BOOLEEN)false;
	traqueur->sauvegarde(false,false,NULL);//s'il y avait sauvegarde des résultats du traqueur, il est stoppé
	if (ViaSequence) {//dans le cas d'une séquence
		delete vid;
		delete vidd;
		vid=NULL;
		vidd=NULL;
	}
	printf ("GEST : FIN de la traque\n");
	return ETHER;
}

/* LancerInter  -  codel INTER of Track
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
LancerInter(int *report)
{
	return LancerEnd(report);
}


/*------------------------------------------------------------------------
 * LoopInitialisation
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_CANNOT_READ_POSTER
 */

/* InitEnBoucleStart  -  codel Start of LoopInitialisation
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
InitEnBoucleStart(int *report)
{
	printf ("GEST : INITIALISATION EN BOUCLE\n");
	return EXEC;
}

/* InitEnBoucleExec  -  codel EXEC of LoopInitialisation
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
InitEnBoucleExec(int *report)
{
	/************************************/
	/*     initialisation permanente    */
	/* la SDI_F n'est pas remise à jour */
	/************************************/
	
	/* récupération des infos (images et compagnie) */
	if (!recupCameraPoster(report)) return END;
	
	/* Balance des blancs */
	if (balanceDesBlancs) faitBalanceDesBlancs();
	
	/* initialisation du traqueur */
	if (codeCouleur_CV) ETAT = traqueur->INIT(imgg,imgd , imgTg,imgTd);
	else ETAT = traqueur->INIT(imgg,imgd , imgg,imgd);
	if (ETAT==0) printf("GEST : NON OK\n");
	else printf("GEST : OK\n");
	
	return EXEC;
}

/* InitEnBoucleEnd  -  codel END of LoopInitialisation
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
InitEnBoucleEnd(int *report)
{
	printf ("GEST : FIN de l'initialisation en boucle\n");
	return ETHER;
}

/* InitEnBoucleInter  -  codel INTER of LoopInitialisation
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
InitEnBoucleInter(int *report)
{
	return InitEnBoucleEnd(report);
}

/*------------------------------------------------------------------------
 * SaveSequence
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_BAD_PATH
 *              S_gest_CANNOT_READ_POSTER
 */

/* EnregistreSequenceStart  -  codel START of SaveSequence
   Returns:  START EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
EnregistreSequenceStart(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	printf ("GEST : ENREGISTREMENT D'UNE SEQUENCE D'IMAGE (sans traque)\n");
	if (!traqueur->sauvegarde((bool)SaveParams->saveImages,(bool)SaveParams->saveResults,SaveParams->savePath)) {
		printf ("GEST : EnregistreSequenceStart : ERREUR\n");
		*report = S_gest_BAD_PATH;
		return ETHER;
	}
	printf ("GEST : EnregistreSequenceStart : OK\n");
	
	return EXEC;
}

/* EnregistreSequenceExec  -  codel EXEC of SaveSequence
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
EnregistreSequenceExec(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	/************************************/
	/*   enregistrement d'une séquence  */
	/*      d'image, pas de traque      */
	/************************************/
	
	/* récupération des infos (images et compagnie) */
	if (!recupCameraPoster(report)) return END;
	
	/* Balance des blancs */
	if (balanceDesBlancs) faitBalanceDesBlancs();
	
	traqueur->enregistreImgs(imgg,imgd);
	return EXEC;
}

/* EnregistreSequenceEnd  -  codel END of SaveSequence
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
EnregistreSequenceEnd(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	traqueur->sauvegarde(false,false,NULL);
	printf ("GEST : FIN de l'enregistrement d'une séquence d'image\n");
	return ETHER;
}

/* EnregistreSequenceInter  -  codel INTER of SaveSequence
   Returns:  END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
EnregistreSequenceInter(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	return EnregistreSequenceEnd(SaveParams,report);
}
   
/*------------------------------------------------------------------------
 * SetGraphicParams
 *
 * Description: 
 *
 * Reports:      OK
 */

/* SetGraphicParamsExec  -  codel EXEC of SetGraphicParams
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetGraphicParamsExec(GEST_GRAPHIC_PARAMS *GraphicParams, int *report)
{
	/* Choix de l'affichage : on lance/arrête l'interface graphique avec les bonnes options */
	switch (GraphicParams->display) {
		case withoutInterface:
			traqueur->avec_interface(false);
			traqueur->affiche_image_sup(false);
			traqueur->affiche_particules(false);
			break;
		case withSimpleInterface:
			traqueur->avec_interface(true);
			traqueur->affiche_image_sup(false);
			traqueur->affiche_particules(false);
			break;
		case SkinProba:
			traqueur->avec_interface(true);
			traqueur->affiche_image_sup(true,(bool)GraphicParams->leftBased);
			traqueur->affiche_proba_peau(true);
			traqueur->affiche_particules((bool)GraphicParams->displayParticles);
			break;
		case Mvt:
			traqueur->avec_interface(true);
			traqueur->affiche_image_sup(true,(bool)GraphicParams->leftBased);
			traqueur->affiche_mvt(true);
			traqueur->affiche_particules((bool)GraphicParams->displayParticles);
			break;
		case Dist:
			traqueur->avec_interface(true);
			traqueur->affiche_image_sup(true,(bool)GraphicParams->leftBased);
			traqueur->affiche_dist(true);
			traqueur->affiche_particules((bool)GraphicParams->displayParticles);
			break;
	}
	
	/* pour savoir si l'image donnée en entrée est débayérisée ou non */
	#ifndef WITH_VIAM
	if (GraphicParams->Debayerisee==(GEST_BOOLEEN)true) {//si l'image a déjà été débayerisée
		if (!debayerisee) {//et si ce n'était pas le cas précédemment
			if (brute!=NULL) cvReleaseImage(&brute);
			free(bufferedCameraImageg);
			free(bufferedCameraImaged);
			tailleImg = imageWidth*imageHeight*4;
			brute = NULL;
			bufferedCameraImageg = (unsigned char *)calloc(tailleImg, sizeof(char));
			bufferedCameraImaged = (unsigned char *)calloc(tailleImg, sizeof(char));
			debayerisee = true;
		}
	} else {//si l'image n'a pas encore été débayerisée
		if (debayerisee) {//et si ce n'était pas le cas précédemment
			if (brute!=NULL) cvReleaseImage(&brute);
			free(bufferedCameraImageg);
			free(bufferedCameraImaged);
			tailleImg = imageWidth*imageHeight;
			brute = cvCreateImage(cvSize(imageWidth,imageHeight),8,1);
			bufferedCameraImageg = (unsigned char *)calloc(tailleImg, sizeof(char));
			bufferedCameraImaged = (unsigned char *)calloc(tailleImg, sizeof(char));
			debayerisee = false;
		}
	}
	#else
	debayerisee = (bool)GraphicParams->Debayerisee;
	#endif
	
	printf ("GEST : SetGraphicParamsExec : OK\n");
	
	return ETHER;
}

/*------------------------------------------------------------------------
 * MonitorMotionForward
 *
 * Description: 
 *
 * Reports:      OK
 */

/* MvtAvanceExec  -  codel EXEC of MonitorMotionForward
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
MvtAvanceExec(int *report)
{
	if (compteur>1) {//on ne peut pas comparer avant
		dist = pfStatecour[2]-pfStateprec[2];
		if (dist < -10) return ETHER;
	}
	printf ("GEST : MvtAvanceExec : OK\n");
	
	return EXEC;
}

/*------------------------------------------------------------------------
 * MonitorMotionBackward
 *
 * Description: 
 *
 * Reports:      OK
 */

/* MvtReculeExec  -  codel EXEC of MonitorMotionBackward
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
MvtReculeExec(int *report)
{
	if (compteur>1) {//on ne peut pas comparer avant
		dist = pfStatecour[2]-pfStateprec[2];
		if (dist > 10) return ETHER;
	}
	printf ("GEST : MvtReculeExec : OK\n");
	
	return EXEC;
}

/*------------------------------------------------------------------------
 * MonitorAtDist
 *
 * Description: 
 *
 * Reports:      OK
 */

/* ADistExec  -  codel EXEC of MonitorAtDist
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ADistExec(GEST_DIST *compareDist, int *report)
{
	
	if (compteur>1) {//on ne peut pas comparer avant
		dist = pfStatecour[2]-pfStateprec[2];
		if (dist < compareDist->distmin) return ETHER;
	}
	printf ("GEST : ADistExec : OK\n");
	
	return EXEC;
}


/*------------------------------------------------------------------------
 * ChangeVarianceMatrix
 *
 * Description: 
 *
 * Reports:      OK
 */

/* ChangeVarExec  -  codel EXEC of ChangeVarianceMatrix
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
ChangeVarExec(GEST_VAR *var, int *report)
{
	
	traqueur->changevar ((TYPE_PARTICULE_GEST)var->type, (int*)var);
	printf ("GEST : ChangeVarExec : OK\n");
	
	return ETHER;
}


/*------------------------------------------------------------------------
 * SaveTracking
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_BAD_PATH
 */

/* SaveTrackingExec  -  codel EXEC of SaveTracking
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SaveTrackingExec(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	if (!traqueur->sauvegarde((bool)SaveParams->saveImages,(bool)SaveParams->saveResults,SaveParams->savePath)) {
		printf ("GEST : SaveTrackingExec : ERREUR\n");
		*report = S_gest_BAD_PATH;
		return ETHER;
	}
	printf ("GEST : SaveTrackingExec : OK\n");
		
	return ETHER;
}


/*------------------------------------------------------------------------
 * StopSaveTracking
 *
 * Description: 
 *
 * Reports:      OK
 */

/* StopSaveTrackingExec  -  codel EXEC of StopSaveTracking
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
StopSaveTrackingExec(int *report)
{
	traqueur->sauvegarde(false,false,NULL);
	printf ("GEST : arrêt de la sauvegarde des résultats du traqueur\n");
		
	return ETHER;
}


/*------------------------------------------------------------------------
 * SaveTrackingOneShot
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_BAD_PATH
 */

/* SaveTrackingOneShotExec  -  codel EXEC of SaveTrackingOneShot
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SaveTrackingOneShotExec(GEST_SAVE_PARAMS *SaveParams, int *report)
{
	if (ONESHOT) {//on est au tour suivant, l'image a donc été enregistrée et on s'arrête là
		traqueur->sauvegarde(false,false,NULL);//on arrête l'enregistrement
		ONESHOT = false;
		return ETHER;
	}//on est au tour suivant, l'image a donc été enregistrée et on s'arrête là
	
	if (!traqueur->sauvegarde(true,false,SaveParams->savePath)) {//on veut forcément enregistrer des images, mais enregistrer le résultat n'a pas de sens ici
		printf ("GEST : SaveTrackingExec : ERREUR\n");
		*report = S_gest_BAD_PATH;
		return ETHER;
	}
	printf ("GEST : SaveTrackingOneShotExec : OK\n");
	ONESHOT = true;	
	return EXEC;//on passe une boucle pour enregistrer une image
}


/*------------------------------------------------------------------------
 * SetTrackingParams
 *
 * Description: 
 *
 * Reports:      OK
 *              S_gest_BAD_COLOR_SPACE
 */

/* SetTrackingParamsExec  -  codel EXEC of SetTrackingParams
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
SetTrackingParamsExec(GEST_TRACKING_PARAMS *TrackingParams, int *report)
{
	//...faudra encore demander le chemin et les numéros de la séquence
	ViaSequence=(bool)TrackingParams->ViaSequence;
	
	ChangeEspaceCoul(TrackingParams->ColorSpace, report);
	
	return ETHER;
}


/*------------------------------------------------------------------------
 * Pause
 *
 * Description: 
 *
 * Reports:      OK
 * 
 */

/* PauseExec  -  codel EXEC of Pause
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
PauseExec(int *report)
{
	PAUSE = true;
	printf ("\n--------------\nLe module gest est mis en PAUSE\n--------------\n");
	
	return ETHER;
}


/*------------------------------------------------------------------------
 * WindowPlacement
 *
 * Description: 
 *
 * Reports:      OK
 * 
 */

/* FenetreExec  -  codel EXEC of WindowPlacement
   Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT
FenetreExec(GEST_WINDOW *WindowParams, int *report)
{
	traqueur->position_interface(WindowParams->w, WindowParams->h);
	
	return ETHER;
}

/*------------------------------------------------------------------------
 * PlatineControl 
 *
 * Description:  speed control of Pan and Tilt
 *
 * Reports:      OK
 *
 */

/* FenetreExec  -  codel EXEC of PlatineControl, Returns:  EXEC END ETHER FAIL ZOMBIE */
ACTIVITY_EVENT  PlatineExec(GEST_BOOLEEN *WithPlatineControl, int *report)
{
	if (*WithPlatineControl) {
		if (ASSERV) {
			printf ("GEST : PlatineExec : with platine (rien à faire)\n");
			return ETHER;
		}
		printf ("GEST : PlatineExec : with platine \n");
		ASSERV=true;
	} else {
		if (!ASSERV) {
			printf ("GEST : PlatineExec : without platine (rien à faire)\n");
			return ETHER;
		}
		printf ("GEST : PlatineExec : without platine \n");
		ASSERV=false;
	}
	
	return ETHER;
}







